// ============================================================
// File Name			:	Client.cs
// Description			:	Client code for ZAPT
// ============================================================
// Table of Contents
// 
// 1. clientCmds
// 2. Minigame Panels
// 3. HUD
// 4. Package
// 5. Infotips
// 6. Audio
// ============================================================

// Number of HUD icons you can have at max.
// Will be reduced if your resolution is too small.
// -1 to disable hud, 0 to render only yourself.
// 15 is the hard-coded server-side maximum.
$ZAPT_Client::CellCount		= 15;
// Set to "false" if you do not like the Director's noises.
$ZAPT_Client::DirectorNoise	= true;
// Don't screw with this or the mod will give you errors.
$ZAPT_Client::Version		= 2.1;

$ZAPT_Client::FilePath = "Add-Ons/GameMode_ZAPT/";
$ZAPT_Client::FilePath::GUI				= $ZAPT_Client::FilePath @ "gui/";
$ZAPT_Client::FilePath::GUI::Help		= $ZAPT_Client::FilePath::GUI @ "help/";
$ZAPT_Client::FilePath::GUI::Images		= $ZAPT_Client::FilePath::GUI @ "images/";
$ZAPT_Client::FilePath::GUI::Vectors	= $ZAPT_Client::FilePath::GUI @ "vectors/";
$ZAPT_Client::FilePath::Sounds			= $ZAPT_Client::FilePath @ "sounds/";

// ============================================================
// Section 1 : clientCmds
// ============================================================
function clientCmdZAPT_DIRECTOR_SOUND_PLAY(%sound, %vector3F)
{
	if($ZAPT_Client::DirectorNoise)
	{
		%sound = getTaggedString(%sound);
		
		// Latter help prevents Kalphiter from doing retarded gay shit to your speakers.
		if(isObject(%sound) && %sound.directorSound)
		{
			if(%vector3F !$= "")
			{
				%x = getWord(%vector3F, 0);
				%y = getWord(%vector3F, 1);
				%z = getWord(%vector3F, 2);
				
				alxPlay(%sound, %x, %y, %z);
			}
			else
			{
				alxPlay(%sound);
			}
		}
	}
}

function clientCmdZAPT_DIRECTOR_SOUND_STOP(%sound)
{
	alxStop(%sound);
}

function clientCmdZAPT_HUD_Destroy()
{
	ZAPTC_destroyHud();
}

function clientCmdZAPT_HUD_Set(%id, %visible, %name, %health, %canvas, %status, %icon0, %icon1, %icon2, %icon3, %icon4, %icon5)
{
	if(!isObject(ZAPT_Players))
	{
		ZAPTC_createHud();
	}
	
	if(isObject(%cell = ZAPT_Players.player[%id]))
	{
		if(%visible)
		{
			%cell.setVisible(true);
			
			ZAPT_Players.setLabel(%id, %name);
			ZAPT_Players.setHealth(%id, %health);
			ZAPT_Players.setCanvas(%id, (%canvas !$= "" ? getTaggedString(%canvas) : ""));
			ZAPT_Players.setStatus(%id, (%canvas !$= "" ? getTaggedString(%status) : ""));
			
			for(%a = 0; %a < 6; %a++)
			{
				ZAPT_Players.setIcon(%id, %a, (%canvas !$= "" ? getTaggedString(%icon[%a]) : ""));
			}
		}
		else
		{
			%cell.setVisible(false);
		}
	}
}

function clientCmdZAPT_MOD_Resgister(%version, %mutations)
{
	echo("\c5ZAPT has been registered with this server.");
	
	if(%version > $ZAPT_Client::Version)
	{
		echo("\c2 +- Warning: You are running an outdated version of ZAPT.");
	}
	else if(%version < $ZAPT_Client::Version)
	{
		echo("\c2 +- Warning: The server is running an outdated version of ZAPT.");
	}
	
	%x = getWord(getRes(), 0);
	%x -= 275;
	
	%i = mFloor(%x / 205);
	
	commandToServer('ZAPT_Register', $ZAPT_Client::Version);
	commandToServer('ZAPT_CellCount', %i);
	
	ZAPTC_addMinigamePanels();
	ZAPTC_addMutations(%mutations);
}

// ============================================================
// Section 2 : Minigame Panels
// ============================================================
function ZAPTC_addMinigamePanels()
{
	ZAPTC_removeMinigamePanels();
	
	$ZAPT_Client::Panels = true;
	
	for(%a = 1; isFile($ZAPT_Client::FilePath::GUI @ "panel_" @ %a @ ".gui"); %a++)
	{
		exec($ZAPT_Client::FilePath::GUI @ "panel_" @ %a @ ".gui");
		
		// The default name of these objects is ZAPT_NewPanel.
		if(!isObject(ZAPT_NewPanel))
		{
			error("ZAPT_addMinigamePanels() : Panel #" @ %a @ " was made by a fucking idiot.");
		}
		else
		{
			CMG_ScrollBox.add(ZAPT_NewPanel);
			ZAPT_NewPanel.setName("ZAPT_MinigamePanel");
		}
	}
	
	ZAPTC_setDefaultZAPTSettings();
	
	return %a;
}

function ZAPTC_addMutations(%mutations)
{
	if(isObject(ZAPT_ModeList))
	{
		ZAPT_ModeList.clear();
		
		for(%a = 0; %a < getFieldCount(%mutations); %a++)
		{
			ZAPT_ModeList.add(getField(%mutations, %a), %a);
		}
		
		ZAPT_ModeList.setSelected(0);
	}
}

function ZAPTC_removeMinigamePanels()
{
	$ZAPT_Client::Panels = false;
	
	// Since all three panels are named the same thing, this loop will delete them all one-by-one.
	while(isObject(ZAPT_MinigamePanel))
	{
		ZAPT_MinigamePanel.delete();
	}
	
	// This is just a preventitive measure.
	while(isObject(ZAPT_NewPanel))
	{
		ZAPT_NewPanel.delete();
	}
}

function ZAPTC_setDefaultZAPTSettings()
{
	if(isObject(ZAPT_ModeList))
	{
		ZAPT_ModeList.clear();
		ZAPT_ModeList.add("No Data", 0);
		ZAPT_ModeList.setSelected(0);
	}
	
	$Minigame::TDMGameModeName = "ZAPT GameMode";
	$Minigame::TDMHelpText = "Kill Zombies.";
	$MiniGame::ZAPT = "0";
	$MiniGame::ZAPT::FF = "1";
	$MiniGame::ZAPT::GameMode = "0";
	$MiniGame::ZAPT::Outbreak = "1";
	$MiniGame::ZAPT::Mobs = "1";
	$MiniGame::ZAPT::PlayerZombies = "1";
	$MiniGame::ZAPT::Points::EatingPlayer = "3";
	$MiniGame::ZAPT::Points::GettingEaten = "0";
	$MiniGame::ZAPT::Points::KillBot = "1";
	$MiniGame::ZAPT::Points::KillPlayer = "2";
	$MiniGame::ZAPT::Points::KillTank = "10";
	$MiniGame::ZAPT::Points::KillTeam = "-5";
	$MiniGame::ZAPT::Tank = "1";
	$MiniGame::ZAPT::Rescue = "1";
	$MiniGame::ZAPT::Suicide = "0";
	$MiniGame::ZAPT::Time::Outbreak = "30";
	$MiniGame::ZAPT::Time::Rescue = "180";
	$MiniGame::ZAPT::Time::Reset = "10";
	$MiniGame::ZAPT::Timer = "1";
}

function CreateMiniGameGUI::sendZAPT(%gui)
{
	%on = $MiniGame::ZAPT;
	
	for(%a = 0; %a < 6; %a++)
	{
		switch(%a)
		{
			case 0: %name = "EatingPlayer";
			case 1: %name = "GettingEaten";
			case 2: %name = "KillBot";
			case 3: %name = "KillPlayer";
			case 4: %name = "KillTank";
			case 5: %name = "KillTeam";
		}
		
		if(!%a)
		{
			eval("%points = \"" @ %name @ "\" SPC $MiniGame::ZAPT::Points::" @ %name @ ";");
		}
		else
		{
			eval("%points = %points TAB \"" @ %name @ "\" SPC $MiniGame::ZAPT::Points::" @ %name @ ";");
		}
	}
	
	for(%a = 0; %a < 3; %a++)
	{
		switch(%a)
		{
			case 0: %name = "Outbreak";
			case 1: %name = "Rescue";
			case 2: %name = "Reset";
		}
		
		if(!%a)
		{
			eval("%timers = \"" @ %name @ "\" SPC $MiniGame::ZAPT::Time::" @ %name @ ";");
		}
		else
		{
			eval("%timers = %timers TAB \"" @ %name @ "\" SPC $MiniGame::ZAPT::Time::" @ %name @ ";");
		}
	}
	
	for(%a = 0; %a < 9; %a++)
	{
		switch(%a)
		{
			case 0: %name = "Outbreak";
			case 1: %name = "Rescue";
			case 2: %name = "Tank";
			case 3: %name = "FF";
			case 4: %name = "Timer";
			case 5: %name = "Mobs";
			case 6: %name = "Suicide";
			case 7: %name = "GameMode"; $MiniGame::ZAPT::GameMode = ZAPT_ModeList.getSelected();
			case 8: %name = "PlayerZombies";
		}
		
		if(!%a)
		{
			eval("%options = \"" @ %name @ "\" SPC $MiniGame::ZAPT::" @ %name @ ";");
		}
		else
		{
			eval("%options = %options TAB \"" @ %name @ "\" SPC $MiniGame::ZAPT::" @ %name @ ";");
		}
	}
	
	commandToServer('ZAPT_MinigameSettings', %on, %options, %points, %timers);
}

// ============================================================
// Section 3 : HUD
// ============================================================
function ZAPTC_createHUD()
{
	ZAPTC_destroyHud();
	
	%resX = getWord(getRes(), 0);
	%resY = getWord(getRes(), 1);
	
	%pos = 0 SPC (%resY - 100);
	%ext = %resX SPC "90";
	
	new GuiSwatchCtrl(ZAPT_Players)
	{
		profile		= "GuiDefaultProfile";
		horizSizing	= "right";
		vertSizing	= "bottom";
		color		= "0 0 0 0";
		minExtent	= "1 1";
		
		extent		= %ext;
		position	= %pos;
		visible		= true;
	};
	
	ZAPT_Players.blockSize = 0.75;
	ZAPT_Players.blockSizeYou = 1.00;
	
	ZAPT_Players.create();
}

function ZAPTC_destroyHud()
{
	if(isObject(ZAPT_Players))
	{
		ZAPT_Players.delete();
	}
}

function ZAPT_Players::create(%gui)
{
	while(%gui.getCount())
	{
		%gui.getObject(0).delete();
	}
	
	%x = getWord(getRes(), 0);
	%x -= 275;
	
	%i = mFloor(%x / 205);
	
	if($ZAPT_Client::CellCount < %i)
	{
		%i = $ZAPT_Client::CellCount;
	}
	
	%cont_pos = getWord(getRes(), 0) - 9;
	
	commandToServer('ZAPT_CellCount', %i);
	
	for(%a = 0; %a <= %i; %a++)
	{
		if(isObject(%gui.player[%a]))
		{
			continue;
		}
		
		if(%a > 0)
		{
			%self = false;
			%cont_ext = mFloor(266 * %gui.blockSize);
		}
		else
		{
			%self = true;
			%cont_ext = mFloor(266 * %gui.blockSizeYou);
		}
		
		// Container
		%gui.player[%a] = new GuiSwatchCtrl()
		{
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			color = "0 0 0 0";
			minExtent = "1 1";
			
			extent = %cont_ext SPC "90";
			position = (%cont_pos - %cont_ext) SPC "0";
			visible = false;
		};
		
		%cont_pos -= %cont_ext + 5;
		
		%gui.player[%a].isPlayer = %self;
		%gui.add(%gui.player[%a]);
		
		// Status Button
		%gui.player[%a].status = new GuiBitmapCtrl()
		{
			profile = "HUDBitmapProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			alignLeft = false;
			keepCached = false;
			lockAspectRatio = false;
			minExtent = "1 1";
			overflowImage = false;
			wrap = false;
			
			bitmap = $ZAPT_Client::FilePath::GUI::Images @ "status.dead";
			extent = "40 40";
			position = (%self ? "15 15" : "6 15");
			visible = true;
		};
		
		%gui.player[%a].add(%gui.player[%a].status);
		
		// Inventory Icons
		%start = (%self ? 238 : 179);
		%buffer = 0;
		
		for(%b = 0; %b < 6; %b++)
		{
			%gui.player[%a].icon[%b] = new GuiBitmapCtrl()
			{
				profile = "HUDBitmapProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				alignLeft = false;
				keepCached = false;
				lockAspectRatio = false;
				minExtent = "1 1";
				overflowImage = false;
				wrap = false;
				
				bitmap = $ZAPT_Client::FilePath::GUI::Images @ "icon.gun";
				position = (%start - %buffer) SPC "8";
				extent = "20 20";
				visible = true;
			};
			
			%gui.player[%a].add(%gui.player[%a].icon[%b]);
			%buffer += 25;
		}
		
		// Base
		%gui.player[%a].canvas = new GuiBitmapCtrl()
		{
			profile = "HUDBitmapProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			alignLeft = false;
			keepCached = false;
			lockAspectRatio = false;
			minExtent = "1 1";
			overflowImage = false;
			wrap = false;
			
			bitmap = $ZAPT_Client::FilePath::GUI::Images @ "canvas.human";
			extent = (%self ? "266 59" : "200 59");
			position = "0 31";
			visible = true;
		};
		
		%gui.player[%a].add(%gui.player[%a].canvas);
		
		// Label
		%gui.player[%a].label = new GuiMLTextCtrl()
		{
			profile = "GuiMLTextProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			allowColorChars = false;
			lineSpacing = "2";
			maxBitmapHeight = "-1";
			maxChars = "-1";
			minExtent = "1 1";
			selectable = false;
			
			extent = (%self ? "254 16" : "192 16");
			position = (%self ? "10 35" : "10 37");
			text = "<color:9b9a96><font:Verdana Bold:" @ (%self ? 16 : 13) @ ">Text";
			textFormat = "<color:9b9a96><font:Verdana Bold:" @ (%self ? 16 : 13) @ ">";
			visible = true;
		};
		
		%gui.player[%a].canvas.add(%gui.player[%a].label);
		
		// Health
		%gui.player[%a].healthLeft = new GuiBitmapCtrl()
		{
			profile = "HUDBitmapProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			alignLeft = false;
			keepCached = false;
			lockAspectRatio = false;
			minExtent = "1 1";
			overflowImage = false;
			wrap = false;
			
			bitmap = $ZAPT_Client::FilePath::GUI::Images @ "health.green.leftcorner";
			extent = "4 14";
			position = (%self ? "69 3" : "51 4");
			visible = true;
		};
		
		%gui.player[%a].healthMid = new GuiBitmapCtrl()
		{
			profile = "HUDBitmapProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			alignLeft = false;
			keepCached = false;
			lockAspectRatio = false;
			minExtent = "1 1";
			overflowImage = false;
			wrap = false;
			
			bitmap = $ZAPT_Client::FilePath::GUI::Images @ "health.green.1px";
			extent = (%self ? "186 14" : "139 14");
			extentNorm = (%self ? 186 : 139);
			position = (%self ? "73 3" : "55 4");
			visible = true;
		};
		
		%gui.player[%a].healthRight = new GuiBitmapCtrl()
		{
			profile = "HUDBitmapProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			alignLeft = false;
			keepCached = false;
			lockAspectRatio = false;
			minExtent = "1 1";
			overflowImage = false;
			wrap = false;
			
			bitmap = $ZAPT_Client::FilePath::GUI::Images @ "health.green.rightcorner";
			extent = "4 14";
			position = (%self ? "258 3" : "193 4");
			visible = true;
		};
		
		%gui.player[%a].canvas.add(%gui.player[%a].healthLeft);
		%gui.player[%a].canvas.add(%gui.player[%a].healthMid);
		%gui.player[%a].canvas.add(%gui.player[%a].healthRight);
	}
	
	if(!PlayGUI.isMember(%gui))
	{
		PlayGUI.add(%gui);
	}
	
	if(%i > -1)
	{
		commandToServer('ZAPT_GetData');
	}
}

function ZAPT_Players::resetPos(%gui)
{
	%resX = getWord(getRes(), 0);
	%resY = getWord(getRes(), 1);
	
	%pos = 0 SPC (%resY - 100);
	%ext = %resX SPC "90";
	
	%gui.position = %pos;
	%gui.extent = %ext;
	
	%gui.create();
}

function ZAPT_Players::setCanvas(%gui, %player, %canvas)
{
	if(isObject(%gui.player[%player]))
	{
		%filepath = filePath(%status);
		
		if(%filePath $= "")
		{
			%file = $ZAPT_Client::FilePath::GUI::Images @ "canvas." @ %canvas;
		}
		else
		{
			%file = %status;
		}
		
		%gui.player[%player].canvas.setBitmap(%file);
	}
	else
	{
		error("\"" @ %player @ "\" is not a valid player.");
	}
}

function ZAPT_Players::setHealth(%gui, %player, %health)
{
	if(isObject(%gui.player[%player]))
	{
		if(%health >= 0)
		{
			if(%health > 0)
			{
				%x = mFloor(%gui.player[%player].healthMid.extentNorm * (%health / 100));
				
				if(%x <= 0)
				{
					%x = 1;
				}
				else if(%x > %gui.player[%player].healthMid.extentNorm)
				{
					%x = %gui.player[%player].healthMid.extentNorm;
				}
				
				%ext = %x SPC getWord(%gui.player[%player].healthMid.extent, 1);
				%gui.player[%player].healthMid.extent = %ext;
				
				%pos = (getWord(%gui.player[%player].healthMid.getPosition(), 0) + %x - 1) SPC getWord(%gui.player[%player].healthRight.position, 1);
				%gui.player[%player].healthRight.position = %pos;
				
				if(%health > 100)
				{
					%color = "blue";
				}
				else if(%health >= 50)
				{
					%color = "green";
				}
				else if(%health >= 20)
				{
					%color = "yellow";
				}
				else
				{
					%color = "red";
				}
				
				%gui.player[%player].healthLeft.setBitmap($ZAPT_Client::FilePath::GUI::Images @ "health." @ %color @ ".leftcorner");
				%gui.player[%player].healthMid.setBitmap($ZAPT_Client::FilePath::GUI::Images @ "health." @ %color @ ".1px");
				%gui.player[%player].healthRight.setBitmap($ZAPT_Client::FilePath::GUI::Images @ "health." @ %color @ ".rightcorner");
				
				%gui.player[%player].healthLeft.setVisible(true);
				%gui.player[%player].healthMid.setVisible(true);
				%gui.player[%player].healthRight.setVisible(true);
			}
			else
			{
				%gui.player[%player].healthLeft.setVisible(false);
				%gui.player[%player].healthMid.setVisible(false);
				%gui.player[%player].healthRight.setVisible(false);
			}
		}
		else
		{
			error("\"" @ %health @ "\" is out of bounds.");
		}
	}
	else
	{
		error("\"" @ %player @ "\" is not a valid player.");
	}
}

function ZAPT_Players::setIcon(%gui, %player, %slot, %icon)
{
	if(isObject(%gui.player[%player]))
	{
		if(isObject(%gui.player[%player].icon[%slot]))
		{
			%filepath = filePath(%icon);
			
			if(%filePath $= "")
			{
				%file = $ZAPT_Client::FilePath::GUI::Images @ "icon." @ %icon;
			}
			else
			{
				%file = %icon;
			}
			
			if(!isFile(%file @ ".png") && !isFile(%file @ ".jpg"))
			{
				%file = "";
			}
			
			%gui.player[%player].icon[%slot].setBitmap(%file);
		}
		else
		{
			error("\"" @ %slot @ "\" is not a valid icon slot.");
		}
	}
	else
	{
		error("\"" @ %player @ "\" is not a valid player.");
	}
}

function ZAPT_Players::setStatus(%gui, %player, %status)
{
	if(isObject(%gui.player[%player]))
	{
		%filepath = filePath(%status);
		
		if(%filePath $= "")
		{
			%file = $ZAPT_Client::FilePath::GUI::Images @ "status." @ %status;
		}
		else
		{
			%file = %status;
		}
		
		if(!isFile(%file @ ".png") && !isFile(%file @ ".jpg"))
		{
			%file = "";
		}
		
		%gui.player[%player].status.setBitmap(%file);
	}
	else
	{
		error("\"" @ %player @ "\" is not a valid player.");
	}
}

function ZAPT_Players::setLabel(%gui, %player, %text)
{
	if(isObject(%gui.player[%player]))
	{
		%gui.player[%player].label.setText(%gui.player[%player].label.textFormat @ %text);
	}
	else
	{
		error("\"" @ %player @ "\" is not a valid player.");
	}
}

// ============================================================
// Section 4 : Package
// ============================================================
package ZAPT_Client
{
	function CMG_ScrollBox::zombieExtend(%this, %yExt)
	{
		%ext = %this.getExtent();
		%pos = %this.getPosition();
		
		%x = getWord(%pos, 0);
		%y = getWord(%pos, 1);
		%w = getWord(%ext, 0);
		%h = getWord(%ext, 1);
		
		%this.resize(%x, %y, %w, %h + %yExt);
	}
	
	function CreateMiniGameGUI::clickCreate(%gui)
	{
		%result = parent::clickCreate(%gui);
		
		if($ZAPT_Client::Panels)
		{
			%gui.sendZAPT();
		}
		
		return %result;
	}
	
	function CreateMiniGameGUI::clickFav(%gui, %fav)
	{
		// This is the only way to tell if we're setting or getting data.
		// Is hidden always by this function, so we have to prefetch.
		%setting = CMG_FavsHelper.isVisible();
		
		if($ZAPT_Client::Panels && isObject(ZAPT_ModeList) && %setting)
		{
			$MiniGame::ZAPT::GameMode = ZAPT_ModeList.getSelected();
		}
		
		%result = parent::clickFav(%gui, %fav);
		
		if($ZAPT_Client::Panels && isObject(ZAPT_ModeList) && !%setting)
		{
			ZAPT_ModeList.setSelected($MiniGame::ZAPT::GameMode);
		}
		
		return %result;
	}
	
	function CreateMiniGameGUI::onWake(%gui)
	{
		%result = parent::onWake(%gui);
		
		// Gamemode_zombie panel visibility
		if(isObject(ZombieMinigameGui))
		{
			ZombieMinigameGui.setVisible(!$ZAPT_Client::Panels);
		}
		
		// Panel Adjustments
		%y = 0;
		
		for(%a = 0; %a < CMG_ScrollBox.getCount(); %a++)
		{
			%panel = CMG_ScrollBox.getObject(%a);
			
			if(%panel.isVisible())
			{
				%panelExt = getWord(%panel.extent, 1);
				%y += (%panelExt + 2);
			}
		}
		
		%shift = %y - getWord(CMG_ScrollBox.getExtent(), 1);
		
		if(%shift != 0)
		{
			CMG_ScrollBox.zombieExtend(%shift);
		}
		
		%y = 0;
		
		for(%a = 0; %a < CMG_ScrollBox.getCount(); %a++)
		{
			%panel = CMG_ScrollBox.getObject(%a);
			%panel.position = "2" SPC %y;
			
			if(%panel.isVisible())
			{
				%panelExt = getWord(%panel.extent, 1);
				%y += (%panelExt + 2);
			}
		}
		
		return %result;
	}
	
	function disconnectedCleanup()
	{
		deleteVariables("$ZAPT::*");
		
		ZAPTC_destroyHud();
		ZAPTC_removeMinigamePanels();
		
		return parent::disconnectedCleanup();
	}
	
	function PlayGUI::hideBrickBox(%gui, %y, %steps, %step)
	{
		if(isObject(ZAPT_Players) && %step == 0)
		{
			if(%y < 0)
			{
				ZAPT_Players.setVisible(false);
			}
			else
			{
				ZAPT_Players.setVisible(true);
			}
		}
		
		return parent::hideBrickBox(%gui, %y, %steps, %step);
	}
	
	function resetCanvas()
	{
		%result = parent::resetCanvas();
		
		if(isObject(ZAPT_Players))
		{
			ZAPT_Players.resetPos();
		}
		
		return %result;
	}
};
activatePackage(ZAPT_Client);

// ============================================================
// Section 5 : Infotips
// ============================================================
if(isFile("Add-Ons/System_ReturnToBlockland/server.cs"))
{
	if(!$RTB::RTBR_InfoTips_Hook)
	{
		exec("Add-Ons/System_ReturnToBlockland/RTBR_InfoTips_Hook.cs");
	}
	
	// Zombie-Specific
	RTB_addInfoTip("As a Zombie, you are immune to fall damage.", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Zombie, use your <key:mouseFire> to munch survivors",
		"As a Zombie, bind your Fire button to eat people.", "ZAPT-Zombies");
	RTB_addInfoTip("As a Zombie, press <key:jet> to activate a special attack!",
		"As a Zombie, you could use a special attack if you had your Jet key bound.", "ZAPT-Zombies");
	RTB_addInfoTip("As a Zombie, you can climb walls by first jumping, then clicking the bricks.", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Zombie, if you are caught on fire you will always die because of it. Avoid fire!", "", "ZAPT-Zombies");
	
	// Zombie-Class-Specific
	RTB_addInfoTip("As a Consumer, eat humans to regain health, become stronger and tougher!", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Consumer, you can shoot projectile puke at survivors, using <key:jet>, to blind them and attract the horde?",
		"If you had your Jet key bound, you could puke on survivors as a Consumer", "ZAPT-Zombies");
		
	RTB_addInfoTip("As a Leaper, you do a lot of damage by knocking into players. The longer you jump, the more damage!", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Leaper, you can jump very far by crouching and then using your jet key.", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Leaper, you can turn around mid-air and kick off walls!", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Leaper, you do much more damage when on fire. This puts a timer on your life, though, so make it count.", "", "ZAPT-Zombies");
	
	RTB_addInfoTip("As a Jockey, you can mount other people's heads by jumping on them. Use your <key:jet> to give yourself a little boost!",
		"As a Jockey, you can mount other people's heads by jumping on them. Bind your Jet key to give yourself a little boost!", "ZAPT-Zombies");
	RTB_addInfoTip("As a Jockey, you prevent your victim from firing a weapon. Jump on isolated survivors to ensure a kill!", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Jockey, you can bounce your victims by pressing <key:jet>. This lets you dodge bullets.",
		"As a Jockey, you could bounce your victims if you had your Jet key  bound.", "ZAPT-Zombies");
		
	RTB_addInfoTip("As a Smasher, you can press <key:jet> to charge at your victims, dealing high damage and burrowing through bricks.",
		"As a Smasher, you could charge if you had your Jet key bound.", "ZAPT-Zombies");
	RTB_addInfoTip("As a Smasher, your melee damage is very high, but at the cost of a lot of stamina. Use it if there isn't enough space to charge!", "", "ZAPT-Zombies");
	
	RTB_addInfoTip("As a Soldier Zombie, you are impevious to most damage, except to the head.", "", "ZAPT-Zombies");
	
	RTB_addInfoTip("As a Tank, you have incredible amounts of strength. Do not hesitate in using it!", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Tank, your melee damage will destroy most vehicles in a short period of time.", "", "ZAPT-Zombies");
	RTB_addInfoTip("As a Tank, your melee attacks knock back survivors. Use this to fling them off of high places and into traps!", "", "ZAPT-Zombies");
	
	// Human-Specific
	RTB_addInfoTip("Shoot to kill. Zombie headshots (even against the Tank!) do extra damage.", "", "ZAPT-Humans");
	RTB_addInfoTip("Smokers can render a survivor helpless from a distance. Bump into buddies with orange names to give them a hand.", "", "ZAPT-Humans");
	RTB_addInfoTip("Jockeys can easily isolate and destroy your team! Shoot them off your buddies and they will help you, too.", "", "ZAPT-Humans");
	RTB_addInfoTip("When a Jockey is on you, you cannot shoot. Find a friend and stand still so he can kill it for you.", "", "ZAPT-Humans");
	RTB_addInfoTip("The Tank can easily decimate your entire team. Work together to whittle it down to zombie mincemeat.", "", "ZAPT-Humans");
	RTB_addInfoTip("The Tank can easily destroy your vehicles. Do not think you're safe because you're in a Tank yourself!", "", "ZAPT-Humans");
	RTB_addInfoTip("The Tank can fling survivors off of roof-tops. Be careful when messing with one in dangerous areas.", "", "ZAPT-Humans");
	RTB_addInfoTip("The Tank's \"Holy Water\" is fire. Light it up and it has a 45-second timer on its life!", "", "ZAPT-Humans");
	RTB_addInfoTip("When mixing a Molotov Cocktail, did you know that using gasoline/petrol instead of alcohol allows for a much more balanced incendiary weapon?", "", "ZAPT-Humans");
	
	// GameModes
	RTB_addInfoTip("In Versus Mode, half of the minigame's players are selected to be zombies at the start of the round.", "", "ZAPT");
	RTB_addInfoTip("The Director in ZAPT will periodically spawn mobs and increase zombie damage.", "", "ZAPT");
	RTB_addInfoTip("You can enable Friendly-Fire in a ZAPT minigame for added challenge and cooperation.", "", "ZAPT");
	RTB_addInfoTip("Disabling \"Rescue on Countdown\" turns the timer to a loss-timer instead of a win-timer.", "", "ZAPT");
	RTB_addInfoTip("ZAPT's timer can be disabled for an endless survival game.", "", "ZAPT");
	RTB_addInfoTip("Disabling player zombies in ZAPT will turn the game from L4D-style modes to a free-for-all bot slaughter.", "", "ZAPT");
	RTB_addInfoTip("The Outbreak is an option that will turn a random player into a zombie to help start the game.", "", "ZAPT");
	RTB_addInfoTip("To make a ZAPT Saferoom, create a door that disables minigame->safehouseVictory on open and enables it on closed.", "", "ZAPT");
	RTB_addInfoTip("Did you know that all bricks in a Safehouse Trigger are immune from destruction?", "", "ZAPT");
	RTB_addInfoTip("Did you know that you can change the Safehouse Trigger's size in your RTB preferences?", "", "ZAPT");
}