// ============================================================
// Project				:	ZAPT
// Author				:	Iban
// Description			:	Original Source Code
// ============================================================
// Table of Contents
// 
// 1. AIConnection
// 2. AIPlayer
// 3. Armor
// 4. fxDTSBrick
// 5. GameConnection
// 6. MinigameSO
// 7. Player
// 8. Trigger
// 9. serverCmds
// 10. Namespaceless
// 11. Objects
// 12. Events
// ============================================================

// ============================================================
// Section 1 : AIConnection
// ============================================================
// Whoops!

// ============================================================
// Section 2 : AIPlayer
// ============================================================
// Natural Functions

// Funct: Tries to find an action for the AI to do to reach %obj.target
// Sends: String representing action.
function AIPlayer::zombieDetectRoute(%obj)
{
	if(%obj.isZombie && %obj.zombieHasTarget())
	{
		%target	= %obj.target;
		
		// Standard Properties
		if(vectorDist(%obj.zombieLastThoughtAt, %obj.getPosition()) < (2 / (1000 / ZombieDirector.getThoughtTime())))
		{
			if(%obj.zombieStuckSince $= "")
			{
				%obj.zombieStuckSince = $sim::time;
				%stuck = false;
			}
			else if($sim::time - %obj.zombieStuckSince >= 0.5)
			{
				%stuck = true;
			}
		}
		else
		{
			%obj.zombieStuckSince = "";
			%stuck = false;
		}
		
		%box 	= %obj.getWorldBox();
		%canSee	= %obj.zombieSee(%target);
		%dist	= vectorDist(%target.getPosition(), %obj.getPosition());
		%fVec	= %obj.getForwardVector();
		%reach	= 3 * getWord(%obj.getScale(), 2);
		%zDist	= getWord(%obj.getPosition(), 2) - getWord(%target.getPosition(), 2);
		
		// Raycast Slush
		%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
		
		%hStart	= %obj.getEyePoint();
		%lStart = vectorAdd(%obj.getPosition(), vectorScale("0 0 0.3", getWord(%obj.getScale(), 2)));
		
		%hEnd	= vectorAdd(%hStart, vectorScale(%fVec, 2));
		%lEnd	= vectorAdd(%lStart, vectorScale(%fVec, 2));
		%fEnd	= vectorAdd(%hEnd, "0 0" SPC (getWord(%box, 5) - getWord(%box, 2)));
		
		%hCol	= getWord(containerRayCast(%hStart, %hEnd, %masks, %obj, %obj.target), 0);
		%lCol	= getWord(containerRayCast(%lStart, %lEnd, %masks, %obj, %obj.target), 0);
		%fCol	= getWord(containerRayCast(%hStart, %fEnd, %masks, %obj, %obj.target), 0);
		
		%hHit	= isObject(%hCol);
		%lHit	= isObject(%lCol);
		%fHit	= isObject(%fCol);
		
		%result = stripTrailingSpaces(%obj.getDatablock().zombieDetectRoute(%obj, %canSee, %dist, %reach, %zDist, %hCol, %lCol, %hHit, %lhit, %fhit));
		
		if(%result $= "")
		{
			if(!%obj.zombieInWater)
			{
				// If we can see and we can reach = attack.
				if(%canSee && %dist <= %reach)
				{
					%result = "attack";
				}
				// If we have no leg obstructions and are even with or above target = crawl.
				else if(!%lHit && (mAbs(%zDist) < 0.5 || %hHit) && %stuck)
				{
					%result = "crouch";
				}
				// If we have no head hit and stuck OR they're above us and we have no float hit...
				else if((!%hHit && %stuck) || (%zDist <= -1 && !%fHit))
				{
					// If no float hit and we can see them...
					if(!%fHit && %canSee)
					{
						%result = "jump";
					}
					else
					{
						%result = "crouchjump";
					}
				}
				// Cant see = bring it around town and/or smash.
				else if(!%canSee)
				{
					if(%stuck)
					{
						%oPos = %obj.getEyePoint();
						%tPos = %target.getEyePoint();
						
						%left = getWord(%fVec, 0);
						%right = getWord(%fVec, 1);
						
						// Left
						if(%left + 1 > 1) %leftNew = %left - 1;
						else %leftNew = %left + 1;
						
						if(%right - 1 < -1) %rightNew = %right + 1;
						else %rightNew = %right - 1;
						
						%leftVec = %leftNew SPC %rightNew SPC "0";
						
						// Right
						if(%left - 1 < -1) %leftNew = %left + 1;
						else %leftNew = %left - 1;
						
						if(%right + 1 > 1) %rightNew = %right - 1;
						else %rightNew = %right + 1;
						
						%rightVec = %leftNew SPC %rightNew SPC "0"; 
						
						%leftPos = vectorAdd(vectorScale(%leftVec, 8), %oPos);
						%rightPos = vectorAdd(vectorScale(%rightVec, 8), %oPos);
						
						%leftDist = vectorDist(%leftPos, %tPos);
						%rightDist = vectorDist(%rightPos, %tPos);
						
						// Statement potentially saves processing power by going for the better move first.
						if(%leftDist < %rightDist)
						{
							if(%finalPos = %obj.zombieSeeFromPoint(%leftPos, %target))
							{
								%result = "moveto" TAB %finalPos;
							}
							else if(%finalPos = %obj.zombieSeeFromPoint(%rightPos, %target))
							{
								%result = "moveto" TAB %finalPos;
							}
						}
						else
						{
							if(%finalPos = %obj.zombieSeeFromPoint(%rightPos, %target))
							{
								%result = "moveto" TAB %finalPos;
							}
							else if(%finalPos = %obj.zombieSeeFromPoint(%leftPos, %target))
							{
								%result = "moveto" TAB %finalPos;
							}
						}
					}
					
					// lol
					if(%result $= "")
					{
						%result = "smash";
					}
				}
			}
			else
			{
				// If we can see and we can reach = attack.
				if(%canSee && %dist <= %reach)
				{
					%result = "attack";
				}
				// If they are below us = swim down
				else if(%zDist >= 1)
				{
					%result = "crouch";
				}
				// If they are above us and out of water, "push up" - an AI hack.
				else if(%zDist <= -0.25 && !%target.zombieInWater)
				{
					%obj.addVelocity("0 0 3");
					%result = "jump";
				}
				// Cant see = smash
				else if(!%canSee)
				{
					%result = "smash";
				}
			}
		}
		
		return %result;
	}
}

// Funct: Determines if %obj has a valid target.
// Sends: bool
function AIPlayer::zombieHasTarget(%obj)
{
	if(%obj.isZombie)
	{
		if(zombieValidTarget(%obj, %obj.target))
		{
			return true;
		}
	}
	
	return false;
}

// Funct: Determines if %obj can raycast to %point, then from %point to %target
// Sends: bool
function AIPlayer::zombieSeeFromPoint(%obj, %point, %target)
{
	if(%obj.isZombie)
	{
		if(isObject(%target) && !%target.isCloaked)
		{
			%start = %obj.getEyePoint();
			%end = %point;
			
			if(vectorDist(%start, %end) <= 100)
			{
				%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
				%ray = containerRayCast(%start, %end, %masks, %obj);
				%hit = getWord(%ray, 0);
				
				// We hit something.
				// We need to get a point far enough away our guy can actually get to it.
				if(isObject(%hit))
				{
					%startRay = posFromRaycast(%ray);
					%startDif = vectorSub(%startRay, %start);
					%startBuf = vectorScale(%startSum, 1 / vectorDist(%startRay, %start));
					
					%start = vectorSub(%startRay, %startSlice);
				}
				// We did not hit something.
				// Our secondary is the point we got in the first place if it passes.
				else
				{
					%start = %point;
				}
				
				%end = %target.getEyePoint();
				
				if(vectorDist(%start, %end) <= 100)
				{
					%ray = containerRayCast(%start, %end, %masks, %obj);
					%col = getWord(%ray, 0);
					
					// We hit out target after getting a side-step point.
					// This means our secondary point is valid for attacking the victim.
					// Send the secondary point for AI Objectivity.
					if(isObject(%col) && %col.getID() == %target.getID())
					{
						return %start;
					}
				}
			}
		}
	}
	
	return false;
}

// Funct: Root function harboring AI.
// Sends: void
function AIPlayer::zombieThink(%obj)
{
	// Function is fully commented.
	
	// Qualifier checks.
	if(%obj.isZombie && %obj.getType() & $TypeMasks::PlayerObjectType)
	{
		%mini = getMiniGameFromObject(%obj);
		
		if(zombieGameSetUpRight(%mini))
		{
			// Goose Chase is the property set up for zombies doing "elbow bends".
			// They are going to an arbitrary point, not an object.
			if(!%obj.zombieGooseChase)
			{
				%armor = %obj.getDatablock();
				
				// Mutation allows bots OR bot is AI only OR is a mobster
				if(%mini.getProperty("bots") || %armor.zombieAIOnly || %obj.zombieMob)
				{
					// We cant think clearly if we're on fire (unless otherwise noted).
					if(!%armor.zombieHaetFire || !%obj.zombieOnFire())
					{
						// Pathfinding
						if(%obj.zombieHasTarget())
						{
							// If our target is a distraction we cannot shake it.
							if(!%obj.target.zombieDistract)
							{
								// Even if we can see our target, we bee-line for distractions always.
								%distraction = ZombieDirector.getDistraction(%obj);
								
								if(!isObject(%distraction))
								{
									// If we cant see our target, we're going to try and ditch him.
									if(!%obj.zombieSee(%obj.target))
									{
										if($sim::time - %obj.zombieNotSeenTargetSince > 15 && (isObject(%obj.mobSpawn) || !isObject(%obj.spawnBrick)))
										{
											%obj.delete();
										}
										else
										{
											// Find a new target.
											%target = %armor.zombieGetTarget(%obj);
											
											// If we have a new target...
											if(isObject(%target) && %target.getID() != %obj.target.getID())
											{
												// Chase him.
												%obj.zombieChase(%target);
											}
											// If we don't and it's been too long...
											else if($sim::time - %obj.zombieNotSeenTargetSince > 2)
											{
												// Just dick around.
												%obj.zombieChase(-1);
											}
										}
									}
									// Keep track of how long it's been.
									else
									{
										%obj.zombieNotSeenTargetSince = $sim::time;
									}
								}
								else
								{
									%obj.zombieChase(%distraction);
								}
							}
							
							// If we still have a target (we can lose it in the above bracket set), find him!
							if(%obj.zombieHasTarget())
							{
								// Uses another function to find a "plan of action"
								%instructions = %obj.zombieDetectRoute();
								%route = getField(%instructions, 0);
								%route2 = getField(%instructions, 1);
								
								// It is an option to have zombieSpecial triggered on a certain route.
								// If our datablock requires it, then we have to try to do a special.
								%special = false;
								
								if(%armor.useSpecialOn[%route])
								{
									%special = %obj.zombieSpecial();
								}
								
								// If it worked, carry on.
								if(!%special)
								{
									switch$(%route)
									{
										// This is called usually when the player is in reach but not touching.
										// For instance, if they're smoked between two objects and we're behind them.
										// It makes them much more effecient killers.
										case "attack":
											%obj.activateStuff();
										
										case "crouch":
											%obj.setImageTrigger(3, true);
											%obj.schedule(250, "setImageTrigger", 3, false);
										
										case "jump":
											%obj.setImageTrigger(2, true);
											%obj.schedule(250, "setImageTrigger", 2, false);
										
										case "crouchjump":
											%obj.setImageTrigger(2, true);
											%obj.schedule(250, "setImageTrigger", 2, false);
											%obj.setImageTrigger(3, true);
											%obj.schedule(250, "setImageTrigger", 3, false);
										
										// This is the "Goose Chase" scenario.
										// It happens when we can see the target from a point to our right our left
										// but not where we are standing.
										case "moveto":
											%obj.zombieGooseChase = true;
											%obj.setAimObject(-1);
											%obj.setAimLocation(%route2);
											%obj.setMoveObject(-1);
											%obj.setMoveDestination(%route2);
											%obj.schedule(1500, "zombieChase", %obj.target);
										
										// Usually a last-ditch effort to clear out a brick wall or something.
										case "smash":
											%obj.zombieSmash();
										
										// Typically called by armor-specific functions as this can be anything.
										case "special":
											%obj.zombieSpecial();
									}
								}
							}
						}
						// Target Location
						else
						{
							%distraction = ZombieDirector.getDistraction(%obj);
							
							if(!isObject(%distraction))
							{
								%target = %armor.zombieGetTarget(%obj);
								
								// We found us some fresh meat.
								if(isObject(%target))
								{
									%obj.zombieChase(%target);
								}
								// Are we too far away from anybody to be of any use?
								else
								{
									if($sim::time - %obj.zombieSpawnTime > 1 && %mini.getNumberOfHumans())
									{
										%ourPos = %obj.getPosition();
										
										if(!ZombieDirector.humanCanSee(%mini, %ourPos) && !ZombieDirector.isPosNearPlayer(%mini, %ourPos, $ZAPT::MaxDist, 0))
										{
											%obj.delete();
										}
									}
									
									if(isObject(%obj))
									{
										// Meander until there's something fun to do.
										%vel = %obj.getVelocity();
										%vel = mAbs(getWord(%vel, 0)) + mAbs(getWord(%vel, 1)) + mAbs(getWord(%vel, 2));
										
										if(%vel <= 0.25)
										{
											%obj.zombieWander();
										}
									}
								}
							}
							else
							{
								%obj.zombieChase(%distraction);
							}
						}
					}
					else
					{
						if(%obj.zombieHasTarget())
						{
							%obj.zombieChase(-1);
						}
						
						%obj.zombieWander();
						
						%obj.zombieCanActivate = true;
						%obj.playThread(0, "activate2");
						%obj.zombieCanActivate = false;
					}
				}
				else
				{
					%obj.delete();
				}
			}
			
			// %obj can be deleted.
			if(isObject(%obj))
			{
				// Used by the route-detection function for the parameter %stuck.
				%obj.zombieLastThoughtAt = %obj.getPosition();
			}
		}
		// If we're not in a valid minigame but have thought, we're probably displaced.
		else
		{
			%obj.delete();
		}
	}
}

// "Dummy" Functions
// See Armor:: function of same name.
function AIPlayer::zombieChase(%obj, %target)
{
	return %obj.getDatablock().zombieChase(%obj, %target);
}

function AIPlayer::zombieWander(%obj)
{
	return %obj.getDatablock().zombieWander(%obj);
}

// ============================================================
// Section 3 : Armor
// ============================================================
// ALL OF THESE can be totally manipulated for a new zombie without any hacking.
// All you have to do is declare Zombie*Armor::<function>(params) and it will use yours first.
// Use "return parent::<function>(params)" to call back to these. (Does not need to be packaged!)

// Funct: Determines skincolor based off of datablock properties.
// Sends: RGBA
function Armor::getSkinColor(%this, %obj)
{
	%redMin = %this.skinColorRedMin * 100;
	%redMax = %this.skinColorRedMax * 100;
	%greenMin = %this.skinColorGreenMin * 100;
	%greenMax = %this.skinColorGreenMax * 100;
	%blueMin = %this.skinColorblueMin * 100;
	%blueMax = %this.skinColorblueMax * 100;
	
	%red = getRandom(%redMin, %redMax) / 100;
	%blue = getRandom(%blueMin, %blueMax) / 100;
	%green = getRandom(%greenMin, %greenMax) / 100;
	
	if(%this.skinColorSym !$= "")
	{
		switch$(%this.skinColorSym)
		{
			case "rg":
				%sym = (%red + %green) / 2;
				%red = %sym;
				%green = %sym;
			
			case "rb":
				%sym = (%red + %blue) / 2;
				%red = %sym;
				%blue = %sym;
			
			case "gb":
				%sym = (%green + %blue) / 2;
				%green = %sym;
				%blue = %sym;
			
			case "rgb":
				%sym = (%red + %green + %blue) / 3;
				%red = %sym;
				%green = %sym;
				%blue = %sym;
			
			default:
				error("\"" @ %this.skinColorSym @ "\" is not a valid symmetry value for armor " @ %this.getName());
		}
	}
	
	%rgba = %red SPC %green SPC %blue SPC "1";
	
	return %rgba;
}

// Funct: Sets the client to control the %obj.
// Sends: void
function Armor::zombieCameraFix(%this, %obj)
{
	if(isObject(%obj))
	{
		%client = %obj.client;
		
		if(isObject(%client))
		{
			%client.setControlObject(%obj);
		}
	}
}

// Funct: Sets the client camera to orbit the %obj.
// Sends: void
function Armor::zombieCameraSpectate(%this, %obj)
{
	if(isObject(%obj))
	{
		%client = %obj.client;
		
		if(isObject(%client))
		{
			%transform = %obj.getTransform();
			%client.camera.setOrbitMode(%obj, %transform, 0.5, 8, 8, 1);
			%client.camera.mode = "Orbit";
			%client.setControlObject(%client.camera);
		}
	}
}

// Funct: Tries to damage a victim or climb a wall.
// Sends: %col
function Armor::zombieClaw(%this, %obj)
{
	// Animation
	%obj.zombieCanActivate = true;
	%obj.playThread(0, "activate2");
	%obj.zombieCanActivate = false;
	
	// Start
	%start = %obj.getEyePoint();
	
	// End
	%aimVec = %obj.zombieAimVec();
	%range = 3 * getWord(%obj.getScale(), 2);
	%end = vectorAdd(%start, vectorScale(%aimVec, %range));
	
	// Masks
	%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
	
	%col = getWord(containerRayCast(%start, %end, %masks, %obj), 0);
	
	if(zombieSameGame(%obj, %col))
	{
		// Damage
		if(%col.getType() & $TypeMasks::PlayerObjectType)
		{
			%this.zombieConsume(%obj, %col);
		}
		// Climb
		else if(%col.getType() & $TypeMasks::FxBrickObjectType)
		{
			if(!isObject(%col.Shape))
			{
				%start = %obj.getPosition();
				%end = vectorSub(%start, "0 0 1");
				
				%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
				
				%floor = getWord(containerRayCast(%start, %end, %masks, %obj), 0);
				
				if(!isObject(%floor))
				{
					if(%obj.getEnergyLevel() > 25)
					{
						%zBrickPlane = getWord(%col.getWorldBox(), 5);
						%zObjPlane = getWord(%obj.getWorldBox(), 2);
						
						if(%zBrickPlane > %zObjPlane)
						{
							%vel = %obj.getVelocity();
							%velZ = getWord(%vel, 2);
							
							if(%velZ < 15)
							{
								%velZ += 7.5;
								
								if(%velZ > 15)
								{
									%velZ = 15;
								}
								
								%obj.setVelocity(getWords(%vel, 0, 1) SPC %velZ);
							}
						}
					}
				}
			}
			
			if(%col.numEvents)
			{
				%col.onActivate(%obj, %obj.client);
			}
		}
		// Damage Vehicle
		else if(%col.getType() & $TypeMasks::VehicleObjectType)
		{
		// Energy
			%energy = %obj.getEnergyLevel();
			
			if(%energy > 6.5 && $sim::time - %obj.zombieLastClaw > 0.1)
			{
				%obj.setEnergyLevel(%energy - 6.5);
				%obj.zombieLastClaw = $sim::time;
				
				if(zombieValidTarget(%obj, %col))
				{
					%pos = %obj.getPosition();
					%damage = %obj.getDatablock().zombieDamage;
					%damage = mDirMod(%damage * 0.60, %damage * 1.30, getMiniGameFromObject(%obj));
					%damage = mCeil(%damage * 0.75);
					
					if(%this.zombieDamType !$= "")
					{
						%damageType = %this.zombieDamType;
					}
					else
					{
						%damageType = $DamageType::Zombies;
					}
					
					%source = %obj;
					
					%col.damage(%source, %pos, %damage, %damageType);
					
					%dif = vectorSub(%col.getPosition(), %obj.getPosition());
					%col.setVelocity(vectorAdd(%col.getVelocity(), vectorScale(%dif, -1.5)));
					
					if(%obj.getDamageLevel())
					{
						if(%damage > %obj.getDamageLevel())
						{
							%heal = %obj.getDamageLevel();
						}
						else
						{
							%heal = %damage;
						}
						
						%obj.setDamageLevel(%obj.getDamageLevel() - getRandom(mFloor(%heal / 2), %heal));
					}
					
					if(!zombieBot(%obj))
					{
						commandToClient(%obj.client, 'centerPrint', "<font:impact:32>*POUND*", 0.5);
					}
				}
			}
		}
		
		return %col;
	}
	
	return 0;
}

// Funct: Damages %col and feeds %obj health.
// Sends: bool
function Armor::zombieConsume(%this, %obj, %col)
{
	if(zombieValidTarget(%obj, %col))
	{
		// Energy
		%energy = %obj.getEnergyLevel();
		
		if(%energy > 6.5 && $sim::time - %obj.zombieLastClaw > 0.1)
		{
			%obj.setEnergyLevel(%energy - 6.5);
			%obj.zombieLastClaw = $sim::time;
			
			%pos = %obj.getPosition();
			%damage = %obj.getDatablock().zombieDamage;
			%damage = mDirMod(%damage * 0.60, %damage * 1.30, getMiniGameFromObject(%obj));
			%damage *= (zombieBot(%obj) ? 0.75 : 1);
			%damage *= ($sim::time - %obj.zombieLastClawDamage >= 1 ? 1 : $sim::time - %obj.zombieLastClawDamage);
			%damage = mCeil(%damage);
			
			
			if(%col.zombieBeacon && zombieBot(%obj))
			{
				%damageType = $DamageType::ConsumerSwarm; 
				
				if(!zombieBot(%col.zombieBeaconClient.player))
				{
					%source = %col.zombieBeaconClient.player;
				}
				else
				{
					%source = %obj;
				}
			}
			else
			{
				if(%this.zombieDamType !$= "")
				{
					%damageType = %this.zombieDamType;
				}
				else
				{
					%damageType = $DamageType::Zombies;
				}
				
				%source = %obj;
			}
			
			%col.damage(%source, %pos, %damage, %damageType);
			%obj.zombieLastClawDamage = $sim::time;
			
			%dif = vectorSub(%col.getPosition(), %obj.getPosition());
			%col.setVelocity(vectorAdd(%col.getVelocity(), vectorScale(%dif, -1.5)));
			
			if(%obj.getDamageLevel())
			{
				if(%damage > %obj.getDamageLevel())
				{
					%heal = %obj.getDamageLevel();
				}
				else
				{
					%heal = %damage;
				}
				
				%obj.setDamageLevel(%obj.getDamageLevel() - getRandom(mFloor(%heal / 2), %heal));
			}
			
			if(!zombieBot(%obj))
			{
				commandToClient(%obj.client, 'centerPrint', "<font:impact:32>*MUNCH*\n<font:impact:16>Om nom nom, nom nom.", 0.5);
			}
			
			return true;
		}
	}
	
	return false;
}

// Funct: Rigs %obj to attack %target
// Sends: bool
function Armor::zombieChase(%this, %obj, %target)
{
	if(%obj.isZombie)
	{
		%mini = getMiniGameFromObject(%obj);
		
		if(zombieGameSetUpRight(%mini))
		{
			if(zombieValidTarget(%obj, %target))
			{
				%obj.target = %target.getID();
				%obj.zombieHadTarget = true;
				%obj.zombieNotSeenTargetSince = $sim::time;
			}
			else
			{
				%obj.target = 0;
			}
			
			%obj.zombieGooseChase = false;
			%obj.setAimObject(%obj.target);
			%obj.setMoveObject(%obj.target);
			
			return true;
		}
	}
	
	return false;
}

// Funct: Handles the death of a zombie, bot or otherwise.
function Armor::zombieDeath(%this, %obj, %client, %source, %sourceClient, %bot, %pos, %damage, %damageType)
{
	return false;
}

// Funct: Dummy function for route detection
// Sends: void (always continue default route algorithm)
function Armor::zombieDetectRoute(%this, %obj, %canSee, %dist, %reach, %zDist, %hCol, %lCol, %hHit, %lhit, %fhit)
{
	return;
}

// Funct: Moves %victim to %obj
// Sends: vector3F
function Armor::zombieDragToObject(%this, %obj, %victim)
{
	if(zombieValidTarget(%obj, %victim))
	{
		%oPos	= %obj.getPosition();
		%vPos	= %victim.getPosition();
		%dif	= getWords(vectorSub(%oPos, %vPos), 0) SPC 0;
		
		%vel	= vectorScale(vectorNormalize(vectorScale(%dif, 1)), 4);
		%curVel	= %victim.getVelocity();
		%newVel	= vectorAdd(%vel, %curVel);
		
		%velX	= getWord(%vel, 0);
		%velY	= getWord(%vel, 1);
		%velZ	= getWord(%vel, 2);
		
		%newX	= getWord(%newVel, 0);
		%newY	= getWord(%newVel, 1);
		%newZ	= getWord(%newVel, 2);
		
		if(%newX > 10)
		{
			%newX = 10;
		}
		else if(%newX < -10)
		{
			%newX = -10;
		}
		
		if(%newY > 10)
		{
			%newY = 10;
		}
		else if(%newY < -10)
		{
			%newY = -10;
		}
		
		if(getWord(vectorSub(%oPos, %vPos), 2) > 0 && %newX < 5 && %newY < 5)
		{
			%newZ += 2.0;
		}
		
		if(%newZ > 8)
		{
			%newZ = 8;
		}
		else if(mAbs(%newZ) < 3 && %velZ >= 0)
		{
			%newZ = 3;
		}
		
		%newVel = %newX SPC %newY SPC %newZ;
		%victim.setVelocity(%newVel);
		return %newVel;
	}
	
	return false;
}

// Funct: Dummy function for heritance.
// Sends: true (always continue default clothing algorithm)
function Armor::zombieDress(%this, %obj)
{
	return true;
}

// Funct: Finds a target for %obj
// Sends: target
function Armor::zombieGetTarget(%this, %obj)
{
	if(%obj.isZombie)
	{
		%mini = getMiniGameFromObject(%obj);
		
		if(zombieGameSetUpRight(%mini))
		{
			%distraction = ZombieDirector.getDistraction(%obj);
			
			if(!isObject(%distraction))
			{
				%distraction = ZombieDirector.getDistraction(%obj);
				
				if(!isObject(%distraction))
				{
					%heaviestTarget = "";
					%heaviestWeight = "";
					
					for(%a = 0; %a < %mini.numMembers; %a++)
					{
						%target = %mini.member[%a].player;
						
						if(isObject(%target) && zombieValidTarget(%obj, %target))
						{
							%weight = ZombieDirector.getWeight(%obj, %target);
							
							if((%weight > 0 || %obj.zombieOmnipotence) && (%heaviestWeight $= "" || %weight > %heaviestWeight))
							{
								%heaviestWeight = %weight;
								%heaviestTarget = %target;
							}
						}
					}
					
					if(isObject(%heaviestTarget))
					{
						%mount = %heaviestTarget.getObjectMount();
						
						if(isObject(%mount) && zombieValidTarget(%obj, %mount))
						{
							%target = %mount;
						}
						else
						{
							%target = %heaviestTarget;
						}
					}
					else
					{
						%target = 0;
					}
				}
				else
				{
					%target = %distraction;
				}
				
				return %target;
			}
		}
	}
	
	return false;
}

// Funct: Determines if %obj can raycast to %target's head or center of mass.
// Sends: bool
function Armor::zombieSee(%this, %obj, %target)
{
	if(%this.isZombieArmor && isObject(%target))
	{
		if(%target.zombieDistract)
		{
			return true;
		}
		else if(!%target.isCloaked || %target.isCloaking)
		{
			%start = %obj.getEyePoint();
			%end = %target.getEyePoint();
			
			if(vectorDist(%start, %end) <= 100)
			{
				%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
				
				%ray = containerRayCast(%start, %end, %masks);
				%col = getWord(%ray, 0);
				
				if(!isObject(%col))
				{
					return true;
				}
			}
		}
	}
	
	return false;
}

// Funct: Dummy function for inheritance
// Sends: void
function Armor::zombieSpecial(%this, %obj)
{
	return;
}

// Funct: Attempts to clear bricks surrounding %obj
// Sends: bool 
function Armor::zombieSmash(%this, %obj)
{
	if(%obj.isZombie && $sim::time - %obj.zombieLastSmash > 0.25)
	{
		%obj.zombieLastSmash = $sim::time;
		
		%client = %obj.client;
		%energy = %obj.getEnergyLevel();
		
		if(%energy >= 50)
		{
			%energy -= 50;
			
			%obj.zombieCanActivate = true;
			%obj.playThread(0, "activate2");
			%obj.zombieCanActivate = false;
			
			%scale = %obj.getScale();
			%scaleFactor = (getWord(%scale, 0) + getWord(%scale, 1) + getWord(%scale, 2)) / 3;
			
			%start = %obj.getEyePoint();
			%aimVec = %obj.zombieAimVec();
			%range = 4 * %scaleFactor;
			%end = vectorAdd(%start, vectorScale(%aimVec, %range));
			%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
			
			%ray = containerRayCast(%start, %end, %masks, %obj);
			%col = getWord(%ray, 0);
			
			if(isObject(%col))
			{
				%mini = getMinigameFromObject(%obj);
				%pos = posFromRaycast(%ray);
				
				ZombieSmashProjectile.brickExplosionForce				= %scaleFactor * 20;      
				ZombieSmashProjectile.brickExplosionRadius				= %scaleFactor * 2;
				ZombieSmashProjectile.brickExplosionMaxVolume			= %scaleFactor * 40;
				ZombieSmashProjectile.brickExplosionMaxVolumeFloating	= %scaleFactor * 60;
				
				%p = new Projectile()
				{
					dataBlock		= ZombieSmashProjectile;
					initialPosition	= %pos;
					initialVelocity	= %obj.getEyeVector();
					sourceObject	= %obj;
					sourceSlot		= 0;
					client			= zombieBot(%obj) ?  %mini.owner : %client;
				};
				
				if(isObject(%p))
				{
					
					if(isObject(%mini))
					{
						%temp = %mini.brickDamage;
						%mini.brickDamage = true;
						
						MissionCleanup.add(%p);
						%p.explode();
						
						%mini.brickDamage = %temp;
					}
					else
					{
						MissionCleanup.add(%p);
						%p.explode();
					}
					
					%obj.setEnergyLevel(%energy);
					
					return true;
				}
			}
		}
	}
	
	return false;
}

// Funct: Moves %obj around randomly (checks for obstructions and flooring)
// Sends: false / vector3F
function Armor::zombieWander(%this, %obj)
{
	if(%this.isZombieArmor)
	{
		%start = %obj.getEyePoint();
		%end = vectorAdd(%start, getRandom(-3, 3) SPC getRandom(-3, 3) SPC "0");
		%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
		
		%col = getWord(containerRayCast(%start, %end, %masks, %obj), 0);
		
		if(!isObject(%col))
		{
			%floor = getWord(containerRayCast(%end, vectorAdd(%end, "0 0 -4"), %masks, %obj), 0);
			
			if(isObject(%floor))
			{
				%obj.setMoveDestination(%end);
				return %start;
			}
		}
	}
	
	return false;
}

// ============================================================
// Section 4 : fxDTSBrick
// ============================================================
// Events
function fxDTSBrick::onHumanActivate(%this, %client)
{
	$InputTarget_["Self"] = %this;
	
	if(isObject(%client))
	{
		if(isObject(%client.player))
		{
			$InputTarget_["Player"] = %client.player;
		}
		
		$InputTarget_["Client"] = %client;
		
		if(zombieSameGame(%this, %client))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
		}
		else
		{
			$InputTarget_["MiniGame"] = -1;
		}
	}
	else
	{
		$InputTarget_["Player"] = -1;
		$InputTarget_["Client"] = -1;
		$InputTarget_["MiniGame"] = -1;
	}
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onHumanActivate", %client);
}

function fxDTSBrick::onHumanTouch(%this, %obj)
{
	$InputTarget_["Self"] = %this;
	
	if(isObject(%obj))
	{
		if(!zombieBot(%obj))
		{
			$InputTarget_["Bot"] = -1;
			$InputTarget_["Player"] = %obj;
			$InputTarget_["Client"] = %obj.client;
		}
		else
		{
			$InputTarget_["Bot"] = %obj;
			$InputTarget_["Player"] = -1;
			$InputTarget_["Client"] = -1;
		}
		
		if(zombieSameGame(%this, %obj))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
		}
		else
		{
			$InputTarget_["MiniGame"] = -1;
		}
	}
	else
	{
		$InputTarget_["Bot"] = -1;
		$InputTarget_["Player"] = -1;
		$InputTarget_["Client"] = -1;
		$InputTarget_["MiniGame"] = -1;
	}
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onHumanTouch", %obj);
}

function fxDTSBrick::onMinigameRand1(%this)
{
	$InputTarget_["Self"] = %this;
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onMinigameRand1", %this.getGroup().client);
}

function fxDTSBrick::onMinigameRand2(%this)
{
	$InputTarget_["Self"] = %this;
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
				
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onMinigameRand2", %this.getGroup().client);
}

function fxDTSBrick::onZombieActivate(%this, %client)
{
	$InputTarget_["Self"] = %this;
	
	if(isObject(%client))
	{
		if(isObject(%client.player))
		{
			$InputTarget_["Player"] = %client.player;
		}
		
		$InputTarget_["Client"] = %client;
		
		if(zombieSameGame(%this, %client))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
		}
		else
		{
			$InputTarget_["MiniGame"] = -1;
		}
	}
	else
	{
		$InputTarget_["Player"] = -1;
		$InputTarget_["Client"] = -1;
		$InputTarget_["MiniGame"] = -1;
	}
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onZombieActivate", %client);
}

function fxDTSBrick::onZombieKill(%this, %obj, %client)
{
	$InputTarget_["Self"] = %this;
	
	if(zombieBot(%obj))
	{
		$InputTarget_["Bot"] = %obj;
	}
	else
	{
		$InputTarget_["Bot"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onZombieKill", %obj);
}

function fxDTSBrick::onZombieKilled(%this, %client)
{
	$InputTarget_["Self"] = %this;
	
	if(isObject(%client))
	{
		if(isObject(%client.player))
		{
			$InputTarget_["Player"] = %client.player;
		}
		else
		{
			$InputTarget_["Player"] = -1;
		}
		
		$InputTarget_["Client"] = %client;
		
		if(zombieSameGame(%this, %client))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
		}
		else
		{
			$InputTarget_["MiniGame"] = -1;
		}
	}
	else
	{
		$InputTarget_["Player"] = -1;
		$InputTarget_["Client"] = -1;
		$InputTarget_["MiniGame"] = -1;
	}
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onZombieKilled", %client);
}

function fxDTSBrick::onZombieSpawned(%this, %obj)
{
	$InputTarget_["Self"] = %this;
	
	if(zombieBot(%obj))
	{
		$InputTarget_["Bot"] = %obj;
	}
	else
	{
		$InputTarget_["Bot"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onZombieSpawned", %obj);
}

function fxDTSBrick::onZombieTouch(%this, %obj)
{
	$InputTarget_["Self"] = %this;
	
	if(isObject(%obj))
	{
		if(!zombieBot(%obj))
		{
			$InputTarget_["Bot"] = -1;
			$InputTarget_["Player"] = %obj;
			$InputTarget_["Client"] = %obj.client;
		}
		else
		{
			$InputTarget_["Bot"] = %obj;
			$InputTarget_["Player"] = -1;
			$InputTarget_["Client"] = -1;
		}
		
		if(zombieSameGame(%this, %obj))
		{
			$InputTarget_["MiniGame"] = getMiniGameFromObject(%this);
		}
		else
		{
			$InputTarget_["MiniGame"] = -1;
		}
	}
	else
	{
		$InputTarget_["Bot"] = -1;
		$InputTarget_["Player"] = -1;
		$InputTarget_["Client"] = -1;
		$InputTarget_["MiniGame"] = -1;
	}
	
	if(isObject(%this.getGroup().client))
	{
		if(isObject(%this.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %this.getGroup().client.player;
		}
		
		$InputTarget_["OwnerClient"] = %this.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%this.processInputEvent("onZombieTouch", %obj);
}

// Content
function fxDTSBrick::contentRestrictionHuman(%this, %action, %direction, %text, %client)
{
	%contentTypeID = %this.contentTypeID();
	
	if(%contentTypeID > -1)
	{
		%blid_permission = contentBLIDCheck(%client, %text);
		
		if((!%client.isZombie && %blid_permission != 0) || %blid_permission == 1)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		return -1;
	}
}

function fxDTSBrick::contentRestrictionZombie(%this, %action, %direction, %text, %client)
{
	%contentTypeID = %this.contentTypeID();
	
	if(%contentTypeID > -1)
	{
		%blid_permission = contentBLIDCheck(%client, %text);
		
		if((%client.isZombie && %blid_permission != 0) || %blid_permission == 1)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		return -1;
	}
}

// Normal

function fxDTSBrick::zombieCanSpawn(%this)
{
	%db = %this.vehicleDatablock;
	
	if(%db.isZombieArmor)
	{
		%mini = getMiniGameFromObject(%this);
		
		// can you believe this is valid syntax? holy shit
		if(!%mini.zombiesDisabled)
		if(zombieGameSetUpRight(%mini) && miniGameCanUse(%mini.owner, %this))
		if(%mini.getProperty("bots") || %db.zombieAIOnly)
		if($sim::time - %this.zombieSpawnTime >= %mini.vehicleRespawnTime / 1000)
		if(!%db.zombieSpawnOnce || %mini.zombieTimeAssembled - %this.zombieSpawnTime >= 0)
			return true;
	}
	
	return false;
}

// Funct: Determines the spawnpoint for a brick (where an object would spawn on top of it)
// Sends: vector7F
function fxDTSBrick::zombieSpawnPoint(%this)
{
	%box	= %this.getWorldBox();
	%point	= vectorAdd(getWords(%box, 0, 1) SPC 0, getWords(%box, 3, 4) SPC 0);
	%point	= getWords(vectorScale(%point, 0.5), 0, 1) SPC getWord(%box, 5);
	%trans	= %point SPC getWords(%this.getTransform(), 3, 6);
	
	return %trans;
}

// ============================================================
// Section 5 : GameConnection
// ============================================================

// Funct: Attempts to resapwn %cient as a tank.
// Sends: void
function GameConnection::spawnAsTank(%client)
{
	if(%client.isZombie)
	{
		echo("ZAPT - Spawning" SPC %client.name SPC "as tank in" SPC %client.minigame.owner.name @ "'s minigame.");
		
		%client.isZombie = true;
		%client.spawningAsTank = true;
		
		%client.spawnPlayer();
		
		if(isObject(%client.player))
		{
			%client.player.setDatablock("ZombieTankArmor");
		}
		
		%client.spawningAsTank = false;
	}
}

// Funct: Sets %client as rescued and respawns as a zombie.
// Sends: bool
function GameConnection::rescue(%client)
{
	%mini = %client.minigame;
	
	if(zombieGameSetUpRight(%mini) && !%client.isZombie && !%client.isRescued && !isObject(%client.player.zombieBeingRiddenBy))
	{
		%client.isRescued = true;
		
		%mini.messageAll('', "<color:94ce24>" @ %client.name @ " <bitmap:" @ $ZAPT::FilePath::Icons @ "CI_Rescue> Escaped");
		
		if(%mini.getProperty("respawn"))
		{
			// It's kind-of stupid seeing this message if the reset timer immediately starts up.
			if(%mini.getNumberOfHumans() > 0)
			{
				messageClient(%client, '', "<color:aaffaa>You have escaped, but will get to play as a zombie until the round is over.");
			}
			
			%client.isZombie = true; // lol
			%client.isDead = false;
		}
		else
		{
			%client.isZombie = false;
			%client.isDead = true;
			messageClient(%client, '', "<color:aaffaa>You have escaped and will get to play next round.");
		}
		
		%client.spawnPlayer();
		
		return true;
	}
	
	return false;
}

// By Space Guy
// Funct: Finds an acceptable target for %client.
// Sends: void
function GameConnection::zombieAutoCameraTarget(%client)
{
	%mini = %client.minigame;
	
	if(zombieGameSetUpRight(%mini))
	{
		messageClient(%client, 'MsgYourSpawn');
		%cam = %client.camera;
		
		for(%i = 0; %i < %mini.numMembers; %i++)
		{
			%member = %mini.member[%i];
			
			if(isObject(%member.player) && %member.player.getState() !$= "Dead")
			{
				%cam.setMode("Corpse", %member.player);
				%client.tdmCameraTarget = %i;
				%client.setControlObject(%cam);
				return;
			}
		}
	}
}

// Funct: Sends a cry for help over team chat.
function GameConnection::zombieHelp(%client, %message, %warning)
{
	%mini = %client.minigame;
	%player = %client.player;
	
	if(zombieGameSetUpRight(%mini) && isObject(%player))
	{
		if(%message $= "")
		{
			switch(getRandom(0, 4))
			{
				case 0: %message = "Yo, a little help over here?";
				case 1: %message = "I'm dying over here!";
				case 2: %message = "HELP!";
				case 3: %message = "Somethings got me!";
				case 4: %message = "Noooooooooo!!";
				default: %message = "Help me...";
			}
		}
		
		switch(%client.isZombie)
		{
			case false: %tag = "<color:aaffaa>Survivor";
			case true: %tag = "<color:ffaaaa>Zombie";
			default:  %tag = "\c0IBAN_FAILS_AT_SCRIPT";
		}
		
		%message = "<color:ffffaa><AUTO> \c7[" @ %tag @ "\c7]" SPC %client.clanPrefix @ "\c3" @ %client.name @ "\c7" @ %client.clanSuffix @ "\c4:" SPC %message;
		%pos = %player.getPosition();
		
		for(%a = 0; %a < %client.minigame.numMembers; %a++)
		{
			%member = %client.minigame.member[%a];
			
			if(%client.isZombie == %member.isZombie)
			{
				messageClient(%member, '', %message);
				
				if(%warning && %client.getID() != %member.getID() && isObject(%member.player) && vectorDist(%member.player.getPosition(), %pos) <= 16)
				{
					commandToClient(%member, 'centerPrint', "<just:right><color:ffffaa><font:impact:24>" @ %client.name @ " is in trouble!\n<font:impact:18>(And very close to you!)", 2);
				}
			}
		}
	}
}

// ============================================================
// Section 6 : MinigameSO
// ============================================================
function MinigameSO::announceTimeLeft(%so)
{
	if(zombieGameSetUpRight(%so))
	{
		if(!isFunction("Mutation_" @ %so.getProperty("name") @ "_Time") || !call("Mutation_" @ %so.getProperty("name") @ "_Time", %so))
		{
			// We cant really lose if there's no way to die
			// And there's no point in winning, either, if we cant lose.
			if(%so.ZAPT_PlayerZombies)
			{
				// Did we lose just now?
				if(%so.zombieLastHumanCount != 0 && %so.getNumberOfHumans() == 0)
				{
					%so.zombieTimeLeftCd = %so.ZAPT_Time_Reset;
					%so.zombieGameLost = true;
				}
				
				// Mark how many are left...
				%so.zombieLastHumanCount = %so.getNumberOfHumans();
				
				// Count down to reset/loss/victory.
				if(%so.zombieLastHumanCount)
				{
					if(%so.ZAPT_Timer)
					{
						%so.zombieTimeLeft--;
					}
					
					%so.zombieTimeLeftCd = %so.ZAPT_Time_Reset;
					%so.zombieGameLost = false;
				}
				else
				{
					%so.zombieTimeLeftCd--;
					%so.zombieGameLost = true;
				}
				
				// Still kicking?
				// Check for a saferoom victory.
				if(!%so.zombieGameLost)
				{
					%safees = false;
					
					for(%a = 0; %a < %so.numMembers; %a++)
					{
						%member = %so.member[%a].player;
						
						if(%member.zombieInSafeHouse)
						{
							%safees = true;
							break;
						}
					}
					
					if(%safees)
					{
						%victory = %so.safehouseCheck();
						%victoryCond = getField(%victory, 0);
						%victoryParam = getField(%victory, 1);
						
						switch(%victoryCond)
						{
							case 1:
								return;
							
							case 2:
								if(%victoryParam > 0)
								{
									%safe = " (" @ %victoryParam @ " safe)";
								}
								
							case 3:
								for(%a = 0; %a < %so.numMembers; %a++)
								{
									%member = %so.member[%a];
									
									if((mFloor($sim::time) % 2) == 0)
									{
										if(!%member.isZombie || %so.ZAPT_Rescue)
										{
											%color = "<color:ffaaaa>";
										}
										else
										{
											%color = "<color:aaffaa>";
										}
									}
									else
									{
										%color = "<color:ffffaa>";
									}
									
									%member.bottomPrint(%color @ "Zombies in Safe House!", 1.25);
									
									if(isObject(%member.player))
									{
										%member.player.setShapeNameColor($MiniGameColorF[%so.colorIdx] SPC "1");
									}
								}
								
								cancel(%so.zombieAnnounceTimeSched);
								%so.zombieAnnounceTimeSched = %so.schedule(1000, "announceTimeLeft");
								return;
						}
					}
				}
				
				if(%so.ZAPT_Timer || %so.zombieGameLost)
				{
					// If (time AND survivors) OR (game is list AND count-down time left)
					// Basically, is game still running/resetting?
					if((%so.zombieTimeLeft > 0 && %so.zombieLastHumanCount) || (%so.zombieGameLost && %so.zombieTimeLeftCd > 0))
					{
						if(%so.zombieLastHumanCount)
						{
							%minutes = mFloor(%so.zombieTimeLeft / 60);
							%seconds = %so.zombieTimeLeft % 60;
							
							if(%minutes)
							{
								%timeString = %minutes @ " minute" @ (%minutes != 1 ? "s" : "") @ " and " @ %seconds @ " second" @ (%seconds != 1 ? "s" : "");
							}
							else
							{
								%timeString = %seconds @ " second" @ (%seconds != 1 ? "s" : "");
							}
							
							for(%a = 0; %a < %so.numMembers; %a++)
							{
								%member = %so.member[%a];
								
								if(%member.isZombie)
								{
									if(%so.ZAPT_Rescue)
									{
										%color = "<color:ffaaaa>";
									}
									else
									{
										%color = "<color:aaffaa>";
									}
								}
								else
								{
									if(!%so.ZAPT_Rescue)
									{
										%color = "<color:ffaaaa>";
									}
									else
									{
										%color = "<color:aaffaa>";
									}
								}
								
								%survivors = %so.zombieLastHumanCount;
								
								if(%so.ZAPT_Rescue)
								{
									%member.bottomPrint(%color @ "The " @ $ZAPT::ArmyName @ " will rescue " @ %survivors @ " survivor" @ (%survivors > 1 ? "s" : "") @ " in " @ %timeString @ "." @ %safe, 1.25);
								}
								else
								{
									%member.bottomPrint(%color @ "The " @ $ZAPT::ArmyName @ " leaves behind " @ %survivors @ " survivor" @ (%survivors > 1 ? "s" : "") @ " in " @ %timeString @ "." @ %safe, 1.25);
								}
								
								if(isObject(%member.player))
								{
									%member.player.setShapeNameColor($MiniGameColorF[%so.colorIdx] SPC "1");
								}
							}
						}
						else
						{
							%minutes = mFloor(%so.zombieTimeLeftCd / 60);
							%seconds = %so.zombieTimeLeftCd % 60;
							
							if(%minutes)
							{
								%timeString = %minutes @ " minute" @ (%minutes != 1 ? "s" : "") @ " and " @ %seconds @ " second" @ (%seconds != 1 ? "s" : "");
							}
							else
							{
								%timeString = %seconds @ " second" @ (%seconds != 1 ? "s" : "");
							}
							
							if(%so.getNumberOfSurvivors() == %so.numMembers)
							{
								%so.bottomPrintAll("<color:aaffaa>Everyone was evacuated! Restarting minigame in " @ %timeString @ ".", 1.25);
							}
							else if(%so.getNumberOfSurvivors() > 0)
							{
								%so.bottomPrintAll("<color:ffaaaa>Everyone is dead or evacuated. Restarting minigame in " @ %timeString @ ".", 1.25);
							}
							else
							{
								%so.bottomPrintAll("<color:ffaaaa>Everyone is dead. Restarting minigame in " @ %timeString @ ".", 1.25);
							}
						}
						
						cancel(%so.zombieAnnounceTimeSched);
						%so.zombieAnnounceTimeSched = %so.schedule(1000, "announceTimeLeft");
					}
					else
					{
						if(%so.getNumberOfSurvivors())
						{
							%memberString = "";
							%deadString = "";
							
							for(%a = 0; %a < %so.numMembers; %a++)
							{
								%member = %so.member[%a];
								
								%survivor = false;
								
								if(!%so.ZAPT_Rescue)
								{
									if(%member.isRescued)
									{
										%survivor = true;
									}
									else if(%so.ZAPT_Timer)
									{
										%left4dead = true;
									}
								}
								else if(%so.ZAPT_Rescue && ((!%member.isDead && !%member.isZombie) || %member.isRescued))
								{
									%survivor = true;
								}
								
								if(%left4dead)
								{
									if(%deadString $= "")
									{
										%deadString = %member.name;
									}
									else
									{
										%deadString = %deadString @ ", " @ %member.name;
									}
								}
								
								if(%survivor)
								{
									if(%memberString $= "")
									{
										%memberString = %member.name;
									}
									else
									{
										%memberString = %memberString @ ", " @ %member.name;
									}
								}
							}
							
							%so.messageAll('', "<color:aaffaa>The following humans escaped: \c3" @ %memberString);
						}
						else
						{
							%so.messageAll('', "<color:ffaaaa>Nobody lived.");
						}
						
						cancel(%so.zombieAnnounceTimeSched);
						%so.reset(%so.owner);
					}
				}
				// If we're neither on a timer or counting down to reset.
				// Survival mode.
				else
				{
					%time = mFloor($sim::time - %so.zombieTimeAssembled);
					%minutes =  mFloor(%time / 60);
					%seconds = %time % 60;
					
					if(%minutes)
					{
						%timeString = %minutes @ " minute" @ (%minutes != 1 ? "s" : "") @ " and " @ %seconds @ " second" @ (%seconds > 1 ? "s" : "");
					}
					else
					{
						%timeString = %seconds @ " second" @ (%seconds > 1 ? "s" : "");
					}
					
					for(%a = 0; %a < %so.numMembers; %a++)
					{
						%member = %so.member[%a];
						
						if(%member.isZombie && !%so.ZAPT_Rescue)
						{
							%color = "<color:ffaaaa>";
						}
						else
						{
							%color = "<color:aaffaa>";
						}
						
						%survivors = %so.zombieLastHumanCount;
						
						%member.bottomPrint(%color @ "There remains " @ %survivors @ " survivor" @ (%survivors != 1 ? "s" : "") @ " after " @ %timeString @ "." @ %safe, 1.25);
						
						if(isObject(%member.player))
						{
							%member.player.setShapeNameColor($MiniGameColorF[%so.colorIdx] SPC "1");
						}
					}
				}
			}
			else
			{
				for(%a = 0; %a < %so.numMembers; %a++)
				{
					%member = %so.member[%a].player;
					
					if(isObject(%member))
					{
						%member.setShapeNameColor($MiniGameColorF[%so.colorIdx] SPC "1");
					}
				}
			}
		}
		
		cancel(%so.zombieAnnounceTimeSched);
		%so.zombieAnnounceTimeSched = %so.schedule(1000, "announceTimeLeft");
	}
}

function MinigameSO::assembleZombieGame(%so)
{
	if(!$ZAPT::AdminOnly || %so.owner.isAdmin)
	{
		%so.zombiesDisabled			= false;
		%so.zombieGameLost			= false;
		%so.zombieLastHumanCount	= %so.numMembers;
		%so.zombieLastDeathTime		= 0;
		%so.zombieSafehouseVictory	= true;
		%so.zombieSelectedInfected	= false;
		%so.zombieStress			= (%so.ZAPT_Stress $= "" ? 100 : %so.ZAPT_Stress);
		%so.zombieTimeAssembled		= $sim::time;
		%so.zombieTimeLeft			= 0;
		%so.zombieTimeLeftCd		= 0;
		%so.zombieTimeSinceTank		= $sim::time;
		
		cancel(%so.zombieAnnounceTimeSched);
		cancel(%so.zombieOutbreakSched);
		
		if(zombieGameSetUpRight(%so))
		{
			// Timer
			if(%so.ZAPT_Timer)
			{
				if(mFloor(%so.ZAPT_Time_Rescue) <= 0)
				{
					%so.ZAPT_Time_Rescue = 180;
				}
				
				%so.zombieTimeLeft = %so.ZAPT_Time_Rescue;
			}
			else
			{
				%so.zombieTimeLeft = 1;
			}
			
			// Reset Time
			if(mFloor(%so.ZAPT_Time_Reset) <= 0)
			{
				%so.ZAPT_Time_Reset = 10;
			}
			
			// Game Modes
			if(!isObject(ZombieMutations.mutation[%so.ZAPT_GameMode]))
			{
				%so.ZAPT_GameMode = 0;
			}
			
			if(%so.zombieAssembledAs != %so.ZAPT_GameMode)
			{
				%name = %so.getProperty("uiName");
				%so.messageAll('', "<color:ffaaaa>You are now playing \c3" @ %name @ "<color:ffaaaa> mode.");
				
				if(%so.zombieAssembledOnce)
				{
					%so.zombieAssembledAs = %so.ZAPT_GameMode;
					%so.reset(%so.owner);
					return;
				}
			}
			
			if(!isFunction("Mutation_" @ %so.getProperty("name") @ "_Assemble") || !call("Mutation_" @ %so.getProperty("name") @ "_Assemble", %so))
			{
				if(%so.ZAPT_Outbreak)
				{
					if(mFloor(%so.ZAPT_Time_Outbreak) < 0)
					{
						%so.ZAPT_Time_Outbreak = 30;
					}
					
					%so.zombieOutbreakSched = %so.schedule(%so.ZAPT_Time_Outbreak * 1000, "zombieOutbreak");
				}
				
				if(%so.numMembers == 1 && !%so.zombieAssembledOnce)
				{
					%so.messageAll('', "<color:ffffaa>You are the Last " @ (%so.member[0].chest ? "Woman" : "Man") @ " on Earth. Certain zombies are disabled until others join.");
				}
			}
			
			for(%a = 0; %a < %so.numMembers; %a++)
			{
				%member = %so.member[%a];
				
				if(%member.zombieHudCells > -1)
				{
					ZombieFriends.clearFriends(%member);
					ZombieFriends.getFriends(%member);
					ZombieFriends.sendData(%member);
				}
			}
			
			%so.zombieAnnounceTimeSched	= %so.schedule(0, "announceTimeLeft");
			%so.zombieAssembledAs		= %so.ZAPT_GameMode;
			%so.zombieAssembledOnce		= true;
			%so.zombieTimeLeftCd		= %so.ZAPT_Time_Reset;
			//%so.zombieTankSched			= %so.schedule(getRandom(20, 80) * 1000, "pickTank");
		}
	}
}

function MinigameSO::clearAllZombies(%so)
{
	%zombies = 0;
	
	for(%a = 0; %a < %so.numMembers; %a++)
	{
		%member = %so.member[%a];
		
		if(%member.isZombie)
		{
			%zombies++;
		}
		
		%member.isDead = false;
		%member.isRescued = false;
		%member.isZombie = false;
		%member.spawningAsTank = false;
		cancel(%member.zombieSpawnAsTankSched);
		
		if(isObject(%member.player) && %member.player.isZombie)
		{
			%member.spawnPlayer();
		}
	}
	
	ZombieDirector.clearMob(%so);
	
	if(isFunction("Mutation_" @ %so.getProperty("name") @ "_ClearZombies"))
	{
		call("Mutation_" @ %so.getProperty("name") @ "_ClearZombies", %so);
	}
	
	return %zombies;
}

function MinigameSO::disableZombies(%so, %disabled)
{
	%so.zombiesDisabled = %disabled;
}

function MinigameSO::escape(%so, %client)
{
	if(zombieGameSetUpRight(%so))
	{
		if(getSimTime() - %so.lastResetTime >= 5000)
		{
			if(%so.getNumberOfHumans())
			{
				%memberString = "";
				
				for(%a = 0; %a < %so.numMembers; %a++)
				{
					%member = %so.member[%a];
					
					if((!%member.isDead && !%member.isZombie) || %member.isRescued)
					{
						if(%memberString $= "")
						{
							%memberString = %member.name;
						}
						else
						{
							%memberString = %memberString @ ", " @ %member.name;
						}
					}
				}
				
				%so.messageAll('', "<color:aaffaa>The following humans escaped: \c3" @ %memberString);
			}
			
			cancel(%so.zombieAnnounceTimeSched);
			%so.reset(%so.owner);
		}
		else if(isObject(%client))
		{
			messageClient(%client, '', "\c5Cannot escape so soon!");
		}
	}
}

// Funct: Counts number of humans that are actively walking around.
// Sends: int
function MinigameSO::getNumberOfHumans(%so)
{
	%count = 0;
	
	for(%a = 0; %a < %so.member[%a]; %a++)
	{
		if(!%so.member[%a].isZombie && isObject(%so.member[%a].player))
		{
			%count++;
		}
	}
	
	return %count;
}

// Funct: Counts number of humans that are actively walking around + rescued.
// Sends: int
function MinigameSO::getNumberOfSurvivors(%so)
{
	%count = 0;
	
	for(%a = 0; %a < %so.member[%a]; %a++)
	{
		%member = %so.member[%a];
		
		if(!%so.ZAPT_Rescue && %member.isRescued)
		{
			%count++;
		}
		else if(%so.ZAPT_Rescue && (!%member.isDead && !%member.isZombie) || %member.isRescued)
		{
			%count++;
		}
	}
	
	return %count;
}

// Funct: Counts number of tank zombies.
// Sends: int
function MinigameSO::getNumberOfTanks(%so)
{
	%count = 0;
	
	for(%a = 0; %a < %so.member[%a]; %a++)
	{
		%member = %so.member[%a];
		
		if(!isEventPending(%member.zombieSpawnAsTankSched))
		{
			if(isObject(%member.player))
			{
				%db = %member.player.getDatablock();
				
				if(%db.isZombieTank)
				{
					%count++;
				}
			}
		}
		else
		{
			%count++;
		}
	}
	
	return %count;
}

// Funct: Counts number of zombies, including tanks.
// Sends: int
function MinigameSO::getNumberOfZombies(%so)
{
	%count = 0;
	
	for(%a = 0; %a < %so.member[%a]; %a++)
	{
		if(%so.member[%a].isZombie || !isObject(%so.member[%a].player))
		{
			%count++;
		}
	}
	
	return %count;
}

// Funct: Gets a value from a mutation.
// Sends: ZombieMutations.mutation[%so.ZAPT_GameMode].%PROPERTY%
function MinigameSO::getProperty(%so, %property)
{
	if(zombieGameSetUpRight(%so))
	{
		if(isObject(ZombieMutations.mutation[%so.ZAPT_GameMode]))
		{
			eval("%result = ZombieMutations.mutation[%so.ZAPT_GameMode]." @ %property @ ";");
			
			return %result;
		}
	}
	
	return false;
}

function MinigameSO::panic(%so)
{
	ZombieDirector.panic(%so);
}

function MinigameSO::pickTank(%so)
{
	if(%so.ZAPT_Tank && !%so.getNumberOfTanks())
	{
		if(ZombieDirector.isTankSpawnForThisMini(%so))
		{
			%zombies = 0;
			
			for(%a = 0; %a < %so.numMembers; %a++)
			{
				%client = %so.member[%a];
				
				if(%client.isZombie && !%client.isDead && %client.score > 0)
				{
					%player = %client.player;
					
					if(!%client.spawningAsTank)
					{
						%zombie[%zombies] = %client;
						%zombies++;
					}
				}
			}
			
			if(%zombies)
			{
				%randomClient = %zombie[getRandom(0, %zombies)];
				
				if(isObject(%randomClient))
				{
					messageClient(%randomClient, '', "<color:ffaaaa>The director has chosen you to play the tank! Get ready...");
					%randomClient.zombieSpawnAsTankSched = %randomClient.schedule(3000, "spawnAsTank");
					
					return true;
				}
			}			
		}
	}
	
	return false;
	//cancel(%so.zombieTankSched);
	// Allows multiple tanks per game.
	// Settings can change freely, so we put this outside of all if{}s.
	//%so.zombieTankSched = %so.schedule(getRandom(45, 75) * 1000, "pickTank");
}

function MinigameSO::safehouseCheck(%so)
{
	if(zombieGameSetUpRight(%so))
	{
		if(%so.zombieSafehouseVictory)
		{
			%safees = 0;
			
			for(%a = 0; %a < %so.numMembers; %a++)
			{
				%member = %so.member[%a].player;
				
				if(isObject(%member) && %member.zombieInSafeHouse && ((!%member.isDead && !%member.isZombie && !%member.zombieHelpless) || %member.isRescued))
				{
					%member.zombieSafeHouseChecked = true;
					%safees++;
				}
				else if(!%member.isZombie)
				{
				
				}
			}
			
			if(%safees > 0 && %safees == %so.getNumberOfHumans())
			{
				if(!ZombieDirector.isZombieInSaferooms(%so))
				{
					if(getSimTime() - %so.lastResetTime >= 5000)
					{
						%memberString = "";
						
						for(%a = 0; %a < %so.numMembers; %a++)
						{
							%member = %so.member[%a];
							
							if((!%member.isDead && !%member.isZombie) || %member.isRescued)
							{
								if(%memberString $= "")
								{
									%memberString = %member.name;
								}
								else
								{
									%memberString = %memberString @ ", " @ %member.name;
								}
							}
						}
						
						cancel(%so.zombieAnnounceTimeSched);
						%so.messageAll('', "<color:aaffaa>The following humans escaped: \c3" @ %memberString);
						%so.reset(%so.owner);
						
						return 1;
					}
					else
					{
						%so.centerPrintAll("\c5Cannot escape so soon!", 0.5);
					}
				}
				else
				{
					return 3;
				}
			}
			else
			{
				return 2 TAB %safees;
			}
		}
		
		return 0;
	}
	
	return -1;
}

function MinigameSO::safehouseVictory(%so, %enabled)
{
	// Just an output event that toggles if safehouseCheck autoflops (on FALSE)
	%so.zombieSafehouseVictory = %enabled;
}

function MinigameSO::zombieOutbreak(%so)
{
	if(!%so.getNumberOfZombies() && %so.numMembers > 1)
	{
		%members = 0;
		%memberList = "";
		
		for(%a = 0; %a < %so.numMembers; %a++)
		{
			%member = %so.member[%a];
			
			if(isObject(%member.player))
			{
				if(!%members)
				{
					%memberList = %member.getID();
				}
				else
				{
					%memberList = %memberList TAB %member.getID();
				}
				
				%members++;
			}
		}
		
		%victim = getField(%memberList, getRandom(0, %members - 1));
		%victim.isZombie = true;
		%victim.player.setDatablock("ZombieStandardArmor");
		%victim.applyBodyParts();
		%victim.applyBodyColors();
		
		serverCmdUnUseTool(%victim);
		
		messageClient(%victim, '', "<color:ffaaaa>You were randomly selected to be the first zombie! Kill your friends.");
		%so.messageAll('', "The Outbreak <bitmap:" @ $ZAPT::FilePath::Icons @ "CI_Outbreak> " @ %victim.name);
	}
}

// ============================================================
// Section 7 : Player
// ============================================================
function Player::addItems(%obj, %item, %num)
{
	if(%obj.getDatablock().getName() $= "PlayerL4DSurvivorArmor")
	{
		%loadoutStyle = true;
	}
	
	%itemsGiven = 0;
	
	if(isObject(%item))
	{
		%maxTools = %obj.getDatablock().maxTools;
		%tool = 0;
		
		for(%a = 0; %a < %maxTools; %a++)
		{
			%tool = %obj.tool[%a];
			
			if(!isObject(%tool))
			{
				%obj.tool[%a] = %item.getID();
				%obj.weaponCount++;
				%itemsGiven++;
				
				if(!zombieBot(%obj))
				{
					messageClient(%obj.client, 'MsgItemPickup', '', %a, %item.getID());
				}
				
				if(%itemsGiven >= %num)
				{
					break;
				}
			}
		}
	}
	
	return %itemsGiven;
}

function Player::burnLoop(%obj, %loopRuns, %loopDamage, %source)
{
	if(%obj.getType() & $TypeMasks::PlayerObjectType && %obj.getState() !$= "Dead")
	{
		%db = %obj.getDatablock();
		
		if(!%obj.zombieInWater)
		{
			if(!%obj.isZombie)
			{
				%damage = %loopDamage;
				%damageType = $DamageType::Burning;
				%loopRuns--;
				%pos = %obj.getPosition();
			}
			else
			{
				%damage = %loopDamage * 1.25;
				%damageType = $DamageType::Burning;
				%pos = %obj.getPosition();
				
				// Algorithm to recalculate damage if it would not kill the zombie in 45s.
				if(%damage * 45 < %db.maxDamage)
				{
					%damage = mCeil((%db.maxDamage / 45) * getWord(%obj.getScale(), 2));
				}
			}
			
			%obj.mountImage("PlayerNapalmBurnImage", 3);
			%obj.damage(%source, %pos, %damage, %damageType);
			
			if(isObject(%obj) && %obj.getState() !$= "Dead")
			{
				%obj.setTempColor("0 0 0 1", 1000);
				%obj.setDamageFlash(50);
				
				cancel(%obj.damageBurnSched);
				
				if(!%db.fireRetardant && %loopRuns > 0)
				{
					%obj.damageBurnSched = %obj.schedule(1000, "burnLoop", %loopRuns, %loopDamage, %source);
					%obj.damageBurnLoops = %loopRuns;
					%obj.damageBurnDamage = %loopDamage;
				}
				else
				{
					%obj.damageBurnLoops = 0;
					%obj.damageBurnDamage = 0;
					%obj.unMountImage(3);
				}
			}
		}
		else if(%obj.zombieOnFire())
		{
			%obj.damageBurnLoops = 0;
			%obj.damageBurnDamage = 0;
			%obj.unMountImage(3);
		}
	}
}

function Player::healLoop(%obj, %loopSpeed, %loopHeal, %healLeft)
{
	if(%obj.getType() & $TypeMasks::PlayerObjectType)
	{
		cancel(%obj.damageBurnSched);
		
		%damage = %obj.getDamageLevel();
		
		if(%damage > 0)
		{
			%heal = 0;
			
			if(%loopHeal > %damage)
			{
				%heal = %damage;
			}
			else
			{
				%heal = %loopHeal;
			}
			
			if(%heal > %healLeft)
			{
				%heal = %healLeft;
			}
			
			%healLeft -= %heal;
			%obj.setDamageLevel(%damage - %heal);
			
			if(%healLeft > 0 && %heal < %damage)
			{
				%obj.nextHealLoop = %obj.schedule(%loopSpeed, "healLoop", %loopSpeed, %loopHeal, %healLeft);
			}
			else
			{
				
			}
		}
	}
}

function Player::zombieAimVec(%obj)
{
	// Space Guy's code (iban no good at math)
	%fvec = %obj.getForwardVector();
	%fX = getWord(%fvec, 0);
	%fY = getWord(%fvec, 1);
	
	%evec = %obj.getEyeVector();
	%eX = getWord(%evec, 0);
	%eY = getWord(%evec, 1);
	%eZ = getWord(%evec, 2);
	
	%eXY = mSqrt((%eX * %eX) + (%eY * %eY));
	%aimVec = (%fX * %eXY) SPC (%fY * %eXY) SPC %eZ;
	
	return %aimvec;
}

function Player::zombieBeacon(%obj, %time, %client)
{
	if(%time > 0)
	{
		%obj.zombieBeacon = true;
		%obj.zombieBeaconClient = %client;
		
		if(!isObject(%obj.getMountedImage(5)))
		{
			%obj.mountImage(ZombieBileImage, 2);
		}
		
		%obj.zombiePukeVision();
		
		if(isEventPending(%obj.zombieBeaconSched))
		{
			%timeAdd = mFloor(getTimeRemaining(%obj.zombieBeaconSched) / 2);
		}
		else
		{
			%timeAdd = 0;
		}
		
		cancel(%obj.zombieBeaconSched);
		%obj.zombieBeaconSched = %obj.schedule(%time + %timeAdd, "zombieBeacon", 0, -1);
	}
	else
	{
		%obj.zombieBeacon = false;
		%obj.zombieBeaconClient = -1;
		
		%slot = %obj.getMountSlot(ZombieBileImage);
		
		if(%slot > -1)
		{
			%obj.unMountImage(%slot);
		}
	}
}

function Player::zombieClaw(%obj)
{
	if(%obj.isZombie)
	{
		return %obj.getDatablock().zombieClaw(%obj);
	}
	else
	{
		return -1;
	}
}

function Player::zombieItemCheck(%obj)
{
	%client = %obj.client;
	
	if(!zombieBot(%client))
	{
		%itemList = %client.zombieLastItemString;
		%state = %client.zombieLastStatusString;
		
		ZombieFriends.getFriendData(%client, %client);
		
		if(%itemList !$= %client.zombieLastItemString || %state !$= %client.zombieLastStatusString)
		{
			ZombieFriends.sendFriendData(%client);
		}
		
		cancel(%obj.zombieItemSched);
		%obj.zombieItemSched = %obj.schedule(1000, "zombieItemCheck");
	}
}

function Player::zombieJockeyDismount(%obj)
{
	if(isObject(%obj))
	{
		if(isObject(%obj.zombieBeingRiddenBy))
		{
			if(isObject(%obj.zombieBeingRiddenBy.zombieRidingWho))
			{
				if(%obj.zombieBeingRiddenBy.zombieRidingWho.getID() == %obj.getID())
				{
					%obj.zombieBeingRiddenBy.zombieRidingWho = "";
					%obj.zombieBeingRiddenBy.zombieRiding = false;
					%obj.zombieBeingRiddenBy.getDatablock().doDismount(%obj);
				}
			}
		}
		
		if(isObject(%obj.zombieRidingWho))
		{
			if(isObject(%obj.zombieRidingWho.zombieBeingRiddenBy))
			{
				if(%obj.zombieRidingWho.zombieBeingRiddenBy.getID() == %obj.getID())
				{
					%obj.zombieRidingWho.zombieBeingRidden = false;
					%obj.zombieRidingWho.zombieBeingRiddenBy = "";
				}
			}
		}
		
		%obj.zombieBeingRidden = false;
		%obj.zombieBeingRiddenBy = "";
		%obj.zombieRiding = false;
		%obj.zombieRidingWho = "";
		
		if(%obj.isZombie)
		{
			if(%obj.getState() !$= "Dead")
			{
				if(zombieBot(%obj))
				{
					applyDefaultCharacterPrefs(%obj);
				}
				else
				{
					%obj.client.applyBodyParts(%obj);
					%obj.client.applyBodyColors(%obj);
				}
			}
		}
		else
		{
			%obj.zombieHelpless = false;
		}
	}
}

function Player::zombieLight(%obj, %light)
{
	if(isObject(%light))
	{
		if(isObject(%obj.light))
		{
			%obj.light.delete();
		}
		
		%obj.light = new FxLight()
		{
			dataBlock = %light;
			enable = true;
			iconsize = 1;
			player = %obj;
		};
		
		if(isObject(%obj.light))
		{
			%obj.light.attachToObject(%obj);
		}
	}
	else if(isObject(%obj.light))
	{
		%obj.light.delete();
	}
}

function Player::zombieMidPoint(%obj)
{
	%top = %obj.getEyePoint();
	%bot = %obj.getPosition();
	
	%mid = getWords(%bot, 0, 1) SPC ((getWord(%top, 2) + getWord(%bot, 2)) / 2);
	
	return %mid;
}

function Player::zombieOnFire(%obj)
{
	if(isEventPending(%obj.damageBurnSched) ||
		(isObject(%obj.getMountedImage(3) && %obj.getMountedImage(3).getID() == PlayerNapalmBurnImage.getID())))
	{
		return true;
	}
	else
	{
		return false;
	}
}

function Player::zombiePukeVision(%obj)
{
	if(%obj.zombieBeacon)
	{
		%obj.setWhiteOut(0.8);
		
		cancel(%obj.zombiePukeVisionSched);
		%obj.zombiePukeVisionSched = %obj.schedule(250, "zombiePukeVision");
	}
}

function Player::zombieSpecial(%obj)
{
	return %obj.getDatablock().zombieSpecial(%obj);
}

function Player::zombieSmash(%obj)
{
	if(%obj.isZombie)
	{
		return %obj.getDatablock().zombieSmash(%obj);
	}
	else
	{
		return -1;
	}
}

// "Dummy" Functions
// See Armor:: function of same name.
function Player::zombieSee(%obj, %target)
{
	return %obj.getDatablock().zombieSee(%obj, %target);
}

// ============================================================
// Section 8 : Trigger
// ============================================================
// Zombie Trigger
function ZombieTrigger::onEnterTrigger(%this, %trigger, %obj)
{
	if(isObject(%obj))
	{
		switch$(%obj.getClassName())
		{
			case "Player":
				%obj.zombieInSafeHouse = true;
				%obj.zombieSafeHouse = %trigger.getID();
			
			case "WheeledVehicle":
				%mountPoints = %obj.getDatablock().numMountPoints;
				
				for(%a = 0; %a < %mountPoints; %a++)
				{
					%mountedObject = %obj.getMountedObject(%a);
					
					if(isObject(%mountedObject))
					{
						%mountedObject.zombieInSafeHouse = true;
						%mountedObject.zombieSafeHouse = %trigger.getID();
					}
				}
		}
	}
}

function ZombieTrigger::onLeaveTrigger(%this, %trigger, %obj)
{
	if(isObject(%obj))
	{
		switch$(%obj.getClassName())
		{
			case "Player":
				%theirTrigger = %obj.zombieSafeHouse;
				
				if(isObject(%theirTrigger))
				{
					if(%theirTrigger.getID() == %trigger.getID())
					{
						%obj.zombieSafeHouse = "";
						%obj.zombieInSafeHouse = false;
					}
					else
					{
						%obj.zombieInSafeHouse = true;
					}
				}
			
			case "WheeledVehicle":
				%mountPoints = %obj.getDatablock().numMountPoints;
				
				for(%a = 0; %a < %mountPoints; %a++)
				{
					%mountedObject = %obj.getMountedObject(%a);
					
					if(isObject(%mountedObject))
					{
						%theirTrigger = %mountedObject.zombieSafeHouse;
						
						if(isObject(%theirTrigger) && %theirTrigger.getID() == %trigger.getID())
						{
							%mountedObject.zombieSafeHouse = "";
							%mountedObject.zombieInSafeHouse = false;
						}
					}
				}
		}
	}
}

function ZombieTrigger::onTickTrigger(%this, %trigger)
{
	for(%a = 0; %a < %trigger.getNumObjects(); %a++)
	{
		%obj = %trigger.getObject(%a);
		
		%obj.zombieInSafeHouse = true;
		%obj.zombieSafeHouse = %trigger.getID();
	}
}

// ============================================================
// Section 9 : serverCmds
// ============================================================
function serverCmdBuddy(%client, %slot, %id)
{
	if(%client.hasZAPT && %client.zombieHudCells > -1)
	{
		%slot = mFloor(%slot);
		%id = trim(%id);
		
		if(%slot > 0 && %slot <= %client.zombieHudCells && %id !$= "")
		{
			if(%id $= mFloatLength(%id, 0))
			{
				%target = findClientByBL_ID(%id);
				
				// Incase somebody has an all-number name...
				if(!isObject(%target))
				{
					%target = findClientByName(%id);
				}
			}
			else
			{
				%target = findClientByName(%id);
			}
			
			if(isObject(%target))
			{
				if(%target.getID() != %client.getID())
				{
					%canTrack = true;
					
					for(%a = 0; %a <= %client.zombieHudCells; %a++)
					{
						%buddy = %client.zombieBuddy[%a];
						
						if(isObject(%buddy) && %buddy.getID() == %target.getID())
						{
							%canTrack = false;
							messageClient(%client, '', "\c3" @ %target.name @ "<color:ffaaaa> is already being tracked.");
							break;
						}
						
						%friend = %client.zombieBuddy[%a];
						
						if(isObject(%friend) && %friend.getID() == %target.getID())
						{
							%isFriend = true;
							
							if(%a == %slot)
							{
								%isFriendInCurrSlot = true;
							}
						}
					}
					
					if(%canTrack)
					{
						if(%isFriend && !%isFriendInCurrSlot)
						{
							ZombieFriends.replaceFriend(%target);
						}
						
						%client.zombieBuddy[%slot] = %target.getID();
						%client.zombieFriend[%slot] = %target.getID();
						messageClient(%client, '', "<color:aaffaa>You will now track \"\c3" @ %target.name @ "<color:aaffaa>\" whenever possible.");
						ZombieFriends.sendFriendData(%target);
					}
				}
				else
				{
					messageClient(%client, '', "<color:ffaaaa>You can always view your own status.");
				}
			}
			else
			{
				messageClient(%client, '', "<color:ffaaaa>\"\c3" @ %id @ "<color:ffaaaa>\" yielded no results.");
			}
		}
		else
		{
			messageClient(%client, '', "<color:ffaaaa>Usage: /buddy [1 - " @ %client.zombieHudCells @ "] name/bl_id");
		}
	}
	else
	{
		messageClient(%client, '', "<color:ffaaaa>You lack the GUI to use this command.");
	}
}

function serverCmdZAPT_CellCount(%client, %cells)
{
	%cells = mFloor(%cells);
	
	if(%cells > 15)
	{
		%cells = 15;
	}
	
	%client.zombieHudCells = %cells;
}

function serverCmdZAPT_GetData(%client)
{
	if($sim::time - %client.zombieLastDataRequest >= 1)
	{
		%mini = %client.minigame;
		
		if(zombieGameSetUpRight(%mini))
		{
			ZombieFriends.sendData(%client);
			%client.zombieLastDataRequest = $sim::time;
		}
	}
}

function serverCmdZAPT_MinigameSettings(%client, %on, %options, %points, %timers)
{
	%mini = %client.minigame;
	
	if(isObject(%mini) && %mini.owner.bl_id == %client.bl_id)
	{
		%mini.ZAPT_Enabled = (%on ? ((!$ZAPT::AdminOnly || %mini.owner.isAdmin) ? true : false) : false);
		
		for(%a = 0; %a < getFieldCount(%points); %a++)
		{
			%phrase = getField(%points, %a);
			
			if(getWordCount(%phrase) >= 2)
			{
				%var = getWord(%phrase, 0);
				%val = getWords(%phrase, getWordCount(%phrase) - 1);
				
				if(mFloatLength(%val, 0) !$= %val)
				{
					%val = 0;
				}
				
				%mini.ZAPT_Points_[%var] = %val;
			}
		}
		
		for(%b = 0; %b < getFieldCount(%timers); %b++)
		{
			%phrase = getField(%timers, %b);
			
			if(getWordCount(%phrase) >= 2)
			{
				%var = getWord(%phrase, 0);
				%val = getWords(%phrase, getWordCount(%phrase) - 1);
				
				if(mFloatLength(%val, 0) !$= %val)
				{
					%val = 0;
				}
				
				%mini.ZAPT_Time_[%var] = %val;
			}
		}
		
		for(%c = 0; %c < getFieldCount(%options); %c++)
		{
			%phrase = getField(%options, %c);
			
			if(getWordCount(%phrase) >= 2)
			{
				%var = getWord(%phrase, 0);
				%val = getWords(%phrase, getWordCount(%phrase) - 1);
				
				if(mFloatLength(%val, 0) !$= %val)
				{
					%val = 0;
				}
				
				%mini.ZAPT_[%var] = %val;
			}
		}
		
		%mini.assembleZombieGame();
	}
}

function serverCmdZAPT_Register(%client, %version)
{
	%client.hasZAPT = true;
	%client.zaptVersion = %version;
}

// ============================================================
// Section 10 : Namespaceless
// ============================================================
function mDirMod(%min, %max, %mini)
{
	%valid = false;
	
	if(zombieGameSetUpRight(%mini))
	{
		if(%min < %max && %max > %min)
		{
			%stress = ZombieDirector.getStress(%mini);
			
			if(%stress > 0 && %stress < 100)
			{
				%valid = true;
			}
			else if(%stress == 0)
			{
				return %min;
			}
			else if(%stress >= 100)
			{
				return %max;
			}
		}
		else
		{
			return %max;
		}
	}
	
	if(%valid)
	{
		%mod = ((%max - %min) * (1 - (%stress / 100))) + %min;
		return %mod;
	}
	else
	{
		error("Invalid Usage of mDirMod!");
		warn("mDirMod is a ZAPT mathematical function used to \"slide\" an float.");
		warn("The outcome is dependant on the director's stress for a minigame.");
		warn("Usage: mDirMod(minimal value, maximum value, ZAPT minigame);");
	}
}

function ZAPT_UpdateAdmin(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - This server now " @ (!%newVal ? "<color:aaffaa>allows" : "<color:ffaaaa>disallows") @ " \c0non-admin zombie minigames.");
	
	if(%newVal)
	{
		for(%a = 0; %a < MiniGameGroup.getCount(); %a++)
		{
			%mini = MiniGameGroup.getObject(%a);
			
			if(!%mini.owner.isAdmin && %mini.ZAPT_Enabled)
			{
				%mini.ZAPT_Enabled = false;
				%mini.schedule(32, "messageAll", '', "\c5Sorry! This minigame is no longer a ZAPT minigame due to new server restrictions.");
			}
		}
	}
}

function ZAPT_UpdateMobSize(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - This server now allows a maximum of <color:ffffaa>" @ %newVal @ " \c0mobs.");
}

function ZAPT_UpdateMobs(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - This server now " @ (%newVal ? "<color:aaffaa>allows" : "<color:ffaaaa>disallows") @ " \c0mob zombies.");
}

function ZAPT_UpdateArmy(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - Our saviors will now be known as the <color:ffffaa>" @ %newVal @ "\c0.");
}

function ZAPT_UpdateColorcode(%oldVal, %newVal)
{
	%message = "\c3+ ZAPT\c0 - Names change color based on their status is now";
	
	if(%newVal)
	{
		schedule(0, 0, "messageAll", '', %message SPC "<color:aaffaa>enabled\c0.");
	}
	else
	{
		schedule(0, 0, "messageAll", '', %message SPC "<color:ffaaaa>disabled\c0.");
	}
}

function ZAPT_UpdateDistance(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - Zombies will now function at <color:ffffaa>" @ %newVal SPC "\c0(" @ (%newVal * 2) SPC "Studs) distance.");
}

function ZAPT_UpdateHeight(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - Safehouse triggers are now <color:ffffaa>" @ %newVal @ "\c0 flat-bricks tall.");
	ZombieSafeRoomBrick.triggerSize = "16 16" SPC $ZAPT::SaferoomHeight;
	
	%keeper = ZombieDirector.safehouseKeeper;
	
	for(%a = 0; %a < %keeper.getCount(); %a++)
	{
		%brick = %keeper.getObject(%a);
		%datablock = %brick.getDatablock();
		
		if(isObject(%datablock.triggerDatablock))
		{
			if(isObject(%brick.trigger))
			{
				%brick.trigger.delete();
			}
			
			%trigX = getWord(%datablock.triggerSize, 0);
			%trigY = getWord(%datablock.triggerSize, 1);
			%trigZ = getWord(%datablock.triggerSize, 2);
			
			if(mFloor(getWord(%brick.rotation, 3)) == 90)
			{
				%scale = (%trigY / 2) SPC (%trigX / 2) SPC (%trigZ / 2);
			}
			else
			{
				%scale = (%trigX / 2) SPC (%trigY / 2) SPC (%trigZ / 2);
			}
				
			
			%brick.trigger = new trigger()
			{
				datablock	= %datablock.triggerDatablock;
				position	= getWords(%brick.getWorldBoxCenter(), 0, 1) SPC getWord(%brick.getWorldBoxCenter(), 2) + ((getWord(%datablock.triggerSize, 2) / 4) + (%datablock.brickSizeZ * 0.1));
				rotation	= "1 0 0 0";
				scale		= %scale;
				polyhedron	= "-0.5 -0.5 -0.5 1 0 0 0 1 0 0 0 1";
				parent		= %brick;
			};
		}
	}
}

function ZAPT_UpdateThink(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ ZAPT\c0 - Zombies will now think every <color:ffffaa>" @ %newVal @ "ms\c0.");
}

function zombieBot(%obj)
{
	if(isObject(%obj))
	{
		if(%obj.getClassName() $= "AIConnection")
		{
			return true;
		}
		else if(%obj.getClassName() $= "GameConnection")
		{
			return false;
		}
		else
		{
			%client = %obj.client;
			
			if(!isObject(%client) || %client.getClassName() !$= "GameConnection")
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}
	
	return false;
}

function zombieFriendlyFire(%obj1, %obj2)
{
	// The intent on this function is to find the source player of all objects
	// then determine if this is an instance of Friendly Fire (in the ZAPT system).
	
	// Are we even trying?
	if(isObject(%obj[1]) && isObject(%obj[2]))
	{
		// Loop through both objects to get the source objects.
		for(%i = 1; isObject(%obj[%i]); %i++)
		{
			// Don't assign this an [%i] to save memory.
			%className = %obj[%i].getClassName();
			
			// Get out client based on object type.
			switch$(%className)
			{
				case "AIPlayer":		%player[%i] = %obj[%i]; %isZombie[%i] = (%obj[%i].isZombie ? true : false);
				case "FlyingVehicle":	%client[%i] = %obj[%i].getControllingClient();
				case "GameConnection":	%client[%i] = %obj[%i];
				case "Player":			%player[%i] = %obj[%i]; %client[%i] = %obj[%i].client;
				case "Projectile":		%client[%i] = %obj[%i].client;
				case "WheeledVehicle":	%client[%i] = %obj[%i].getControllingClient();
			}
			
			// No matter where we got the client, the player is always just a property away.
			if(%isZombie[%i] $= "" && isObject(%client[%i]))
			{
				if(isObject(%client[%i].player))
				{
					%player[%i] = %client[%i].player;
					%isZombie[%i] = (%client[%i].player.isZombie ? true : false);
				}
				else
				{
					if(%client[%i].isZombie != %client[%i].zombieDiedAs)
					{
						%isZombie[%i] = (%client[%i].zombieDiedAs ? true : false);
					}
					else
					{
						%isZombie[%i] = (%client[%i].isZombie ? true : false);
					}
				}
			}
			
			if(%isZombie[%i] $= "")
			{
				%isZombie[%i] = false;
			}
		}
		
		if(isObject(%player[1]) && isObject(%player[2]) && %player[1].getID() == %player[2].getID())
		{
			return false;
		}
		else if(%isZombie[1] == %isZombie[2])
		{
			return true;
		}
	}
	
	return false;
}

function zombieSameGame(%obj1, %obj2)
{
	if(isObject(%obj1) && isObject(%obj2))
	{
		%obj1mini = getMiniGameFromObject(%obj1);
		
		if(isObject(%obj1mini))
		{
			%obj2mini = getMiniGameFromObject(%obj2);
			
			if(isObject(%obj2mini) && %obj1mini.getID() == %obj2mini.getID())
			{
				return true;
			}
		}
	}
	
	return false;
}

function zombieGameSetUpRight(%mini)
{
	// Are we even trying?
	if(isObject(%mini) && %mini.ZAPT_Enabled && (!$ZAPT::AdminOnly || %mini.owner.isAdmin))
	{
		// Really, the above is all that really matters in a standard install.
		// However, if we're running TDM we need to check for gamemodes.
		if(isObject(GameModeStorerSO) && %mini.gameMode !$= "")
		{
			%gameMode = GameModeStorerSO.modeShortTitle[%mini.gameMode];
			
			if(%gameMode $= "ZAPT")
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			return true;
		}
	}
	else
	{
		return false;
	}
}

function zombieValidSpawnBrick(%brick)
{
	if(isObject(%brick))
	{
		%db = %brick.getDatablock();
		
		if(%db.isZombieSpawn)
		{
			%mini = getMiniGameFromObject(%brick);
			
			if(zombieGameSetUpRight(%mini))
			{
				return true;
			}
		}
	}
	
	return false;
}

function zombieValidTarget(%obj, %target)
{
	if(isObject(%obj) && isObject(%target))
	{
		if(%obj.isZombie && %obj.getType() & $TypeMasks::PlayerObjectType)
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(zombieGameSetUpRight(%mini) && (miniGameCanDamage(%target, %obj) || miniGameCanDamage(%obj, %target)) && zombieSameGame(%mini.owner, %obj))
			{
				if((!%target.isZombie || %target.zombieBeacon) && %target.getType() & ($TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::ProjectileObjectType))
				{
					return true;
				}
			}
		}
	}
	
	return false;
}

// ============================================================
// Section 11 : Objects
// ============================================================
// Beacon
function ZombieBeaconProjectile::radiusDamage(%this, %obj, %col, %idk, %pos, %damageType)
{
	%source = %obj.sourceObject;
	
	if(zombieValidTarget(%source, %col) && %col.getType() & $TypeMasks::PlayerObjectType)
	{
		// Prevents the blast from hitting an object through a wall or whatever.
		%start = %obj.getWorldBoxCenter();
		%end = %col.zombieMidPoint();
		%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
		%hit = getWord(containerRayCast(%start, %end, %masks, %source), 0);
		
		if(isObject(%hit) && %hit.getID() == %col.getID())
		{
			if(!zombieBot(%source))
			{
				if(!zombieBot(%col) && !%col.zombieBeacon)
				{
					messageClient(%source.client, '', "<color:ffffaa>" @ %col.client.name @ " was hit by your bile. They are a zombie beacon.");
					messageClient(%col.client, '', "<color:ffffaa>" @ %source.client.name @ " exploded on you. You are now a zombie beacon.");
				}
				else
				{
					messageClient(%source.client, '', "<color:ffffaa>A survivor was hit by your bile. They are a zombie beacon.");
				}
			}
			else if(!zombieBot(%col) && !%col.zombieBeacon)
			{
				messageClient(%col.client, '', "<color:ffffaa>A consumer exploded on you. You are now a zombie beacon.");
			}
			
			%col.zombieBeacon(10000, isObject(%source.client) ? %source.client : -1);
			
			if(isObject(%mini = getMiniGameFromObject(%col)))
			{
				%mini.panic();
			}
		}
	}
}

// Bile
function ZombieBileProjectile::onCollision(%this, %obj, %col, %idk, %pos, %dir, %wtf)
{
	%source = %obj.sourceObject;
	
	if(zombieValidTarget(%source, %col) && %col.getType() & $TypeMasks::PlayerObjectType)
	{
		if(!zombieBot(%source))
		{
			if(!zombieBot(%col))
			{
				messageClient(%source.client, '', "<color:ffffaa>" @ %col.client.name @ " was hit by your bile. They are a zombie beacon.");
				
				if(!%col.zombieBeacon)
				{
					messageClient(%col.client, '', "<color:ffffaa>" @ %source.client.name @ " puked on you. You are now a zombie beacon.");
				}
			}
			else
			{
				messageClient(%source.client, '', "<color:ffffaa>A survivor was hit by your bile. They are a zombie beacon.");
			}
		}
		else if(!zombieBot(%col) && !%col.zombieBeacon)
		{
			messageClient(%col.client, '', "<color:ffffaa>A consumer puked on you. You are now a zombie beacon.");
		}
		
		%col.zombieBeacon(10000, isObject(%source.client) ? %source.client : -1);
		
		if(isObject(%mini = getMiniGameFromObject(%col)))
		{
			%mini.panic();
		}
	}
}

// Add-Ons
function ZombieClawRightImage::onFire(%this, %obj, %slot)
{
	%obj.zombieClaw();
}

function ZombieFistRightImage::onFire(%this, %obj, %slot)
{
	%obj.zombieClaw();
}

function ZombieFistSmasherImage::onFire(%this, %obj, %slot)
{
	%obj.zombieClaw();
}

function ZombieFistChargingImage::onFire(%this, %obj, %slot)
{
	// Do nothing.
}

// Pipebomb
function ZombiePipebombImage::onCharge(%this, %obj, %slot)
{
	%obj.playThread(2, "spearReady");
	%obj.zombieMolotovSlot = %obj.currTool;
}

function ZombiePipebombImage::onChargeAborted(%this, %obj, %slot)
{
	%obj.playThread(2, "ready");
}

function ZombiePipebombImage::onDone(%this, %obj, %slot)
{
	%obj.unMountImage(%slot);
}

function ZombiePipebombImage::onFire(%this, %obj, %slot)
{
	%obj.playthread(2, "spearThrow");
	%molotovSlot = %obj.zombieMolotovSlot;
	%obj.zombieMolotovSlot = -1;
	%client = %obj.client;
	
	%proj = parent::onFire(%this, %obj, %slot);
	
	%obj.tool[%molotovSlot] = 0;
	%obj.weaponCount--;
	
	if(isObject(%client))
	{
		messageClient(%client, 'MsgItemPickup', '', %molotovSlot, 0);
		serverCmdUnUseTool(%client);
	}
	
	if(isObject(%proj))
	{
		ZombieDirector.distractKeeper.add(%proj);
		%proj.zombieDistract = true;
		%proj.zombieNoiseSched = %proj.getDatablock().makeNoise(%proj, 1500, 0);
	}
	
	return %proj;
}

function ZombiePipebombProjectile::makeNoise(%this, %obj, %time, %spam)
{
	if(isObject(%obj))
	{
		// Noise
		serverPlay3D(Beep_EKG_Sound, %obj.getTransform());
		
		%nextTime = mFloor(%time * 0.75);
		
		if(%nextTime < 150)
		{
			%spam++;
			
			if(%spam >= 10)
			{
				%obj.explode();
				return;
			}
			
			%nextTime = 150;
		}
		
		initContainerRadiusSearch(%obj.getPosition(), 64, $TypeMasks::PlayerObjectType);
		%col = containerSearchNext();
		
		while(isObject(%col))
		{
			if(%col.isZombie && miniGameCanDamage(%obj, %col))
			{
				%client = %col.client;
				
				if(isObject(%client))
				{
					if(!%col.zombieBlindedBy[%obj.getID()])
					{
						%col.zombieBlindedBy[%obj.getID()] = true;
						messageClient(%client, '', "<color:aaffff>That noise! What is that NOISE?");
					}
					
					%dist = vectorDist(%col, %obj);
					
					if(%dist <= 32)
					{
						%col.setWhiteOut(1);
					}
					else
					{
						%col.setWhiteOut(0.75 + (0.25 * (64 - %dist) / 32));
					}
				}
			}
			
			%col = containerSearchNext();
		}
		
		cancel(%obj.zombieNoiseSched);
		%obj.zombieNoiseSched = %this.schedule(%time, "makeNoise", %obj, %nextTime, %spam);
	}
}

function ZombiePipebombProjectile::onCollision(%this, %obj, %col, %fade, %pos, %normal)
{
	// Oddly enough, there is no engine-built way to have something not explode on impact.
	// Overriding this function dose the trick, though.
}

// Safehouse
function ZombieSafeRoomBrick::onDeath(%this, %brick)
{
	if(isObject(%brick.trigger))
	{
		%brick.trigger.delete();
	}
	
	ZombieDirector.safehouseKeeper.remove(%brick);
	
	return parent::onDeath(%brick);
}

function ZombieSafeRoomBrick::onPlant(%this, %brick)
{
	%result = parent::onPlant(%this, %brick);
	
	if(isObject(%brick))
	{
		%datablock = %brick.getDatablock();
		
		if(isObject(%datablock.triggerDatablock))
		{
			if(isObject(%brick.trigger))
			{
				%brick.trigger.delete();
			}
			
			%trigX = getWord(%datablock.triggerSize, 0);
			%trigY = getWord(%datablock.triggerSize, 1);
			%trigZ = getWord(%datablock.triggerSize, 2);
			
			if(mFloor(getWord(%brick.rotation, 3)) == 90)
			{
				%scale = (%trigY / 2) SPC (%trigX / 2) SPC (%trigZ / 2);
			}
			else
			{
				%scale = (%trigX / 2) SPC (%trigY / 2) SPC (%trigZ / 2);
			}
				
			
			%brick.trigger = new trigger()
			{
				datablock	= %datablock.triggerDatablock;
				position	= getWords(%brick.getWorldBoxCenter(), 0, 1) SPC getWord(%brick.getWorldBoxCenter(), 2) + ((getWord(%datablock.triggerSize, 2) / 4) + (%datablock.brickSizeZ * 0.1));
				rotation	= "1 0 0 0";
				scale		= %scale;
				polyhedron	= "-0.5 -0.5 -0.5 1 0 0 0 1 0 0 0 1";
				parent		= %brick;
			};
		}
		
		ZombieDirector.safehouseKeeper.add(%brick);
	}
	
	return %result;
}

function ZombieSafeRoomBrick::onRemove(%this, %brick)
{
	if(isObject(%brick.trigger))
	{
		%brick.trigger.delete();
	}
	
	ZombieDirector.safehouseKeeper.remove(%brick);
	
	return parent::onRemove(%brick);
}

// Spawns
function ZombieSpawn::onPlant(%this, %brick)
{
	%result = parent::onPlant(%this, %brick);
	
	if(isObject(%brick))
	{
		ZombieKeeper.add(%brick);
	}
	
	return %result;
}

function ZombieSpawnMob::onPlant(%this, %brick)
{
	parent::onPlant(%this, %brick);
	
	if(isObject(%brick))
	{
		%brick.hordeDatablock = ZombieStandardArmor;
		ZombieKeeperMob.add(%brick);
	}
}

function ZombieSpawnTank::onPlant(%this, %brick)
{
	parent::onPlant(%this, %brick);
	
	if(isObject(%brick))
	{
		ZombieKeeperTank.add(%brick);
	}
}

// ============================================================
// Section 12 : Events
// ============================================================
if(!$ZAPT_Temp::RegisteredEvents)
{
	registerInputEvent("fxDTSBrick", "onHumanActivate", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onHumanTouch", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onMinigameRand1", "Self fxDTSBrick" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onMinigameRand2", "Self fxDTSBrick" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onZombieActivate", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onZombieKill", "Self fxDTSBrick" TAB "Bot Player" TAB "Client GameConnection" TAB "MiniGame MiniGame"TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onZombieKilled", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onZombieSpawned", "Self fxDTSBrick" TAB "Bot Player" TAB "MiniGame MiniGame"TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	registerInputEvent("fxDTSBrick", "onZombieTouch", "Self fxDTSBrick" TAB "Player Player" TAB "Client GameConnection" TAB "MiniGame MiniGame" TAB "OwnerPlayer Player" TAB "OwnerClient GameConnection");
	
	registerOutputEvent("GameConnection", "Rescue", "", false);
	registerOutputEvent("GameConnection", "SpawnAsTank", "", false);
	
	registerOutputEvent("Minigame", "DisableZombies", "bool", false);
	registerOutputEvent("Minigame", "Escape", "", false);
	registerOutputEvent("Minigame", "Panic", "", false);
	registerOutputEvent("Minigame", "PickTank", "", false);
	registerOutputEvent("Minigame", "SafehouseVictory", "bool", false);
	
	registerOutputEvent("Player", "AddItems", "datablock ItemData" TAB "int 1 5 1", false);
	registerOutputEvent("Player", "BurnLoop", "int 1 60 5" TAB "int 1 100 5", false);
	registerOutputEvent("Player", "HealLoop", "float 100 2500 100 500" TAB "int 1 20 5" TAB "int 20 100 100", false);
	
	ZombieEvents.addEvent("onMinigameRand1");
	ZombieEvents.addEvent("onMinigameRand2");
	
	if(isObject(ContentRestrictionsSO))
	{
		ContentRestrictionsSO.addRestriction("Zombie", "contentRestrictionZombie", 10101649);
		ContentRestrictionsSO.addRestriction("Human", "contentRestrictionHuman", 10201649);
	}
	
	$ZAPT_Temp::RegisteredEvents = true;
}