// ============================================================
// Project				:	ZAPT
// Author				:	Iban
// Description			:	Packaged Default Functions
//								Kept in a seperate file so
//								I can collapse them easily.
// ============================================================
// Table of Contents
// 
// 1. AIConnection
// 2. Armor
// 3. fxDTSBrick
// 4. GameConnection
// 5. MinigameSO
// 6. Namespaceless
// 7. Observer
// 8. Player
// 9. ProjectileData
// 10. serverCmds
// 11. Vehicle
// 12. WeaponImage
// ============================================================
package ZAPT_Server
{
	// ============================================================
	// Section 1 : AI*
	// ============================================================
	// AIConnection
	function AIConnection::onDrop(%client, %reason)
	{
		if(%client.zombieSmasherAI)
		{
			%client.onClientLeaveGame();
			removeFromServerGuidList(%client.guid);
			removeTaggedString(%client.name);
		}
		else
		{
			return Parent::onDrop(%client, %reason);
		}
	}
	
	// AIPlayer
	function AIPlayer::changeDatablock(%obj, %db)
	{
		if(%obj.isZombie == %db.isZombieArmor)
		{
			return parent::changeDatablock(%obj, %db);
		}
	}
	
	// ============================================================
	// Section 2 : Armor
	// ============================================================
	function Armor::Damage(%this, %obj, %sourceObject, %pos, %damage, %damageType)
	{
		if(isObject(%obj) && %obj.getType() & $TypeMasks::PlayerObjectType)
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(zombieGameSetUpRight(%mini))
			{
				%objClient		= %obj.client;
				%sourceClient	= %sourceObject.client;
				%sourcePlayer	= %sourceClient.player;
				%spawnBrick		= %sourceObject.spawnBrick;
				%bot			= zombieBot(%obj);
				
				// Friendly Fire
				if(zombieFriendlyFire(%sourceObject, %obj) && !%obj.isZombie)
				{
					%damage = mFloor(%damage / 5);
				}
				// Headshot Damage - by SpaceGuy
				else if(%obj.isZombie && fileName(%obj.dataBlock.shapeFile) $= "m.dts")
				{
					%fVec = %obj.getForwardVector();
					%vec1 = -getWord(%fVec, 1) SPC getWord(%fVec, 0) SPC 0;	
					%vec2 = vectorNormalize(%sourceObject.initialVelocity);
					
					%damLoc = vectorDot(%vec1, %vec2) > 0;
					%scaleZ = getWord(%obj.getScale(), 2);
					
					if(getWord(%pos, 2) > getWord(%obj.getWorldBoxCenter(), 2) - (3.3 * %scaleZ))
					{
						%damage *= 1.5;
						%headshot = true;
					}
				}
				
				// Target Update
				if(%bot && !%sourcePlayer.isZombie && !%obj.target.zombieDistract)
				{
					%obj.zombieChase(%sourcePlayer);
				}
				
				%result = parent::Damage(%this, %obj, %sourceObject, %pos, %damage, %damageType);
				
				// Post-Death Management
				if(%obj.getState() $= "Dead")
				{
					// Awards
					if(%obj.isZombie)
					{
						if(%bot)
						{
							%award = "KillBot";
						}
						else
						{
							%award = "KillPlayer";
						}
					}
					else
					{
						if(%sourceObject.isZombie || %sourceClient.isZombie)
						{
							%award = "EatingPlayer";
							%objAward = "GettingEaten";
						}
						else
						{
							%award = "KillTeam";
						}
					}
					
					if(isObject(%sourceClient) && %award !$= "")
					{
						%sourceClient.incScore(%mini.ZAPT_Points_[%award]);
					}
					
					if(isObject(%objClient) && %objAward !$= "")
					{
						%objClient.incScore(%mini.ZAPT_Points_[%award]);
					}
					
					if(%obj.isZombie != %sourceObject.isZombie)
					{
						if(!%obj.isZombie && isObject(%sourceObject.spawnBrick))
						{
							%sourceObject.spawnBrick.onZombieKill(%sourceObject, %objClient);
						}
					}
					
					if(%obj.isZombie)
					{
						// Decapitation
						if(%headshot)
						{
							%hideStr = "headskin";
							
							for(%i = 0; $hat[%i] !$= ""; %i++)
							{
								%hideStr = %hideStr TAB $hat[%i];
							}
							
							for(%i = 0; $accent[%i] !$= ""; %i++)
							{
								%hideStr = %hideStr TAB $accent[%i];
							}
							
							for(%i = 0; %i < getFieldCount(%hideStr); %i++)
							{
								%node = getField(%hideStr, %i);
								%obj.hideNode(%node);
							}
						}
						
						if(isObject(%spawnBrick))
						{
							%spawnBrick.onZombieKilled(%sourceObject.client);
						}
						
						%this.zombieDeath(%obj, %objClient, %sourceObject, %sourceClient, %bot, %pos, %damage, %damageType);
					}
					else
					{
					
					}
				}
				
				return %result;
			}
		}
		
		return parent::Damage(%this, %obj, %sourceObject, %pos, %damage, %damageType);
	}
	
	function Armor::onCollision(%this, %obj, %col, %pos, %vel)
	{
		%result = parent::onCollision(%this, %obj, %col, %pos, %vel);
		
		%mini = getMiniGameFromObject(%obj);
		
		if(zombieGameSetUpRight(%mini))
		{
			if(zombieBot(%obj) && zombieValidTarget(%obj, %col))
			{
				if(%col.getType() & $TypeMasks::VehicleObjectType)
				{
					if(%col.getMountedObjectCount() > 0)
					{
						if(!%obj.zombieHasTarget() || (%obj.target.getID() != %col.getID() && !%obj.target.zombieDistract))
						{
							%obj.zombieChase(%col);
						}
						
						%obj.zombieClaw();
					}
					else if(%obj.zombieHasTarget() && %obj.target.getID() == %col.getID() && !%col.zombieDistract)
					{
						%obj.zombieChase(-1);
					}
				}
				else
				{
					if(!%obj.zombieHasTarget() || (%obj.target.getID() != %col.getID() && !%obj.target.zombieDistract))
					{
						%obj.zombieChase(%col);
					}
					
					%obj.zombieClaw();
				}
			}
			else if(!%obj.isZombie && !%obj.zombieHelpless && !%obj.zombieSmoked && %obj.getState() !$= "Dead" &&
				!%col.isZombie && %col.zombieSmoked && %col.getState() !$= "Dead")
			{
				if(isObject(%col.zombieSmokedBy.zombieSmokingWho) && %col.zombieSmokedBy.zombieSmokingWho.getID() == %col.getID())
				{
					if(!zombieBot(%col.zombieSmokedBy))
					{
						if(!zombieBot(%obj.client) && !zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3" @ %obj.client.name @ "<color:ffaaaa> broke your strangle on \c3" @ %col.client.name @ "<color:ffaaaa>!");
						}
						else if(!zombieBot(%obj.client) && zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3A survivor <color:ffaaaa>broke your strangle on \c3" @ %col.client.name @ "<color:ffaaaa>!");
						}
						else if(zombieBot(%obj.client) && !zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3" @ %obj.client.name @ "<color:ffaaaa> broke your strangle on \c3another survivor<color:ffaaaa>!");
						}
						else
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3A survivor <color:ffaaaa>broke your strangle on \c3another survivor<color:ffaaaa>!");
						}
					}
					
					if(isObject(%col.getMountedImage(2)) && %col.getMountedImage(2).getID() == ZombieSmokerConstrictImage.getID())
					{
						%col.unMountImage(2);
					}
					
					%col.zombieHelpless = false;
					cancel(%col.zombieSmokedBy.zombieStrangleSched);
					%col.zombieSmoked = false;
					%col.zombieSmokedBy = "";
					
					if(!zombieBot(%obj.client))
					{
						if(!zombieBot(%col.client))
						{
							messageClient(%obj.client, '', "<color:aaffaa>You rescued \c3" @ %col.client.name @ "<color:aaffaa>!");
						}
						else
						{
							messageClient(%obj.client, '', "<color:aaffaa>You rescued \c3a survivor<color:aaffaa>!");
						}
					}
					
					if(!zombieBot(%col.client))
					{
						if(!zombieBot(%obj.client))
						{
							messageClient(%col.client, '', "<color:aaffaa>You were rescued by \c3" @ %obj.client.name @ "<color:aaffaa>!");
						}
						else
						{
							messageClient(%col.client, '', "<color:aaffaa>You were rescued<color:aaffaa>!");
						}
					}
				}
			}
		}
		
		return %result;
	}
	
	function Armor::onDamage(%this, %obj, %damage)
	{
		%result = parent::onDamage(%this, %obj, %damage);
		
		// Health
		if(!zombieBot(%obj))
		{
			ZombieFriends.sendFriendData(%obj.client);
		}
		
		return %result;
	}
	
	function Armor::onDisabled(%this, %obj)
	{
		%obj.zombieJockeyDismount();
		return parent::onDisabled(%this, %obj);
	}
	
	function Armor::onEnterLiquid(%this, %obj)
	{
		if(isEventPending(%obj.damageBurnSched))
		{
			cancel(%obj.damageBurnSched);
			%obj.damageBurnLoops = 0;
			%obj.damageBurnDamage = 0;
			%obj.unMountImage(3);
		}
		
		%obj.zombieInWater = true;
		return parent::onEnterLiquid(%this, %obj);
	}
	
	function Armor::onImpact(%this, %obj, %col, %pos, %vel)
	{
		if(!%obj.isZombie && !%obj.zombieSmoked)
		{
			return parent::onImpact(%this, %obj, %col, %pos, %vel);
		}
	}
	
	function Armor::onLeaveLiquid(%this, %obj)
	{
		%obj.zombieInWater = false;
		return parent::onLeaveLiquid(%this, %obj);
	}
	
	function Armor::onNewDatablock(%this, %obj)
	{
		%result = parent::onNewDatablock(%this, %obj);
		cancel(%obj.client.zombieClassVerify);
		
		if(%this.isZombieArmor)
		{
			%obj.isZombie = true;
			%obj.playThread(2, "ArmReadyBoth");
			
			%obj.mountImage(ZombieJumpHack, 2);
			%obj.mountImage(ZombieCrouchHack, 3);
			%obj.mountImage(ZombieSpecialHack, 4);
			
			%obj.setShapeNameColor("0.00 0.45 0.45 1.00");
			
			if(%this.zombieScale)
			{
				%obj.setScale(%this.zombieScale);
			}
			
			%obj.unMountImage($WeaponSlot);
			%obj.playThread(0, "root");
			%obj.playThread(1, "root");
			%obj.playThread(2, "root");
			
			if(!zombieBot(%obj))
			{
				%obj.client.applyBodyParts();
				%obj.client.applyBodyColors();
				
				if(%this.zombieUIName !$= "")
				{
					%font = "<just:right><font:impact:32>";
					%help = %this.zombieHelp;
					
					if(%help !$= "")
					{
						%help = "\n<font:impact:20>(" @ %help @ ")";
					}
					
					if(%this.zombieNoSwap || getSimTime() - %obj.spawnTime > 2500)
					{
						%obj.client.schedule(32, "centerPrint", %font @ "<color:aaffaa>You are" SPC %this.zombieUIName @ %help, 3);
						cancel(%obj.client.zombieClassVerify);
					}
					else
					{
						%obj.client.schedule(32, "centerPrint", %font @ "<color:ffaaaa>You are" SPC %this.zombieUIName @ %help, 3);
						%obj.client.zombieClassVerify = %obj.client.schedule(2500, "centerPrint", %font @ "<color:aaffaa>You are" SPC %this.zombieUIName @ %help, 1);
					}
				}
				
				if(zombieGameSetUpRight(getMiniGameFromObject(%obj)))
				{
					%obj.client.isZombie = true;
				}
			}
			else
			{
				%obj.name = "Zombie";
				
				applyDefaultCharacterPrefs(%obj);
			}
		}
		else if(%obj.isZombie)
		{
			%obj.isZombie = false;
			%obj.playThread(2, "root");
			
			if(!zombieBot(%obj))
			{
				%obj.client.applyBodyParts();
				%obj.client.applyBodyColors();
			}
			else
			{
				applyDefaultCharacterPrefs(%obj);
			}
		}
		
		%obj.zombieLight();
		
		if(!zombieBot(%obj))
		{
			commandToClient(%obj.client, 'showEnergyBar', %this.showEnergyBar);
			
			if(!%obj.isZombie)
			{
				%obj.zombieItemCheck();
			}
		}
		
		for(%a = 0; %a < 7; %a++)
		{
			switch(%a)
			{
				case 0:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieSmokeImage");
				case 1:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieClawRightImage");
				case 2:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieClawRightLeft");
				case 3:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieFistLeftImage");
				case 4:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieFistRightImage");
				case 5:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieFistSmasherImage");
				case 6:
					%armor = ZombieSmokerArmor.getID();
					%slot = %obj.getMountSlot("ZombieFistChargingImage");
			}
			
			if(%this.getID() != %armor.getID() && %slot > -1)
			{
				%obj.unMountImage(%slot);
			}
		}
		
		return %result;
	}
	
	function Armor::onReachDestination(%this, %obj)
	{
		if(%this.isZombieArmor && isObject(%obj.target))
		{
			%obj.zombieChase(%obj.target);
		}
		
		return parent::onReachDestination(%this, %obj);
	}
	
	function Armor::onTrigger(%this, %obj, %trigger, %value)
	{
		if(%obj.isZombie && %value)
		{
			switch(%trigger)
			{
				// Left Click
				case 0:
					return parent::onTrigger(%this, %obj, %trigger, %value);
				// ???
				case 1:
					return parent::onTrigger(%this, %obj, %trigger, %value);
				// Jump
				case 2:
					return parent::onTrigger(%this, %obj, %trigger, %value);
				// Crouch
				case 3:
					return parent::onTrigger(%this, %obj, %trigger, %value);
				// Right Click
				case 4:
					%timeSinceSpawn = (getSimTime() * 0.001) - (%obj.spawnTime * 0.001);
					
					if(!zombieBot(%obj) && !%this.zombieNoSwap && %timeSinceSpawn >= 0.5 && %timeSinceSpawn <= 2.5)
					{
						%obj.client.zombieDidNotWant = %this.getID();
						cancel(%obj.client.zombieClassVerify);
						return %obj.client.schedule(32, "spawnPlayer");
					}
					else if(%timeSinceSpawn > 2.5)
					{
						return %obj.zombieSpecial();
					}
					
				default:
					return parent::onTrigger(%this, %obj, %trigger, %value);
			}
		}
		else if(!%obj.zombieSmoked)
		{
			return parent::onTrigger(%this, %obj, %trigger, %value);
		}
	}
	
	// ============================================================
	// Section 3 : fxDTSBrick
	// ============================================================
	function fxDTSBrick::colorVehicle(%this)
	{
		%result = parent::colorVehicle(%this);
		
		if(zombieBot(%this.vehicle.isZombie))
		{
			applyDefaultCharacterPrefs(%this.vehicle);
		}
		
		return %result;
	}
	
	function fxDTSBrick::onActivate(%this, %obj, %client)
	{
		if(!%obj.zombieHelpless)
		{
			if(!zombieBot(%obj))
			{
				if(%obj.isZombie)
				{
					%this.onZombieActivate(%client);
				}
				else
				{
					%this.onHumanActivate(%client);
				}
			}
			
			return parent::onActivate(%this, %obj, %client);
		}
	}
	
	function fxDTSBrick::onPlayerTouch(%this, %obj)
	{
		if(isObject(%obj) && %obj.getType() & $TypeMasks::PlayerObjectType)
		{
			if(%obj.isZombie)
			{
				%this.onZombieTouch(%obj);
			}
			else
			{
				%this.onHumanTouch(%obj);
			}
		}
		
		return parent::onPlayerTouch(%this, %obj);
	}
	
	function fxDTSBrick::setReColorVehicle(%this, %bool)
	{
		%result = parent::setReColorVehicle(%this, %bool);
		
		if(%this.vehicle.isZombie)
		{
			applyDefaultCharacterPrefs(%this.vehicle);
		}
		
		return %result;
	}
	
	function fxDTSBrick::spawnProjectile(%this, %vel, %db, %var, %scale, %client)
	{
		%result = parent::spawnProjectile(%this, %vel, %db, %var, %scale, %client);
		
		if(isObject(%result) && %db.zombieDistraction)
		{
			ZombieDirector.distractKeeper.add(%result);
			%result.zombieDistract = true;
			
			if(%db.zombieMakeNoise)
			{
				%result.zombieNoiseSched = %db.makeNoise(%result, 1500, 0);
			}
		}
		
		return %result;
	}
	
	function fxDTSBrick::spawnVehicle(%this, %vehicle)
	{
		if(%this.vehicleDatablock.isZombieArmor)
		{
			if(%this.zombieCanSpawn())
			{
				%result = parent::spawnVehicle(%this, %vehicle);
				
				if(isObject(%this.vehicle))
				{
					%this.zombieSpawnTime = $sim::time;
					%this.onZombieSpawned(%this.vehicle);
				}
			}
		}
		else
		{
			%result = parent::spawnVehicle(%this, %vehicle);
		}
		
		return %result;
	}
	
	// ============================================================
	// Section 4 : GameConnection
	// ============================================================
	function GameConnection::applyBodyColors(%client)
	{
		%result = parent::applyBodyColors(%client);
		
		%obj = %client.player;
		
		if(isObject(%obj) && %obj.isZombie)
		{
			// Hides visors to make zombie colored faces visible.
			if(%client.hat == 1 && %client.accent == 1)
			{
				%obj.hideNode("Visor");
			}
			
			// Goes through arms, hands, feet, hips, and chest checking for a color match to face.
			%skin = %client.headColor;
			%zSkin = %obj.getDatablock().getSkinColor(%obj);
			
			%obj.setNodeColor("headskin", %zSkin);
			
			if(%client.rArmColor $= %skin)
			{
				%obj.setNodeColor($RArm[0], %zSkin);
				%obj.setNodeColor($RArm[1], %zSkin);
			}
			
			if(%client.lArmColor $= %skin)
			{
				%obj.setNodeColor($LArm[0], %zSkin);
				%obj.setNodeColor($LArm[1], %zSkin);
			}
			
			if(%client.rHandColor $= %skin)
			{
				%obj.setNodeColor($RHand[0], %zSkin);
				%obj.setNodeColor($RHand[1], %zSkin);
			}
			
			if(%client.lHandColor $= %skin)
			{
				%obj.setNodeColor($LHand[0], %zSkin);
				%obj.setNodeColor($LHand[1], %zSkin);
			}
			
			if(%client.chestColor $= %skin)
			{
				%obj.setNodeColor($Chest[0], %zSkin);
				%obj.setNodeColor($Chest[1], %zSkin);
			}
			
			if(%client.hipColor $= %skin)
			{
				%obj.setNodeColor($Hip[0], %zSkin);
				%obj.setNodeColor($Hip[1], %zSkin);
			}
			
			if(%client.rLegColor $= %skin)
			{
				%obj.setNodeColor($RLeg[0], %zSkin);
				%obj.setNodeColor($RLeg[1], %zSkin);
			}
			
			if(%client.lLegColor $= %skin)
			{
				%obj.setNodeColor($LLeg[0], %zSkin);
				%obj.setNodeColor($LLeg[1], %zSkin);
			}
			
			if(%obj.zombieDecalFace $= "")
			{
				%obj.zombieDecalFace = getRandom(0, 7);
				
				// Face Paint
				switch(%obj.zombieDecalFace)
				{
					case 0: %obj.zombieDecalFace = "ZombieFaceA";
					case 1: %obj.zombieDecalFace = "ZombieFaceB";
					case 2: %obj.zombieDecalFace = "ZombieFaceC";
					case 3: %obj.zombieDecalFace = "ZombieFaceD";
					case 4: %obj.zombieDecalFace = "ZombieFaceE";
					case 5: %obj.zombieDecalFace = "ZombieFaceF";
					case 6: %obj.zombieDecalFace = "ZombieFaceG";
					case 7: %obj.zombieDecalFace = "ZombieFaceI";
					default: %obj.zombieDecalFace = "smiley";
				}
			}
			
			%obj.setFaceName(%obj.zombieDecalFace);
		}
		
		return %result;
	}
	
	function GameConnection::applyBodyParts(%client)
	{
		%result = parent::applyBodyParts(%client);
		
		%obj = %client.player;
		
		if(isObject(%obj) && %obj.isZombie)
		{
			// Hides visors to make zombie colored faces visible.
			if(%client.hat == 1 && %client.accent == 1)
			{
				%obj.hideNode("Visor");
			}
			
			%db = %obj.getDatablock();
			
			for(%a = 1; %a <= %db.zombieHideNodes; %a++)
			{
				%obj.hideNode(%db.zombieHideNode[%a]);
			}
			
			%obj.hideNode("armor");
			
			for(%a = 1; %a <= %db.zombieShowNodes; %a++)
			{
				%obj.unHideNode(%db.zombieShowNode[%a]);
			}
			
			%db.zombieDress(%obj);
		}
		
		return %result;
	}
	
	function GameConnection::autoAdminCheck(%client)
	{
		%client.hasZAPT = false;
		%client.zaptVersion = 0;
		%client.zombieHudCells = -1;
		
		commandToClient(%client, 'ZAPT_MOD_Resgister', $ZAPT::Version, ZombieMutations.mutationList());
		return parent::autoAdminCheck(%client);
	}
	
	function GameConnection::createPlayer(%client, %pos)
	{
		%result = parent::createPlayer(%client, %pos);
		
		if(zombieGameSetUpRight(%client.minigame))
		{
			if(%client.isZombie)
			{
				if(%client.minigame.ZAPT_PlayerZombies)
				{
					%obj = %client.player;
					
					if(isObject(%obj))
					{
						%spawnBrick = %client.zombieSpawnBrick;
						
						if(isObject(%spawnBrick))
						{
							%client.zombieSpawnBrick = -1;
							%vdb = %spawnBrick.vehicleDataBlock;
							
							if(isObject(%vdb) && %vdb.isZombieArmor)
							{
								%obj.setDatablock(%vdb);
							}
							else
							{
								%obj.setDatablock(ZombieStandardArmor);
							}
						}
						else
						{
							%obj.setDatablock(ZombieStandardArmor);
						}
						
						%client.applyBodyParts();
						%client.applyBodyColors(%client);
					}
				}
				else
				{
					%client.isZombie = false;
				}
			}
		}
		
		return %result;
	}
	
	function GameConnection::onClientEnterGame(%client)
	{
		return parent::onClientEnterGame(%client);
	}
	
	function GameConnection::onDeath(%client, %sourceObject, %sourceClient, %damageType, %damageArea)
	{
		%mini = %client.minigame;
		
		if(zombieGameSetUpRight(%mini))
		{
			%client.zombieDiedAs = %client.isZombie;
			
			if(!%client.isZombie)
			{
				%mini.zombieLastDeathTime = $sim::time;
				
				if(%mini.getProperty("respawn"))
				{
					if(%mini.ZAPT_PlayerZombies)
					{
						%transformed = true;
						%client.isZombie = true;
						messageClient(%client, '', "<color:ffaaaa>You are now a Zombie.");
					}
				}
				else
				{
					%client.isDead = true;
					messageClient(%client, '', "<color:ffaaaa>You are now Dead. There's no respawning in <color:ffffaa>" @ %mini.getProperty("uiName") @ "<color:ffaaaa>, so sit back.");
					messageClient(%client, 'MsgYourSpawn');
					%client.zombieCameraSched = %client.schedule(2000, "zombieAutoCameraTarget");
				}
			}
			else
			{
				%kill = true;
				
				if(!isObject(%sourceClient) || %client.getID() == %sourceClient.getID())
				{
					%kill = false;
				}
				
				if(%kill)
				{
					%msg = getTaggedString($DeathMessage_Murder[%damageType]);
				}
				else
				{
					%msg = getTaggedString($DeathMessage_Suicide[%damageType]);
				}
				
				%msg = strReplace(%msg, "%1", "Zombie" SPC %client.getPlayerName());
				
				if(%kill)
				{
					%msg = strReplace(%msg, "%2", %sourceClient.getPlayerName());
				}
				
				if(isObject(%mini))
				{
					%mini.messageAll('', %msg);
				}
				else
				{
					messageAll('',%msg);
				}
				
				%damageType = 0;
				%client.player.lastDirectDamageType = 0;
			}
		}
		
		%result = parent::onDeath(%client, %sourceObject, %sourceClient, %damageType, %damageArea);
		
		if(%transformed)
		{
			if(%client.zombieHudCells > -1)
			{
				ZombieFriends.clearFriends(%client);
				ZombieFriends.getFriends(%client);
				ZombieFriends.sendData(%client);
			}
			
			ZombieFriends.replaceFriend(%client);
		}
		
		ZombieFriends.sendFriendData(%client);
		
		if(%client.isDead)
		{
			%client.centerPrint("");
		}
		
		return %result;
	}
	
	function GameConnection::spawnPlayer(%client)
	{
		if(isObject(%client.player.tempBrick))
		{
			%client.player.tempBrick.delete();
		}
		
		if(%client.isAFK || %client.isDead)
		{
			%mini = getMiniGameFromObject(%mini);
			
			if((zombieGameSetUpRight(%mini) && !%mini.getProperty("respawn")) || %client.isAFK)
			{
				%client.isDead = true;
				messageClient(%client, 'MsgYourSpawn');
				return;
			}
			else
			{
				%client.isDead = false;
			}
		}
		else if(%client.isZombie)
		{
			%mini = getMiniGameFromObject(%mini);
			
			if(zombieGameSetUpRight(%mini) && !%mini.ZAPT_PlayerZombies)
			{
				%client.isZombie = false;
			}
		}
		
		%result = parent::spawnPlayer(%client);
		
		ZombieFriends.sendFriendData(%client);
		
		if(%client.zombieHudCells > -1)
		{
			ZombieFriends.sendData(%client);
		}
		
		return %result;
	}
	
	// ============================================================
	// Section 5 : MinigameSO
	// ============================================================
	function MinigameSO::addMember(%so, %client)
	{
		%client.isDead = false;
		%client.isRescued = false;
		%client.isZombie = false;
		
		if(zombieGameSetUpRight(%so))
		{
			switch(%so.ZAPT_GameMode)
			{
				case 0:
					if(%so.ZAPT_PlayerZombies && $sim::time - %so.zombieTimeAssembled > 30)
					{
						%client.isZombie = true;
						%client.spawnPlayer();
						messageClient(%client, '', "\c5You will get to play as a human next round.");
					}
					
					%result = parent::addMember(%so, %client);
					
				case 1:
					%result = parent::addMember(%so, %client);
					
					if(isObject(%client.player))
					{
						%client.player.delete();
					}
					
					%client.isDead = true;
					%client.zombieCameraSched = %client.schedule(2000, "zombieAutoCameraTarget");
					messageClient(%client, '', "\c5This minigame is on \c3Versus Mode\c5. You will get to play next round!");
					
				case 2:
					%result = parent::addMember(%so, %client);
					
					if(isObject(%client.player))
					{
						%client.player.delete();
					}
					
					%client.isDead = true;
					%client.zombieCameraSched = %client.schedule(2000, "zombieAutoCameraTarget");
					messageClient(%client, '', "\c5This minigame is on \c3Survival Mode\c5. You will get to play next round!");
					
				default:
					%result = parent::addMember(%so, %client);
			}
		}
		else
		{
			%result = parent::addMember(%so, %client);
		}
		
		ZombieFriends.clearFriends(%client);
		ZombieFriends.getFriends(%client);
		ZombieFriends.sendData(%client);
		ZombieFriends.setFriend(%client);
		
		return %result;
	}
	
	function MinigameSO::displayScoresList(%so)
	{
		// TDM Function.
		// Bottom Print interrupts timer and is ultimately useless in this game mode.
		
		if(!zombieGameSetUpRight(%so))
		{
			return parent::displayScoresList(%so);
		}
	}
	
	function MinigameSO::endGame(%so)
	{
		cancel(%so.zombieAnnounceTimeSched);
		cancel(%so.zombieOutbreakSched);
		cancel(%so.zombieTankSched);
		
		%so.clearAllZombies();
		%so.zombieGameEnding = true;
		
		for(%a = 0; %a < %so.numMembers; %a++)
		{
			%member = %so.member[%a];
			
			if(%member.zombieHUDOpen)
			{
				ZombieFriends.clearFriends(%member);
				commandToClient(%member, 'ZAPT_HUD_DESTROY');
				%member.zombieHUDOpen = false;
			}
		}
		
		return parent::endGame(%so);
	}
	
	function MinigameSO::onAdd(%so)
	{
		parent::onAdd(%so);
		%so.assembleZombieGame();
		ZombieEvents.triggerEvent("onMinigameRand" @ getRandom(1, 2), %so);
	}

	function MinigameSO::pickSpawnPoint(%so, %client)
	{
		if(%client.spawningAsTank)
		{
			%point = ZombieDirector.pickSpawnPointTank(%client);
		}
		
		if(%point $= "")
		{
			if(%client.isZombie)
			{
				%point = ZombieDirector.pickSpawnPoint(%so, %client);
			}
			
			if(%point $= "")
			{
				%point = parent::pickSpawnPoint(%so, client);
			}
		}
		
		return %point;
	}
	
	function MinigameSO::removeMember(%so, %client)
	{
		%client.isDead = false;
		%client.isZombie = false;
		%client.zombieVersusInRow = 0;
		cancel(%client.zombieSpawnAsTankSched);
		
		ZombieFriends.removeFriend(%client);
		
		%result = parent::removeMember(%so, %client);
		
		if(%client.zombieHUDOpen)
		{
			ZombieFriends.clearFriends(%client);
			commandToClient(%client, 'ZAPT_HUD_DESTROY');
			%client.zombieHUDOpen = false;
		}
		
		return %result;
	}
	
	function MinigameSO::reset(%so, %client)
	{
		cancel(%so.zombieAnnounceTimeSched);
		cancel(%so.zombieOutbreakSched);
		cancel(%so.zombieTankSched);
		%so.zombieAssembledOnce = false;
		
		%so.clearAllZombies();
		
		%so.assembleZombieGame();
		
		%result = parent::reset(%so, %client);
		
		ZombieEvents.triggerEvent("onMinigameRand" @ getRandom(1, 2), %so);
		
		return %result;
	}
	
	function MinigameSO::updatePlayerDatablock(%so)
	{
		if(zombieGameSetUpRight(%so))
		{
			for(%a = 0; %a < %so.numMembers; %a++)
			{
				%member = %so.member[%a].player;
				
				if(isObject(%member) && !%member.isZombie)
				{
					%member.setDatablock(%so.PlayerDatablock);
				}
			}
		}
		else
		{
			return parent::updatePlayerDatablock(%so);
		}
	}
	
	// ============================================================
	// Section 6 : Namespaceless
	// ============================================================
	function applyDefaultCharacterPrefs(%obj)
	{
		if(%obj.isZombie)
		{
			%db = %obj.getDatablock();
			
			if(fileName(%db.shapeFile) $= "m.dts")
			{
				hideAllNodes(%obj);
				
				if(%db.zombieDress(%obj))
				{
					if(!isObject(%obj.zombieClothedAs))
					{
						// Blob Data
						%obj.zombieClothedAs	= %db.getID();
						%obj.zombieDecalChest	= getRandom(0, 20);
						%obj.zombieDecalFace	= getRandom(0, 7);
						%obj.zombieHat			= getRandom(0, 7);
						%obj.zombieGarmentCount	= 9;
						%obj.zombieGender		= getRandom(0, 1);
						%obj.zombieSkinColor	= %db.getSkinColor(%obj);
						
						// No-Stupid-Hats Filter
						if(%obj.zombieHat != 4 && %obj.zombieHat != 6 && %obj.zombieHat != 7)
						{
							%obj.zombieHat = 0;
						}
						
						// Tie-Dye
						for(%a = 0; %a < 9; %a++)
						{
							switch(%a)
							{
								case 0: %garment = $Hat[%obj.zombieHat];
								case 1: %garment = $Chest[%obj.zombieGender];
								case 2: %garment = $LArm[%obj.zombieGender];
								case 3: %garment = $RArm[%obj.zombieGender];
								case 4: %garment = "lhand";
								case 5: %garment = "rhand";
								case 6: %garment = "pants";
								case 7: %garment = "lshoe";
								case 8: %garment = "rshoe";
							}
							
							// Hands cannot be clothed. Hats, Chest, and Hips cannot be skin-colored. Othewise, 50% chance of naked node.
							if(%a != 4 && %a != 5 && (%a == 0 || %a == 1 || %a == 6 || getRandom(0, 1)))
							{
								%color = (getRandom(50, 125) / 255) SPC (getRandom(50, 125) / 255) SPC (getRandom(50, 125) / 255) SPC "1";
							}
							else
							{
								%color = "skin";
							}
							
							%obj.zombieGarment[%a] = %garment;
							%obj.zombieGarmentColor[%a] = %color;
						}
						
						// Face Paint
						switch(%obj.zombieDecalFace)
						{
							case 0: %obj.zombieDecalFace = "ZombieFaceA";
							case 1: %obj.zombieDecalFace = "ZombieFaceB";
							case 2: %obj.zombieDecalFace = "ZombieFaceC";
							case 3: %obj.zombieDecalFace = "ZombieFaceD";
							case 4: %obj.zombieDecalFace = "ZombieFaceE";
							case 5: %obj.zombieDecalFace = "ZombieFaceF";
							case 6: %obj.zombieDecalFace = "ZombieFaceG";
							case 7: %obj.zombieDecalFace = "ZombieFaceI";
							default: %obj.zombieDecalFace = "smiley";
						}
						
						// Chest Hairs
						switch(%obj.zombieDecalChest)
						{
							case 0: %obj.zombieDecalChest = "Mod-Police";
							case 1: %obj.zombieDecalChest = "Mod-Suit";
							case 2: %obj.zombieDecalChest = "Mod-Pilot";
							case 3: %obj.zombieDecalChest = "Hoodie";
							case 4: %obj.zombieDecalChest = "Chef";
							case 5: %obj.zombieDecalChest = "DrKleiner";
							case 6: %obj.zombieDecalChest = "worm_engineer";
							case 7: %obj.zombieDecalChest = "worm_sweater";
							case 8: %obj.zombieDecalChest = "ZAPT-BasketBall";
							case 9: %obj.zombieDecalChest = "ZAPT-BillyMays";
							case 10: %obj.zombieDecalChest = "ZAPT-Bowling";
							case 11: %obj.zombieDecalChest = "ZAPT-MallCop";
							case 12: %obj.zombieDecalChest = "ZAPT-StoClerk";
							case 13:
								%obj.zombieDecalChest = "ZAPT-Suit";
								%obj.zombieGarmentColor[1] = %obj.zombieGarmentColor[1] !$= "skin" ? "0 0 0 1" : "skin";
								%obj.zombieGarmentColor[2] = %obj.zombieGarmentColor[2] !$= "skin" ? "0 0 0 1" : "skin";
								%obj.zombieGarmentColor[3] = %obj.zombieGarmentColor[3] !$= "skin" ? "0 0 0 1" : "skin";
								%obj.zombieGarmentColor[6] = %obj.zombieGarmentColor[6] !$= "skin" ? "0 0 0 1" : "skin";
								%obj.zombieGarmentColor[7] = %obj.zombieGarmentColor[7] !$= "skin" ? "0.4 0.2 0 1" : "skin";
								%obj.zombieGarmentColor[8] = %obj.zombieGarmentColor[8] !$= "skin" ? "0.4 0.2 0 1" : "skin";
							default: %obj.zombieDecalChest = "AAA-None";
						}
					}
					else if(%obj.zombieClothedAs.getID() != %db.getID())
					{
						%obj.zombieClothedAs = %db.getID();
						%obj.zombieSkinColor = %db.getSkinColor(%obj);
					}
				}
				
				// Skincell Oxygen Deprivation
				%obj.setNodeColor("headskin", %obj.zombieSkinColor);
				
				for(%a = 0; %a < %obj.zombieGarmentCount; %a++)
				{
					if(%obj.zombieGarment[%a] !$= "none")
					{
						%garmentColor = %obj.zombieGarmentColor[%a];
						
						if(%garmentColor $= "skin")
						{
							%garmentColor = %obj.zombieSkinColor;
						}
						
						%obj.unHideNode(%obj.zombieGarment[%a]);
						%obj.setNodeColor(%obj.zombieGarment[%a], %garmentColor);
					}
				}
				
				// Limb Ripper
				for(%a = 1; %a <= %db.zombieHideNodes; %a++)
				{
					%obj.hideNode(%db.zombieHideNode[%a]);
				}
				
				// Limb Gluer
				for(%a = 1; %a <= %db.zombieShowNodes; %a++)
				{
					%obj.unHideNode(%db.zombieShowNode[%a]);
				}
				
				%obj.setFaceName(%obj.zombieDecalFace);
				%obj.setDecalName(%obj.zombieDecalChest);
			}
			else
			{
				return parent::applyDefaultCharacterPrefs(%obj);
			}
		}
		else
		{
			return parent::applyDefaultCharacterPrefs(%obj);
		}
	}
	
	function destroyServer()
	{
		deleteVariables("$ZAPT_Temp::*");
		
		echo("\nIniating ZAPT Object Clean-Up");
		%deleted = 0;
		
		// Destroys server-oriented objects.
		for(%a = RootGroup.getCount() - 1; %a >= 0; %a--)
		{
			%obj = RootGroup.getObject(%a);
			
			if(strstr(%obj.getName(), "Zombie") == 0)
			{
				%deleted++;
				echo(" +- Deleting" SPC %obj.getName() SPC "(" @ %obj.getID() @ ")");
				%obj.schedule(32, "delete");
			}
		}
		
		echo(%deleted SPC "zombie object(s) deleted.\n");
		
		return parent::destroyServer();
	}
	
	function getMiniGameFromObject(%obj)
	{
		%result = parent::getMiniGameFromObject(%obj);
		
		if(isObject(%obj) && !%obj.zombieGMGFOLoopHack && !isObject(%result))
		{
			%type = %obj.getType();
			
			if(%type & ($TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType))
			{
				%retry = %obj.spawnBrick;
				
				if(isObject(%retry) && isObject(%retry.getGroup().client))
				{
					%retry = %retry.getGroup().client;
				}
			}
			else if(%type & ($TypeMasks::FxBrickAlwaysObjectType | $TypeMasks::FxBrickObjectType))
			{
				%retry = %obj.getGroup().client;
			}
			
			if(isObject(%retry))
			{
				%result = parent::getMiniGameFromObject(%retry);
			}
		}
		
		return %result;
	}
	
	function miniGameCanDamage(%obj1, %obj2)
	{
		%canDamage = parent::miniGameCanDamage(%obj1, %obj2);
		
		if(%canDamage == 1&& isObject(%obj1) && isObject(%obj2))
		{
			%mini = getMiniGameFromObject(%obj1);
			
			if(zombieGameSetUpRight(%mini) && (%mini.ZAPT_FF && zombieFriendlyFire(%obj1, %obj2)))
			{
				return false;
			}
			// Tests for a safehouse trigger on either object (assuming they're a brick).
			else if(%mini.getProperty("safe"))
			{
				%ss = ZombieDirector.safehouseKeeper;
				
				if(isObject(%ss) && %ss.getCount() > 0)
				{
					for(%a = 1; isObject(%obj[%a]); %a++)
					{
						if(%obj[%a].getClassName() $= "fxDTSBrick" && !isObject(%obj[%a].Shape))
						{
							%brickMinX = mFloatLength(getWord(%obj[%a].getWorldBox(), 0), 1);
							%brickMinY = mFloatLength(getWord(%obj[%a].getWorldBox(), 1), 1);
							%brickMinZ = mFloatLength(getWord(%obj[%a].getWorldBox(), 2), 1);
							
							%brickMaxX = mFloatLength(getWord(%obj[%a].getWorldBox(), 3), 1);
							%brickMaxY = mFloatLength(getWord(%obj[%a].getWorldBox(), 4), 1);
							%brickMaxZ = mFloatLength(getWord(%obj[%a].getWorldBox(), 5), 1);
							
							for(%b = 0; %b < %ss.getCount(); %b++)
							{
								%obj[%a] = %ss.getObject(%b);
								
								if(zombieSameGame(%obj[%a], %obj[%a]))
								{
									%trigger = %obj[%a].trigger;
									
									%triggerMinX = mFloatLength(getWord(%trigger.getWorldBox(), 0), 1);
									%triggerMinY = mFloatLength(getWord(%trigger.getWorldBox(), 1), 1);
									%triggerMinZ = mFloatLength(getWord(%trigger.getWorldBox(), 2), 1);
									
									%triggerMaxX = mFloatLength(getWord(%trigger.getWorldBox(), 3), 1);
									%triggerMaxY = mFloatLength(getWord(%trigger.getWorldBox(), 4), 1);
									%triggerMaxZ = mFloatLength(getWord(%trigger.getWorldBox(), 5), 1);
									
									if(%triggerMinX <= %brickMinX && %triggerMinY <= %brickMinY && %triggerMinZ <= %brickMinZ && 
										%triggerMaxX >= %brickMaxX && %triggerMaxY >= %brickMaxY && %triggerMaxZ >= %brickMaxZ)
									{
										// HOLY SHIT WE CAN FINALLY STOP THIS.
										return false;
									}
								}
							}
						}
					}
				}
			}
		}
		
		return %canDamage;
	}
	
	function miniGameCanUse(%obj1, %obj2)
	{
		%result = parent::miniGameCanUse(%obj1, %obj2);
		
		if(%result == 1)
		{
			%mini = getMiniGameFromObject(%obj1);
			
			if(zombieGameSetUpRight(%mini) && %mini.numMembers == 1)
			{
				for(%a = 1; isObject(%obj[%a]); %a++)
				{
					if(%obj[%a].getClassName() $= "fxDTSBrick")
					{
						if(%obj[%a].vehicleDatablock.zombieConstrict)
						{
							return false;
						}
					}
				}
			}
		}
		
		return %result;
	}
	
	// ============================================================
	// Section 7 : Observer
	// ============================================================
	function Observer::onTrigger(%this, %obj, %trigger, %state)
	{
		if(isObject(%client = %obj.getControllingClient()))
		{
			%mini	= getMiniGameFromObject(%client);
			%cam	= %client.camera;
			%dead	= %client.isDead;
			%zombie	= (%client.isZombie || %client.player.isZombie ? true : false);
			
			// Down
			if(%state)
			{
				switch(%trigger)
				{
					case 0:
						if((isObject(%mini) && !%mini.getProperty("respawn")) && %dead)
						{
							//Fire: Switch target forwards one
							cancel(%client.zombieCameraSched);
							
							for(%i = %client.tdmCameraTarget + 1; %i < (%mini.numMembers + %client.tdmCameraTarget); %i++)
							{
								%member = %mini.member[%i % %mini.numMembers];
								
								if(isObject(%member.player) && %member.player.getState() !$= "Dead" && !%member.isZombie)
								{
									%cam.setMode("Corpse", %member.player);
									%client.tdmCameraTarget = %i;
									%client.setControlObject(%cam);
									return;
								}
							}
							
							return;
						}
						
					case 2:
						if(isObject(%client.player.zombieChargerConn))
						{
							%client.player.zombieChargerConn.setTrigger(%trigger, %state);
						}
						else if((isObject(%mini) && !%mini.getProperty("respawn")) && %dead)
						{
							//Jump: Switch camera mode
							cancel(%client.zombieCameraSched);
							
							if(%obj.mode $= "Observer" && isObject(%client.player))
							{
								%client.schedule(10, "setControlObject", %client.player);
								return;
							}
							else if(%obj.mode $= "Corpse")
							{
								%obj.setMode("Observer");
								return;
							}
							
							for(%i = %client.tdmCameraTarget + 1;%i < (%mini.numMembers + %client.tdmCameraTarget); %i++)
							{
								%member = %mini.member[%i % %mini.numMembers];
								
								if(isObject(%member.player) && %member.player.getState() !$= "Dead" && !%member.isZombie)
								{
									%cam.setMode("Corpse", %member.player);
									%client.tdmCameraTarget = %i;
									%client.setControlObject(%cam);
									return;
								}
							}
							
							return;
						}
						
					case 4:
						if((isObject(%mini) && !%mini.getProperty("respawn")) && %dead)
						{
							cancel(%client.zombieCameraSched);
							
							for(%i = %mini.numMembers - %client.tdmCameraTarget; %i > -(%client.tdmCameraTarget); %i--)
							{
								%member = %mini.member[(%i + %mini.numMembers) % %mini.numMembers];
								
								if(isObject(%member.player) && %member.player.getState() !$= "Dead" && !%member.isZombie)
								{
									%cam.setMode("Corpse", %member.player);
									%client.tdmCameraTarget = %mini.numMembers - %i;
									%client.setControlObject(%cam);
									return;
								}
							}
							
							return;
						}
				}
			}
			else
			{
				switch(%trigger)
				{
					case 2:
						if(isObject(%client.player.zombieChargerConn))
						{
							%client.player.zombieChargerConn.setTrigger(%trigger, %state);
						}
				}
			}
		}
		
		return parent::onTrigger(%this, %obj, %trigger, %state);
	}
	
	// ============================================================
	// Section 8 : Player
	// ============================================================
	function Player::activateStuff(%obj)
	{
		if(%obj.isZombie)
		{
			%obj.zombieClaw();
		}
		else if(!%obj.zombieHelpless && !%obj.zombieSmoked && %obj.getState() !$= "Dead")
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(zombieGameSetUpRight(%mini))
			{
				%start = %obj.getEyePoint();
				%aimVec = %obj.zombieAimVec();
				%range = 3 * getWord(%obj.getScale(), 2);
				%end = vectorAdd(%start, vectorScale(%aimVec, %range));
				%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
				
				%col = getWord(containerRayCast(%start, %end, %masks, %obj), 0);
				
				if(isObject(%col) && %col.getType() & $TypeMasks::PlayerObjectType && zombieSameGame(%col, %obj) &&
					!%col.isZombie && %col.zombieSmoked && %col.getState() !$= "Dead" &&
					isObject(%col.zombieSmokedBy.zombieSmokingWho) && %col.zombieSmokedBy.zombieSmokingWho.getID() == %col.getID())
				{
					if(!zombieBot(%col.zombieSmokedBy))
					{
						if(!zombieBot(%obj.client) && !zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3" @ %obj.client.name @ "<color:ffaaaa> broke your strangle on \c3" @ %col.client.name @ "<color:ffaaaa>!");
						}
						else if(!zombieBot(%obj.client) && zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3A survivor <color:ffaaaa>broke your strangle on \c3" @ %col.client.name @ "<color:ffaaaa>!");
						}
						else if(zombieBot(%obj.client) && !zombieBot(%col.client))
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3" @ %obj.client.name @ "<color:ffaaaa> broke your strangle on \c3another survivor<color:ffaaaa>!");
						}
						else
						{
							messageClient(%col.zombieSmokedBy.client, '', "\c3A survivor <color:ffaaaa>broke your strangle on \c3another survivor<color:ffaaaa>!");
						}
					}
					
					if(isObject(%col.getMountedImage(2)) && %col.getMountedImage(2).getID() == ZombieSmokerConstrictImage.getID())
					{
						%col.unMountImage(2);
					}
					
					%col.zombieHelpless = false;
					cancel(%col.zombieSmokedBy.zombieStrangleSched);
					%col.zombieSmoked = false;
					%col.zombieSmokedBy = "";
					
					if(!zombieBot(%obj.client))
					{
						if(!zombieBot(%col.client))
						{
							messageClient(%obj.client, '', "<color:aaffaa>You rescued \c3" @ %col.client.name @ "<color:aaffaa>!");
						}
						else
						{
							messageClient(%obj.client, '', "<color:aaffaa>You rescued \c3a survivor<color:aaffaa>!");
						}
					}
					
					if(!zombieBot(%col.client))
					{
						if(!zombieBot(%obj.client))
						{
							messageClient(%col.client, '', "<color:aaffaa>You were rescued by \c3" @ %obj.client.name @ "<color:aaffaa>!");
						}
						else
						{
							messageClient(%col.client, '', "<color:aaffaa>You were rescued<color:aaffaa>!");
						}
					}
				}
			}
		}
		
		return parent::activateStuff(%obj);
	}
	
	function Player::changeDatablock(%obj, %armor)
	{
		if(!zombieBot(%obj))
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(zombieGameSetUpRight(%mini))
			{
				%obj.client.isZombie = %armor.isZombieArmor;
			}
		}
		
		return parent::changeDatablock(%obj, %armor);
	}
	
	function Player::doOverhealDecay(%obj)
	{
		parent::doOverhealDecay(%obj);
		
		// Health
		if(!zombieBot(%obj) && zombieGameSetUpRight(getMiniGameFromObject(%obj)))
		{
			ZombieFriends.sendFriendData(%obj.client);
		}
	}
	
	function Player::playThread(%obj, %slot, %thread)
	{
		if(%obj.isZombie)
		{
			if(%thread $= "root")
			{
				%thread = "ArmReadyBoth";
			}
			else if(!%obj.zombieCanActivate && (%thread $= "activate" || %thread $= "activate2"))
			{
				return;
			}
		}
		
		return parent::playThread(%obj, %slot, %thread);
	}
	
	function Player::unHideNode(%obj, %node)
	{
		%db = %obj.getDatablock();
		
		if(%db.zombieHideNodes > 0)
		{
			for(%a = 1; %a <= %db.zombieHideNodes; %a++)
			{
				if(%node $= %db.zombieHideNode[%a])
				{
					return;
				}
			}
		}
		
		parent::unHideNode(%obj, %node);
	}
	
	function Player::setShapeNameColor(%obj, %color)
	{
		if($ZAPT::ConditionColors)
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(zombieGameSetUpRight(%mini))
			{
				if(%obj.isZombie)
				{
					%color = "0.0 0.5 0.4 1.0";
				}
				else
				{
					if(%obj.zombieHelpless || %obj.zombieSmoked)
					{
						%color = "1.0 0.4 0.0 1.0";
					}
					else if(%obj.zombieBeacon)
					{
						%color = "1.0 0.0 1.0 1.0";
					}
					else
					{
						%color = $MiniGameColorF[%mini.colorIdx];
					}
				}
			}
			else if(isObject(%mini))
			{
				%color = $MiniGameColorF[%mini.colorIdx];
			}
			else
			{
				%color = "1 1 1 1";
			}
			
			parent::setShapeNameColor(%obj, %color);
		}
		else
		{
			parent::setShapeNameColor(%obj, %color);
		}
	}
	
	function Player::setVelocity(%obj, %vel)
	{
		if(%obj.isZombie)
		{
			%db = %obj.getDatablock();
			
			if(%db.zombieVelNull)
			{
				%vel = vectorScale(%vel, %db.zombieVelNull);
			}
		}
		
		return parent::setVelocity(%obj, %vel);
	}
	
	// ============================================================
	// Section 9 : ProjectileData
	// ============================================================
	function ProjectileData::impactImpulse(%this, %obj, %col, %idk)
	{
		// hacky hacky hacky om nom nom
		%impact = %this.impactImpulse;
		%vertical = %this.verticalImpulse;
		
		if(%col.isZombie)
		{
			if(%col.zombieSmoking)
			{
				%this.impactImpulse *= %db.zombieVelNull;
				%this.verticalImpulse *= %db.zombieVelNull;
			}
			else
			{
				%db = %col.getDatablock();
				
				if(%db.zombieVelNull !$= "")
				{
					%this.impactImpulse *= %db.zombieVelNull;
					%this.verticalImpulse *= %db.zombieVelNull;
				}
			}
		}
		
		%result = parent::impactImpulse(%this, %obj, %col, %idk);
		
		%this.impactImpulse = %impact;
		%this.verticalImpulse = %vertical;
		
		return %result;
	}
	
	function ProjectileData::radiusImpulse(%this, %obj, %col, %dir, %pos, %impulse, %fade)
	{
		if(%col.isZombie)
		{
			if(%col.zombieSmoking)
			{
			
			}
			else
			{
				%db = %col.getDatablock();
				
				if(%db.zombieVelNull !$= "")
				{
					%impulse *= %db.zombieVelNull;
				}
			}
		}
		
		return parent::radiusImpulse(%this, %obj, %col, %dir, %pos, %impulse, %fade);
	}
		
	// ============================================================
	// Section 10 : serverCmds
	// ============================================================
	function serverCmdAddEvent(%client, %delay, %input, %target, %idk1, %idk2, %output, %arg1, %arg2, %arg3, %arg4, %arg5, %arg6, %arg7, %arg8)
	{
		if(isObject(%client.wrenchBrick))
		{
			ZombieEvents.brickKeeper.add(%client.wrenchBrick);
		}
		
		return parent::serverCmdAddEvent(%client, %delay, %input, %target, %idk1, %idk2, %output, %arg1, %arg2, %arg3, %arg4, %arg5, %arg6, %arg7, %arg8);
	}
	
	function serverCmdClearEvents(%client)
	{
		if(ZombieEvents.brickKeeper.isMember(%client.wrenchBrick))
		{
			ZombieEvents.brickKeeper.remove(%client.wrenchBrick);
		}
		
		return parent::serverCmdClearEvents(%client);
	}
	
	function serverCmdLight(%client)
	{
		if(!%client.player.isZombie)
		{
			return parent::serverCmdLight(%client);
		}
	}
	
	function serverCmdInstantUseBrick(%client, %brick)
	{
		%obj = %client.player;
		
		if(!%obj.isZombie && !%obj.zombieHelpless)
		{
			return parent::serverCmdInstantUseBrick(%client, %brick);
		}
	}
	
	function serverCmdTeamMessageSent(%client, %message)
	{
		if(zombieGameSetUpRight(%client.minigame))
		{
			if(getSimTime() - %client.spamProtectStart >= 10000 || %client.isAdmin)
			{
				%message = trim(stripMLControlChars(trim(%message)));
				
				if(strlen(%message) > 0 && (%message !$= %client.lastChatText || %client.isAdmin))
				{
					if(getSimTime() - %client.lastChatTime < 2000)
					{
						%client.spamMessageCount++;
					}
					
					if(%client.spamMessageCount < 4 || %client.isAdmin)
					{
						%client.lastChatTime = getSimTime();
						%client.lastChatText = %message;
						
						switch(%client.isZombie)
						{
							case false: %tag = "<color:aaffaa>Survivor";
							case true: %tag = "<color:ffaaaa>Zombie";
							default:  %tag = "\c0IBAN_FAILS_AT_SCRIPT";
						}
						
						echo("(T" @ (%client.isZombie ? "-Z" : "-S") @ ")" @ %client.name @ ":" SPC %message);
						%message = "\c7[" @ %tag @ "\c7]" SPC %client.clanPrefix @ "\c3" @ %client.name @ "\c7" @ %client.clanSuffix @ "\c4:" SPC %message;
						
						for(%a = 0; %a < %client.minigame.numMembers; %a++)
						{
							%member = %client.minigame.member[%a];
							
							if(%client.isZombie == %member.isZombie)
							{
								messageClient(%member, '', %message);
							}
						}
						
						return;
					}
					else
					{
						messageClient(%client, '', "\c5You are flooding the server.");
					}
				}
				else if(%message $= %client.lastChatText)
				{
					messageClient(%client, '', "\c5Do not repeat yourself.");
				}
			}
		}
		else
		{
			return parent::serverCmdTeamMessageSent(%client, %message);
		}
	}
	
	function serverCmdSetWrenchData(%client, %data)
	{
		%brick = %client.wrenchBrick;
		
		if(isObject(%brick) && getTrustLevel(%client, %brick) >= 2)
		{
			%db = %brick.getDatablock();
			
			if(%db.specialBrickType $= "VehicleSpawn")
			{
				%fieldCount = getFieldCount(%data);
				
				// Disect Data
				for(%a = 0; %a < %fieldCount; %a++)
				{
					%datum = getField(%data, %a);
					%name = getWord(%datum, 0);
					%value = getSubStr(%datum, strlen(%name) + 1, strlen(%datum) - 1);
					
					%datumName[%a] = %name;
					%datumValue[%name] = %value;
				}
				
				// Restrict Data
				if(isObject(%datumValue["VDB"]))
				{
					%canSpawn = true;
					
					if(%datumValue["VDB"].isZombieArmor != %db.isZombieNormSpawn)
					{
						%canSpawn = false;
					}
					
					if(!%canSpawn)
					{
						if(%datumValue["VDB"].isZombieArmor)
						{
							commandToClient(%client, 'centerPrint', "\c5You cannot spawn zombies anywhere but on a \c3Zombie Spawn\c5.", 4);
						}
						else
						{
							commandToClient(%client, 'centerPrint', "\c5You cannot spawn this on a \c3" @ %db.uiName @ "\c5.", 4);
						}
						
						%datumValue["VDB"] = "";
					}
					else if(%datumValue["VDB"].isZombieArmor)
					{
						%strippedName = trim(strreplace(%datumValue["VDB"].uiName, "Zombie - ", ""));
						commandToClient(%client, 'centerPrint', "\c5Your \c3" @ %strippedName @ "\c5 will be spawned when the Director chooses.", 4);
					}
				}
				
				// Reassemble Data
				for(%c = 0; %datumName[%c] !$= ""; %c++)
				{
					if(%newData !$= "")
					{
						%newData = %newData @ "\t";
					}
					
					%newData = %newData @ %datumName[%c] @ " " @ %datumValue[%datumName[%c]];
				}
				
				%data = %newData;
			}
		}
		
		return parent::serverCmdSetWrenchData(%client, %data);
	}
	
	function serverCmdSuicide(%client)
	{
		%player = %client.player;
		
		if(isObject(%player))
		{
			%mini = %client.minigame;
			
			if(zombieGameSetUpRight(%mini))
			{
				if(%player.getDatablock().zombieImportant)
				{
					if(!isEventPending(%player.zombieGiveAwaySched))
					{
						%strippedName = trim(strreplace(%player.getDatablock().uiName, "Zombie - ", ""));
						messageClient(%client, '', "<color:aaffaa>Hold on there! We're giving your \c3" @ %strippedName @ "<color:aaffaa> to another person.");
						%player.zombieGiveAwaySched = %player.schedule(3000, "zombieGiveAway");
					}
					
					return;
				}
				else if(!%mini.getProperty("respawn"))
				{
					if(!%client.isZombie)
					{
						messageClient(%client, '', "<color:ffaaaa>You wont get to respawn if you die in <color:ffffaa>" @  %mini.getProperty("uiName") @ "<color:ffaaaa>!");
						return;
					}
				}
				else if(!%mini.ZAPT_Suicide)
				{
					if(!%client.isZombie)
					{
						messageClient(%client, '', "<color:ffaaaa>Don't be a jackass; your team needs you.");
						return;
					}
					else if(isEventPending(%client.zombieClassVerify))
					{
						messageClient(%client, '', "<color:ffaaaa>You can reroll your spawn by right-clicking when your Zombie Name is in red!");
						return;
					}
				}
			}
		}
		
		return parent::serverCmdSuicide(%client);
	}
	
	function serverCmdUseFXCan(%client, %color)
	{
		%obj = %client.player;
		
		if(!%obj.isZombie && !%obj.zombieHelpless)
		{
			return parent::serverCmdUseFXCan(%client, %color);
		}
	}
	
	function serverCmdUseInventory(%client, %tool)
	{
		%obj = %client.player;
		
		if(!%obj.isZombie && !%obj.zombieHelpless)
		{
			return parent::serverCmdUseInventory(%client, %tool);
		}
	}
	
	function serverCmdUseSprayCan(%client, %color)
	{
		%obj = %client.player;
		
		if(!%obj.isZombie && !%obj.zombieHelpless)
		{
			return parent::serverCmdUseSprayCan(%client, %color);
		}
	}
	
	function serverCmdUseTool(%client, %tool)
	{
		%obj = %client.player;
		
		if(!%obj.isZombie && !%obj.zombieHelpless)
		{
			return parent::serverCmdUseTool(%client, %tool);
		}
	}
	
	// Camera
	function servercmdDropCameraAtPlayer(%client)
	{
		%mini = getMiniGameFromObject(%client);
		
		if(isObject(%mini) && !%mini.getProperty("respawn"))
		{
			if(!%client.isZombie && %client.isDead)
			{
				%client.centerPrint("<color:ffaaaa>You have no lives left. Stop trying to cheat.", 2);
				return;
			}
		}
		
		return parent::servercmdDropCameraAtPlayer(%client);
	}
	
	function servercmdDropPlayerAtCamera(%client)
	{
		%mini = getMiniGameFromObject(%client);
		
		if(isObject(%mini) && !%mini.getProperty("respawn"))
		{
			if(!%client.isZombie && %client.isDead)
			{
				%client.centerPrint("<color:ffaaaa>You have no lives left. Stop trying to cheat.", 2);
				return;
			}
		}
		
		return parent::servercmdDropPlayerAtCamera(%client);
	}
	
	// ============================================================
	// Section 11 : Vehicle
	// ============================================================
	function Vehicle::onActivate(%vehicle, %player, %client, %pos, %dir)
	{
		if(!%player.isZombie)
		{
			return parent::onActivate(%vehicle, %player, %client, %pos, %dir);
		}
	}
	
	// ============================================================
	// Section 12 : WeaponImage
	// ============================================================
	function WeaponImage::onFire(%this, %obj, %slot)
	{
		%result = parent::onFire(%this, %obj, %slot);
		
		if(isObject(%result) && isObject(%obj) && !%obj.isZombie)
		{
			%mini = getMiniGameFromObject(%result);
			
			if(zombieGameSetUpRight(%mini))
			{
				for(%a = 0; %this.stateName[%a] !$= ""; %a++)
				{
					if(%this.stateScript[%a] $= "onFire")
					{
						if(isObject(%this.stateSound[%a]))
						{
							%obj.zombieNoiseMake = $sim::time;
						}
						
						break;
					}
				}
			}
		}
		
		return %result;
	}
	
	function WeaponImage::isRaycastCritical(%this, %obj, %slot, %col, %pos, %normal, %hit)
	{
		if(%this.melee && %col.isZombie && miniGameCanDamage(%obj, %col))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
};

if(!$ZAPT_Temp::PackageServerEnabled)
{
	$ZAPT_Temp::PackageServerEnabled = true;
	
	activatePackage(ZAPT_Server);
}