// ============================================================
// Project				:	ZAPT
// Author				:	Iban
// Description			:	Script Objects
// ============================================================
// Table of Contents
// 
// 1. Director
// 2. Events
// 3. Friends
// 4. Mutations
// ============================================================

// ============================================================
// Section 1 : AIConnection
// ============================================================
function ZombieDirector::clearMob(%so, %mini)
{
	// Direct Rogue Zombies
	%ss = %so.mobZKeeper;
	
	if(isObject(%ss))
	{
		%clearing = 0;
		%clearList = "";
		
		for(%a = 0; %a < %ss.getCount(); %a++)
		{
			%zombie = %ss.getObject(%a);
			
			if(isObject(%zombie))
			{
				%zMini = getMiniGameFromObject(%zombie);
				
				if(isObject(%zMini) && %zMini.getID() == %mini.getID())
				{
					%clearing++;
					
					if(%clearList $= "")
					{
						%clearList = %zombie.getID();
					}
					else 
					{
						%clearList = %clearList SPC %zombie.getID();
					}
				}
			}
		}
		
		if(%clearList !$= "")
		{
			%words = getWordCount(%clearList);
			
			for(%a = 0; %a < %words; %a++)
			{
				%zombie = getWord(%clearList, %a);
				%zombie.delete();
			}
		}
	}
	else
	{
		error("CRITICAL ERROR: ZombieDirector.mobZKeeper not found!");
	}
}

function ZombieDirector::direct(%so)
{
	// Direct Zombies
	%ss = %so.zombieKeeper;
	
	if(isObject(%ss))
	{
		for(%a = 0; %a < %ss.getCount(); %a++)
		{
			%spawn	= %ss.getObject(%a);
			%mini	= getMiniGameFromObject(%spawn);
			%zombie	= %spawn.vehicle;
			
			// Thinking
			if(isObject(%zombie))
			{
				if(%zombie.isZombie)
				{
					// Spaces out zombie thought as to ease computer load.
					%time = (%so.getThoughtTime() / %ss.getCount()) * %a;
					
					cancel(%zombie.zombieThinkSched);
					%zombie.zombieThinkSched = %zombie.schedule(%time, "zombieThink");
				}
			}
			// Decision Spawning
			else if(%spawn.zombieCanSpawn())
			{
				%spawnPos = getWords(%spawn.zombieSpawnPoint(), 0, 2);
				
				if(%so.isPosNearPlayer(%mini, %spawnPos, $ZAPT::MaxDist, 16))
				{
					%spawn.spawnVehicle();
					
					if(isObject(%spawn.vehicle))
					{
						%spawn.zombieSpawnTime = $sim::time;
						%spawn.vehicle.zombieSpawnTime = $sim::time;
					}
				}
			}
		}
	}
	else
	{
		error("CRITICAL ERROR: ZombieDirector.zombieKeeper not found!");
	}
	
	// Direct Rogue Zombies
	%ss = %so.mobZKeeper;
	
	if(isObject(%ss))
	{
		for(%a = 0; %a < %ss.getCount(); %a++)
		{
			%zombie = %ss.getObject(%a);
			
			if(%zombie.isZombie)
			{
				%time = (%so.getThoughtTime() / %ss.getCount()) * %a;
				
				cancel(%zombie.zombieThinkSched);
				%zombie.zombieThinkSched = %zombie.schedule(%time, "zombieThink");
			}
		}
	}
	else
	{
		error("CRITICAL ERROR: ZombieDirector.mobZKeeper not found!");
	}
	
	%ss = MiniGameGroup;
	
	if(isObject(%ss))
	{
		for(%a = 0; %a < %ss.getCount(); %a++)
		{
			%mini = %ss.getObject(%a);
			
			if(zombieGameSetUpRight(%mini))
			{
				%so.directMini(%mini);
			}
		}
	}
	
	// Used with modulus to keep the directoring from changing pads every 30 seconds.
	%so.thoughtCount++;
	// Helps ensure there isn't "multi-threaded" thoughts going on.
	cancel(%so.nextThought);
	// Reschedule another thought.
	%so.nextThought = %so.schedule(%so.getThoughtTime(), "direct");
}

function ZombieDirector::directMini(%so, %mini)
{
	if(zombieGameSetUpRight(%mini))
	{
		%stress = %so.getStress(%mini);
		
		if(%so.getThoughtTime() >= 1000 || %so.thoughtCount % mCeil(1000 / %so.getThoughtTime()) == 0)
		{
			%tanks = %mini.getNumberOfTanks();
			
			if(!%tanks)
			{
				if(%stress < 50 && !getRandom(0, %stress))
				{
					%mini.pickTank();
				}
				else if(%stress < 60 && !getRandom(0, %stress))
				{
					%so.panic(%mini);
				}
			}
		}
	}
}

function ZombieDirector::getDistraction(%so, %obj)
{
	%ss = %so.distractKeeper;
	
	if(isObject(%ss))
	{
		if(isObject(%obj))
		{
			%mini = getMiniGameFromObject(%obj);
			
			if(isObject(%mini))
			{
				%pos = %obj.getPosition();
				
				%distractDist = 0;
				%distractObj = -1;
				
				for(%a = 0; %a < %ss.getCount(); %a++)
				{
					%distraction = %ss.getObject(%a);
					
					if(zombieValidTarget(%obj, %distraction))
					{
						%distractionPos = %distraction.getPosition();
						
						if(!isObject(%distractObj) || vectorDist(%distractionPos, %pos) < %distractDist)
						{
							%distractDist = vectorDist(%distractionPos, %pos);
							%distractObj = %distraction;
						}
					}
				}
				
				return %distractObj;
			}
		}
	}
	else
	{
		error("CRITICAL ERROR: ZombieDirector.distractKeeper not found!");
	}
}

function ZombieDirector::getMeanPos(%so, %mini)
{
	if(zombieGameSetUpRight(%mini))
	{
		%denom = 0;
		%gross = "0 0 0";
		
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a].player;
			
			if(isObject(%member) && !%member.isZombie)
			{
				%gross = vectorAdd(%member.getPosition(), %gross);
				%denom++;
			}
		}
		
		// If we have a valid factor to divide by, find the most effective spawn point.
		if(%denom > 0)
		{
			%x = getWord(%gross, 0) / %denom;
			%y = getWord(%gross, 1) / %denom;
			%z = getWord(%gross, 2) / %denom;
			
			%point = %x SPC %y SPC %z;
			
			return %point;
		}
	}
}

function ZombieDirector::getThoughtTime(%so)
{
	return $ZAPT::ThinkEvery;
}

// Collect data from all getStress* functions and publish it.
function ZombieDirector::getStress(%so, %mini)
{
	if(zombieGameSetUpRight(%mini))
	{
		if(!%mini.zombieGameLost)
		{
			// This saves a lot of processing power by just recalculating every thought cycle instead of on-demand.
			if($sim::time - %mini.zombieStressCalcTime > %so.getThoughtTime() / 1000)
			{
				// The goal here is to create a number between 0 (survivors are yawning) and 100 (director is laughing).
				// This number will be a multiplier to determine how aggressive zombies will become.
				for(%a = 0; %a < %so.calcs; %a++)
				{
					eval("%input = %so." @ getField(%so.calc[%a], 0) @ "(%mini, " @ getField(%so.calc[%a], 1) @ ");");
					%stress += getWord(%input, 0);
					%maxStress += getWord(%input, 1);
				}
				
				if(isFunction("Mutation_" @ %mini.getProperty("name") @ "_Stress"))
				{
					%custom = call("Mutation_" @ %mini.getProperty("name") @ "_Stress", %mini);
					%stress += getWord(%custom, 0);
					%maxStress += getWord(%custom, 1);
				}
				
				%stress = mFloor((%stress / %maxStress) * 100);
				
				if(%stress > 100)
				{
					%stress = 100;
				}
				else if(%stress < 0)
				{
					%stress = 0;
				}
				
				%mini.zombieStressCalcTime = $sim::time;
				%mini.zombieStressLevel = %stress;
				
				return %stress;
			}
			else
			{
				return %mini.zombieStressLevel;
			}
		}
		else
		{
			return 100;	
		}
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Gets sum of distance traveled by all players.
// If distance is smaller than required, anger director.
function ZombieDirector::getStressCamping(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		%dist = 0;
		%reqDist = 8 * (%so.getThoughtTime() / 1000);
		
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a].player;
			
			if(isObject(%member) && !%member.isZombie)
			{
				if(%member.zombieLastPosLogged)
				{
					%dist += vectorDist(%member.zombieLastPos, %member.getPosition());
				}
				else
				{
					%dist += %reqDist;
					%member.zombieLastPosLogged = true;
				}
				
				%member.zombieLastPos = %member.getPosition();
			}
		}
		
		if(%dist > %reqDist * %mini.numMembers)
		{
			return %cap SPC %cap;
		}
		else if(%dist >= (mCeil(%reqDist / 2) * %mini.numMembers))
		{
			return ((%dist / %reqDist) * %cap) SPC %cap;
		}
		else
		{
			return 0 SPC %cap;
		}
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Calculates time from most recent death versus remaining survivors.
function ZombieDirector::getStressDeaths(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		if(%mini.zombieLastDeathTime - %mini.zombieTimeAssembled > 0)
		{
			%time = $sim::time - %mini.zombieLastDeathTime;
			%pratical = %time / (60 * (4 / %mini.getNumberOfHumans()));
			%stress = mFloor(%cap * %pratical);
			
			return %stress SPC %cap;
		}
		else
		{
			return 0 SPC %cap;
		}
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Gets sum of all player damage
// Multiplies cap by sum / humans for stress.
function ZombieDirector::getStressHealth(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		%damage = 0;
		%people++;
		
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a];
			
			if(!%member.isZombie && isObject(%member.player))
			{
				%people++;
				%damage += mFloor(%member.player.getDamagePercent() * 100);
			}
		}
		
		%stress = %cap * (%damage / (%people * 100));
		
		return %stress SPC %cap;
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Gets stress based on how long (by a minute) it's been since last panic.
function ZombieDirector::getStressPanic(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		if(%mini.zombieLastDeathTime - %mini.zombieLastPanic > 0)
		{
			%stress = %cap - (%cap * ((%sim::time - %mini.zombieLastPanic) / 60));
			
			if(%stress < 0)
			{
				%stress = 0;
			}
			else if(%stress > %cap)
			{
				%stress = %cap;
			}
			
			return %stress SPC %cap;
		}
		else
		{
			return 0 SPC %cap;
		}
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Gets stressed on number of remaining human players.
function ZombieDirector::getStressPlayers(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		%stress = mFloor(%cap * ((%mini.getNumberOfHumans() - 1) / 4));
		
		if(%stress > %cap * 2)
		{
			%stress = %cap * 2;
		}
		
		return %stress SPC %cap;
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Adds, without consideration, %cap per safehouse survivor.
function ZombieDirector::getStressSafe(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a].player;
			
			if(isObject(%member) && %member.zombieInSafeHouse && ((!%member.isDead && !%member.isZombie && !%member.zombieHelpless) || %member.isRescued))
			{
				%safees++;
			}
		}
		
		// Buffers the stress amazingly.
		return -(%safees * %cap) SPC 0;
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

// Comments within.
function ZombieDirector::getStressTime(%so, %mini, %cap)
{
	if(zombieGameSetUpRight(%mini))
	{
		if(%mini.ZAPT_Timer)
		{
			// Anticipating Rescue?
			if(%mini.ZAPT_Rescue)
			{
				%stress = %cap - ((%mini.zombieTimeLeft / %mini.ZAPT_Time_Rescue) * %cap);
				%maxStress = %cap;
			}
			else
			{
				// This mode is odd to the director's fickle sensitivity because:
				// 1. The timer running out pleases him.
				// 2. Having just the inverse of the other mode means that
				//		players would get swarmed immediately, but things would oddly relax
				//		as it goes on.
				//
				// My solution is:
				// Things build to a head the closer to the middle it goes.
				// Once it hits the mid point, things began to cool as the director feels he has won.
				
				%maxTime = %mini.ZAPT_Time_Rescue;
				%maxStress = %cap;
				
				// Are we above the halfway point?
				if(%maxTime - %mini.zombieTimeLeft < mFloor(%maxTime / 2))
				{
					%stress = (%mini.zombieTimeLeft / %mini.ZAPT_Time_Rescue) * %cap;
				}
				else
				{
					%stress = %cap - ((%mini.zombieTimeLeft / %mini.ZAPT_Time_Rescue) * %cap);
				}
				
				%stress -= %time;
				
			}
		}
		else
		{
			// Gets quarter-minutes.
			%time = mFloor(($sim::time - %mini.zombieTimeAssembled) / 15);
			
			%stress = -mFloor(%cap / 2) + (12.5 * %time);
			%maxStress = %cap;
			
			if(%stress > %cap * 3)
			{
				%stress = %cap * 3;
			}
			
			// Since there is no limit to a timerless round, the stress can superass maxstress.
			// If things go above 100, shit goes haywire.
		}
		
		return %stress SPC %maxStress;
	}
	else
	{
		error("Did not supply a ZAPT minigame.");
		return -1;
	}
}

function ZombieDirector::getWeight(%so, %obj, %target)
{
	if(isObject(%obj) && isObject(%target))
	{
		%mini = getMiniGameFromObject(%obj);
		
		if(zombieValidTarget(%obj, %target))
		{
			// This is a new system that helps Zombies cope with enemy vehicles.
			%mount = %obj.getObjectMount();
			
			if(!isObject(%mount) || !zombieValidTarget(%obj, %mount))
			{
				// Weight is a better way for factoring one target against another than distance.
				// It allows me to consider various decision making steps other than "herp derp they be closest."
				%weight = 0;
				
				// Obtains (even a negative number) based on how far we are..
				// 44 is approximately the size of a 64x64 baseplate in Torque units.
				%dist = -mFloor(vectorDist(%obj.getPosition(), %target.getPosition())) + 44;
				
				// Creates another 100-based number off of how damaged we are.
				// (The more damage, the more delicious)
				%dam = 50 - mFloor(50 * %target.getDamagePercent());
				
				// Biled?
				%beacon = mFloor(%target.zombieBeacon ? 200 : 0);
				
				// Can we see them?
				%sight = mFloor(%obj.zombieSee(%target) ? 100 : -100);
				
				%weight = %dist + %dam + %beacon + %sight;
			}
			else
			{
				%mounted = %mount.getMountedObjectCount();
				%mostMounted = %mount.getDatablock().numMountPoints;
				%weight = 0;
				
				// Stress Factors
				%dist = -mFloor(vectorDist(%mount.getPosition(), %target.getPosition())) + 44;
				%sight = mFloor(%obj.zombieSee(%mount) ? 100 : -100);
				
				%weight = %dist + %sight;
				
				for(%a = 0; %a < %mostMounted; %a++)
				{
					%mountObj = %mount.getMountedObject(%a);
					
					if(isObject(%mountObj))
					{
						%weight += 50 - mFloor(50 * %target.getDamagePercent());
						%weight = mFloor(%target.zombieBeacon ? 200 : 0);
					}
				}
			}
			
			return %weight;
		}
	}
	
	return -1;
}

function ZombieDirector::getZombieCount(%so)
{
	%ss = %so.zombieKeeper;
	
	if(isObject(%ss))
	{
		%spawns = 0;
		%spawnList = "";
		%spawnListSecondary = "";
		
		for(%a = 0; %a < %ss.getCount(); %a++)
		{
			%spawn = %ss.getObject(%a);
			
			if(isObject(%spawn.vehicle))
			{
				%spawns++;
			}
		}
		
		return %spawns;
	}
	else
	{ 
		error("CRITICAL ERROR: ZombieDirector.zombieKeeper not found!");
	}
}

function ZombieDirector::humanCanSee(%so, %mini, %point)
{
	if(isObject(%mini) && %mini.getNumberOfHumans())
	{
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a];
			
			if(!%member.isZombie && !%member.isDead)
			{
				%player = %member.player;
				
				if(isObject(%player) && %player.getState() !$= "Dead")
				{
					%start = %player.getEyePoint();
					%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
					%col = getWord(containerRayCast(%start, %point, %masks, %player), 0);
					
					if(!isObject(%col))
					{
						return true;
					}
				}
			}
		}
	}
	
	return false;
}

function ZombieDirector::isPosNearPlayer(%so, %mini, %pos, %max, %min)
{
	%pos = posFromTransform(%pos);
	
	if(zombieGameSetUpRight(%mini))
	{
		%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType);
		%near = false;
		
		for(%a = 0; %a < %mini.numMembers; %a++)
		{
			%member = %mini.member[%a].player;
			
			if(isObject(%member) && !%member.isZombie)
			{
				%dist = vectorDist(%pos, %member.getPosition());
				
				if(%dist <= %max || %max < 0)
				{
					if(%dist <= %min || %min < 0)
					{
						%col = getWord(containerRayCast(%member.getEyePoint(), %pos, %masks, %member), 0);
					
						if(isObject(%col))
						{
							%near = true;
						}
						else
						{
							return false;
						}
					}
					else
					{
						%near = true;
					}
				}
			}
		}
		
		return %near;
	}
	
	return false;
}

function ZombieDirector::isTankSpawnForThisMini(%so, %mini)
{
	if(isObject(%mini))
	{
		%ss = %so.tankKeeper;
		
		if(isObject(%ss))
		{
			%spawns = 0;
			
			for(%a = 0; %a < %ss.getCount(); %a++)
			{
				%spawn = %ss.getObject(%a);
				%spawnMini = getMiniGameFromObject(%spawn);
				
				if(isObject(%spawnMini) && %spawnMini.getID() == %mini.getID())
				{
					%spawns++;
				}
			}
			
			return %spawns;
		}
		else
		{
			error("CRITICAL ERROR: ZombieDirector.tankKeeper not found!");
		}
	}
}

function ZombieDirector::isZombieInSaferooms(%so, %mini)
{
	if(zombieGameSetUpRight(%mini))
	{
		%keeper = %so.safehouseKeeper;
		
		if(isObject(%keeper))
		{
			for(%a = 0; %a < %keeper.getCount(); %a++)
			{
				%brick = %keeper.getObject(%a);
				%bmini = getMiniGameFromObject(%brick);
				
				if(isObject(%bmini) && %mini.getID() == %bmini.getID())
				{
					%trigger = %brick.trigger;
					
					if(isObject(%trigger))
					{
						for(%b = 0; %b < %trigger.getNumObjects(); %b++)
						{
							%obj = %trigger.getObject(%b);
							
							if(%obj.isZombie)
							{
								%omini = getMiniGameFromObject(%obj);
								
								if(isObject(%omini) && %mini.getID() == %omini.getID())
								{
									return true;
								}
							}
						}
					}
				}
			}
			
			return false;
		}
		else
		{
			error("CRITICAL ERROR: ZombieDirector.safehouseKeeper not found!");
		}
	}
}

function ZombieDirector::panic(%so, %mini)
{
	if(zombieGameSetUpRight(%mini) && $ZAPT::Mobs && %mini.ZAPT_Mobs && $sim::time - %mini.zombieLastPanic > 15 && $sim::time - %mini.zombieTimeAssembled > 30)
	{
		%sKeeper = %so.mobKeeper;
		%zKeeper = %so.mobZKeeper;
		
		if(isObject(%sKeeper) && isObject(%zKeeper) && %zKeeper.getCount() < $ZAPT::MobSize)
		{
			%ceil = $ZAPT::MobSize - %zKeeper.getCount();
			
			if(%ceil > 0)
			{
				%ceil = mDirMod(%ceil / 2, %ceil * 1.25, %mini);
				
				if(%ceil + %zKeeper.getCount() > $ZAPT::MobSize)
				{
					%ceil = $ZAPT::MobSize - %zKeeper.getCount();
				}
			}
			
			if(%ceil > 0)
			{
				%spawnCount = 0;
				
				for(%a = 0; %a < %sKeeper.getCount(); %a++)
				{
					%spawn = %sKeeper.getObject(%a);
					
					if(isObject(%spawn))
					{
						%sMini = getMiniGameFromObject(%spawn);
						
						if(isObject(%sMini) && %mini.getID() == %sMini.getID())
						{
							%spawnPoint = %spawn.zombieSpawnPoint();
							
							if(%so.isPosNearPlayer(%mini, getWords(%spawnPoint, 0, 2), $ZAPT::MaxDist, 16))
							{
								%spawn[%spawnCount++] = %spawn;
							}
						}
					}
				}
				
				if(%spawnCount)
				{
					%spawnPer = %ceil / %spawnCount;
					%spawnBuff = %ceil - (%spawnPer * %spawnCount);
					%spawned = 0;
					
					for(%a = 1; %a <= %spawnCount; %a++)
					{
						%spawnPoint = %spawn[%a].zombieSpawnPoint();
						
						for(%rand = %spawnPer + %spawnBuff; %rand > 0; %rand--)
						{
							%zombie = new AIPlayer()
							{
								canSetIFLs = false;
								dataBlock = ZombieStandardArmor;
								maxPitchSpeed = 10;
								maxYawSpeed = 10;
								zombieMob = true;
								zombieOmnipotence = true;
							};
							
							if(isObject(%zombie))
							{
								MissionCleanup.add(%zombie);
								
								%spawned++;
								
								%zombie.spawnBrick = %spawn[%a];
								%zombie.mobSpawn = %spawn[%a];
								%zombie.setTransform(%spawnPoint);
								
								%zKeeper.add(%zombie);
								
								%zombie.zombieThink();
							}
						}
						
						%spawnBuff = 0;
					}
				}
			}
			
			if(%spawned)
			{
				%mini.zombieLastPanic = $sim::time;
				%mini.messageAll('', "<color:ffaaaa>They're coming...");
				
				echo("ZAPT - Spawned" SPC %spawned SPC "mob zombies in" SPC %mini.owner.name @ "'s minigame.");
			}
		}
	}
}

function ZombieDirector::pickSpawnPoint(%so, %mini, %client, %ignorePref)
{
	if(zombieGameSetUpRight(%mini))
	{
		%ss = %so.zombieKeeper;
		
		if(isObject(%ss))
		{
			%spawns = 0;
			%spawnList = "";
			
			for(%a = 0; %a < %ss.getCount(); %a++)
			{
				%spawn = %ss.getObject(%a);
				%spawnMini = getMiniGameFromObject(%spawn);
				
				if(zombieSameGame(%client, %spawn) && miniGameCanUse(%spawnMini.owner, %spawn))
				{
					%db = %spawn.vehicleDatablock;
					%spawnPos = %spawn.zombieSpawnPoint();
					
					if(isObject(%db) && %db.isZombieArmor && !%db.zombieAIOnly && (!%db.zombieAdminOnly || %client.isAdmin))
					{
						if(%so.isPosNearPlayer(%mini, %spawnPos, %ignorePref != 2 ? $ZAPT::MaxDist : -1,  8))
						{
							if(%ignorePref || %db.getID() != %client.zombieDidNotWant)
							{
								if(!%spawns)
								{
									%spawnList = %spawn.getID();
								}
								else
								{
									%spawnList = %spawnList TAB %spawn.getID();
								}
								
								%spawns++;
							}
						}
					}
				}
			}
			
			if(%spawns)
			{
				%spawnBrick = getField(%spawnList, getRandom(0, %spawns - 1));
				%spawnPoint = %spawnBrick.zombieSpawnPoint();
				
				%client.zombieSpawnBrick = %spawnBrick;
				
				return %spawnPoint;
			}
			// Rerun the test, this time ignoring any preferences.
			else if(%ignorePref < 2)
			{
				return %so.pickSpawnPoint(%mini, %client, %ignorePref++);
			}
		}
		else
		{
			error("CRITICAL ERROR: ZombieDirector.zombieKeeper not found!");
		}
	}
	
	return "";
}

function ZombieDirector::pickSpawnPointTank(%so, %client)
{
	%mini = getMiniGameFromObject(%client);
	
	if(zombieGameSetUpRight(%mini))
	{
		%ss = %so.tankKeeper;
		
		if(isObject(%ss))
		{
			%epicenter = %so.getMeanPos(%mini);
			
			if(%epicenter !$= "")
			{
				%bestSpawn = 0;
				%spawnDist = 0;
				
				for(%a = 0; %a < %ss.getCount(); %a++)
				{
					%spawn = %ss.getObject(%a);
					%spawnMini = getMiniGameFromObject(%spawn);
					
					if(zombieSameGame(%client, %spawn) && miniGameCanUse(%spawnMini.owner, %spawn))
					{
						%dist = vectorDist(%spawn.getPosition(), %epicenter);
						
						if(%dist < %spawnDist || !isObject(%bestSpawn))
						{
							%bestSpawn = %spawn.getID();
							%spawnDist = %dist;
						}
					}
				}
				
				if(isObject(%bestSpawn))
				{
					%spawnPoint = vectorSub(%bestSpawn.getWorldBoxCenter(), "0 0" SPC (%bestSpawn.getDatablock().brickSizeZ - 3) * 0.1) SPC getWords(%bestSpawn.getTransform(), 3, 6);
					%client.zombieSpawnBrick = %bestSpawn;
					
					return %spawnPoint;
				}
			}
			// Otherwise, pick one at random.
			else
			{
				%spawns = 0;
				%spawnList = "";
				
				for(%a = 0; %a < %ss.getCount(); %a++)
				{
					%spawn = %ss.getObject(%a);
					%spawnMini = getMiniGameFromObject(%spawn);
					
					if(zombieGameSetUpRight(%spawnMini) && zombieSameGame(%client, %spawn) && miniGameCanUse(%spawnMini.owner, %spawn))
					{
						if(!%spawns)
						{
							%spawnList = %spawn.getID();
						}
						else
						{
							%spawnList = %spawnList TAB %spawn.getID();
						}
						
						%spawns++;
					}
				}
				
				if(%spawns)
				{
					%spawnBrick = getField(%spawnList, getRandom(0, %spawns - 1));
					%spawnPoint = vectorSub(%spawnBrick.getWorldBoxCenter(), "0 0" SPC (%spawnBrick.getDatablock().brickSizeZ - 3) * 0.1) SPC getWords(%spawnBrick.getTransform(), 3, 6);
					
					%client.zombieSpawnBrick = %spawnBrick;
					
					return %spawnPoint;
				}
			}
		}
		else
		{
			error("CRITICAL ERROR: ZombieDirector.tankKeeper not found!");
		}
	}
	
	return "";
}

// ============================================================
// Section 2 : Events
// ============================================================
function ZombieEvents::addEvent(%so, %str)
{
	if(!%so.isEvent(%str))
	{
		%id = inputEvent_GetInputEventIdx(%str);
		
		if(%id >= 0)
		{
			%so.eventID[%id] = %str;
			%so.eventName[%str] = %id;
			
			return true;
		}
	}
	
	return false;
}

function ZombieEvents::isEvent(%so, %str)
{
	if(%so.eventID[%str] !$= "")
	{
		return true;
	}
	else if(%so.eventName[%str] !$= "")
	{
		return true;
	}
	else
	{
		return false;
	}
}

function ZombieEvents::removeEvent(%so, %str)
{
	if(%so.isEvent(%str))
	{
		if(%so.eventID[%str] !$= "")
		{
			%so.eventName[%so.eventID[%str]] = "";
			%so.eventID[%str] = "";
			return true;
		}
		else if(%so.eventName[%str] !$= "")
		{
			%so.eventID[%so.eventName[%str]] = "";
			%so.eventID[%str] = "";
			return true;
		}
	}
	
	return false;
}

function ZombieEvents::triggerEvent(%so, %str, %mini)
{
	%count = 0;
	
	if(isObject(%mini) && %so.isEvent(%str))
	{
		%keeper = %so.brickKeeper;
		
		for(%a = 0; %a < %keeper.getCount(); %a++)
		{
			%brick = %keeper.getObject(%a);
			
			if(%brick.numEvents > 0)
			{
				%bmini = getMiniGameFromObject(%brick);
				
				if(isObject(%bmini) && %bmini.getID() == %mini.getID())
				{
					for(%b = 0; %b < %brick.numEvents; %b++)
					{
						if(%brick.eventInput[%b] $= %str)
						{
							// We can activate 25 events a second witout hitting the ceiling.
							// This allows as many bricks as one can imagine to go off without problems.
							// The only real issue is, every 25 events takes a second.
							// If you have 100 events, it will take 4 full seconds for everything to finish.
							%count++;
							%timer = %count * 40;
							eval(%brick.getID() @ ".schedule(" @ %timer @ ", \"" @ %str @ "\");");
							
							// This is important, because a single brick can list one event 25 times.
							// We need to calculate for these, too.
							for(%c = %b; %c < %brick.numEvents; %c++)
							{
								if(%brick.eventInput[%b] $= %str)
								{
									%count++;
								}
							}
							
							break;
						}
					}
				}
			}
		}
	}
	else if(!isObject(%mini))
	{
		error("ZombieEvents::triggerEvent - %mini is not real.");
	}
	else
	{
		error("ZombieEvents::triggerEvent - " @ %str @ " is not a real event.");
	}
	
	return %count;
}

// ============================================================
// Section 3 : Friends
// ============================================================
// Removes all of %client's friends.
function ZombieFriends::clearFriends(%so, %client)
{
	%client.zombieFriends = 0;
	
	for(%a = 0; %a < 15; %a++)
	{
		%client.zombieFriend[%a] = "";
	}
	
	commandToClient(%client, 'ZAPT_HUD_Destroy');
}

// Gets a friend for %client to track.
function ZombieFriends::getFriend(%so, %client)
{
	if(isObject(%client))
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			// Okay, let me get my thoughts down on this process....
			// Initially, we are just searching for ANYBODY to fill the gap.
			// This desperate, light-search has the following criteria:
			// - Exists.
			// - Not already a friend.
			// But, we can find a better friend using stricter criteria.
			// To a zombie, a better friend is..
			// - A zombie.
			// To a human, we want.
			// - A human...
			// - That isn't rescued...
			// - Who has a player.
			// If we meet the "strict" criteria at any point, immediately return that value.
			
			%biffles = false;
			%friends = "";
			
			for(%a = 0; %a < %mini.numMembers; %a++)
			{
				%biffle = false;
				%friend = false;
				%member = %mini.member[%a].getID();
				
				// Not us, and not already a friend.
				if(!%member.zombieDoNotFriend && %member != %client.getID() && !%so.isFriend(%client, %member))
				{
					// If we have no %selected, just go ahead and make him our friend.
					if(!%biffles && !isObject(%selected))
					{
						// Carry on because we have an "OK" friend.
						%selected = %member;
					}
					
					// Stricter check. These are biffles.
					if(%client.isZombie && %member.isZombie)
					{
						%biffle = true;

					}
					else if(!%client.isZombie && isObject(%member.player) && !%member.isZombie && !%member.isRescued)
					{
						%biffle = true;
					}
					else if(!%biffles)
					{
						%friend = true;
					}
					
					if(%biffle && !%biffles)
					{
						%biffles = true;
						%friends = "";
					}
					
					if(%biffle || %friend)
					{
						if(getWordCount(%friends))
						{
							%friends = %friends SPC %member.getID();
						}
						else
						{
							%friends = %member.getID();
						}
					}
				}
			}
			
			if(getWordCount(%friends))
			{
				%selected = getWord(%friends, getRandom(0, getWordCount(%friends) - 1));
				return %selected;
			}
			else
			{
				return "";
			}
		}
		else
		{
			%so.clearFriends(%client);
		}
	}
}

// Gets %client's data from %friend's perspective.
function ZombieFriends::getFriendData(%so, %client, %friend)
{
	if(isObject(%client))
	{
		%data = "1" TAB %client.name;
		%getFull = false;
		%player = %client.player;
		
		// Obtain the zombie/reflective POV.
		// Omniscient.
		if(%friend.isZombie || %client.getID() == %friend.getID())
		{
			if(!isObject(%player) || %player.getState() $= "Dead")
			{
				if(%client.isDead && !%client.isZombie)
				{
					%data = %data TAB 0 TAB "human" TAB "dead";
				}
				else
				{
					%data = %data TAB 0 TAB "zombie" TAB "dead";
				}
			}
			else
			{
				%getFull = true;
			}
		}
		// Obtain the human POV.
		// Ignorant to zombie status.
		else
		{
			if(%client.isRescued)
			{
				%data = %data TAB 0 TAB "human" TAB "rescued";
			}
			else if(!isObject(%player) || %player.getState() $= "Dead" || %player.isZombie)
			{
				%data = %data TAB 0 TAB "human" TAB "dead";
			}
			else
			{
				%getFull = true;
			}
		}
		
		if(%getFull)
		{
			if(!%player.isZombie)
			{
				%health = 100;
				%health += mFloor(100 * ((%player.pseudoHealth + %player.tf2Overheal) / %player.getDatablock().maxDamage));
				%health -= mFloor(100 * %player.getDamagePercent());
				
				if(%health < 0) %health = 0;
				
				if(%player.zombieHelpless)
				{
					%status = "helpless";
				}
				else if(%player.zombieBeacon)
				{
					%status = "beacon";
				}
				else
				{
					%status = "";
				}
				
				%data = %data TAB %health TAB "human" TAB %status;
				
				for(%a = 0; %a < 6; %a++)
				{
					%tool = %player.tool[%a];
					
					if(isObject(%tool))
					{
						%tool = %tool.getID();
						
						if(%tool.zombieHUDIcon $= "")
						{
							if(!%tool.image.showBricks)
							{
								switch(%tool)
								{
									case (isObject(ZombieBileItem) ? ZombieBileItem.getID() : -1):
										%icon = "bile";
									case (isObject(ZombieMedpackItem) ? ZombieMedpackItem.getID() : -1):
										%icon = "medpack";
									case (isObject(ZombieMolotovItem) ? ZombieMolotovItem.getID() : -1):
										%icon = "molotov";
									case (isObject(ZombiePillsItem) ? ZombiePillsItem.getID() : -1):
										%icon = "pills";
									case (isObject(ZombiePipebombItem) ? ZombiePipebombItem.getID() : -1):
										%icon = "pipebomb";
									default:
										%icon = "gun";
								}
							}
							else
							{
								%icon = "gear";
							}
						}
						else
						{
							%icon = %tool.zombieHUDIcon;
						}
						
						%data = %data TAB %icon;
					}
					else
					{
						%data = %data TAB "";
					}
				}
				
				%client.zombieLastItemString = getFields(%data, 5, 10);
			}
			else
			{
				%health = 100;
				%health += mFloor(100 * ((%player.pseudoHealth + %player.tf2Overheal) / %player.getDatablock().maxDamage));
				%health -= mFloor(100 * %player.getDamagePercent());
				
				if(%health < 0) %health = 0;
				
				%icon = %player.getDatablock().zombieHUDIcon;
				
				if(%icon !$= "")
				{
					%status = %icon;
				}
				else
				{
					%status = "";
				}
				
				%data = %data TAB %health TAB "zombie" TAB %status;
			}
		}
		
		%client.zombieLastStatusString = %status;
		
		return %data;
	}
	else
	{
		return "0";
	}
}

// Finds people for %client to track.
function ZombieFriends::getFriends(%so, %client)
{
	if(isObject(%client) && %client.hasZAPT && %client.zombieHudCells > -1)
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			%friends = 0;
			
			for(%a = 0; %a <= %client.zombieHudCells; %a++)
			{
				if(%a > 0)
				{
					%buddy = %client.zombieBuddy[%a];
					%friend = %client.zombieFriend[%friends];
					%suggestion = %so.getFriend(%client);
					
					if(isObject(%suggestion) && !%so.isFriend(%client, %suggestion))
					{
						%client.zombieFriend[%friends] = %suggestion;
					}
					else
					{
						%client.zombieFriend[%friends] = "";
					}
				}
				else
				{
					%client.zombieFriend[%friends] = %client.getID();
				}
				
				if(isObject(%client.zombieFriend[%friends]))
				{
					%friends++;
				}
			}
			
			%client.zombieFriends = %friends;
		}
	}
}

// Tests if %testing is tracked by %client.
function ZombieFriends::isFriend(%so, %client, %testing)
{
	if(zombieSameGame(%client, %testing) && %client.zombieHudCells > -1)
	{
		for(%a = 0; %a <= %client.zombieHudCells; %a++)
		{
			%friend = %client.zombieFriend[%a];
			
			if(isObject(%friend) && %friend.getID() == %testing.getID())
			{
				return true;
			}
		}
	}
	
	return false;
}

// Sends %client's friends' data to %client.
function ZombieFriends::sendData(%so, %client)
{
	if(isObject(%client) && %client.hasZAPT && %client.zombieHudCells > -1)
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			for(%a = 0; %a <= %client.zombieHudCells; %a++)
			{
				%data = "";
				
				if(isObject(%client.zombieBuddy[%a]) && zombieSameGame(%client, %client.zombieBuddy[%a]))
				{
					%client.zombieFriend[%a] = %client.zombieBuddy[%a];
				}
				else if(!isObject(%client.zombieFriend[%a]))
				{
					%suggestion = %so.getFriend(%client);
					
					if(isObject(%suggestion))
					{
						%client.zombieFriend[%a] = %suggestion.getID();
					}
				}
				
				if(isObject(%client.zombieFriend[%a]))
				{
					%data = %so.getFriendData(%client.zombieFriend[%a], %client);
				}
				else
				{
					%data = "0";
				}
				
				%visible = getField(%data, 0);
				
				if(%visible)
				{
					for(%b = 1; %b < getFieldCount(%data); %b++)
					{
						%field = getField(%data, %b);
						
						switch(%b)
						{
							case 1: %name = %field;
							case 2: %hp = %field;
							case 3: eval("%canvas = '" @ %field @ "';");
							case 4: eval("%status = '" @ %field @ "';");
							default: eval("%icon[" @ (%b - 5) @ "] = '" @ %field @ "';");
						}
					}
					
					%client.zombieHUDOpen = true;
					commandToClient(%client, 'ZAPT_HUD_SET', %a, 1, %name, %hp, %canvas, %status, %icon0, %icon1, %icon2, %icon3, %icon4, %icon5);
				}
				else
				{
					%client.zombieHUDOpen = true;
					commandToClient(%client, 'ZAPT_HUD_SET', %a, 0);
				}
			}
		}
		else if(%client.zombieHUDOpen)
		{
			commandToClient(%client, 'ZAPT_HUD_DESTROY');
			%client.zombieHUDOpen = false;
		}
	}
}

// Sends %client's data to all whom have him friended.
function ZombieFriends::sendFriendData(%so, %client)
{
	if(isObject(%client))
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			for(%a = 0; %a < %mini.numMembers; %a++)
			{
				%member = %mini.member[%a];
				
				if(%member.zombieHudCells > -1 && %so.isFriend(%member, %client))
				{
					for(%b = 0; %b <= %member.zombieHudCells; %b++)
					{
						%friend = %member.zombieFriend[%b];
						
						if(isObject(%friend) && %friend.getID() == %client.getID())
						{
							%data = %so.getFriendData(%client, %member);
							
							%visible = getField(%data, 0);
							
							if(%visible)
							{
								for(%c = 1; %c < getFieldCount(%data); %c++)
								{
									%field = getField(%data, %c);
									
									switch(%c)
									{
										case 1: %name = %field;
										case 2: %hp = %field;
										case 3: eval("%canvas = '" @ %field @ "';");
										case 4: eval("%status = '" @ %field @ "';");
										default: eval("%icon[" @ (%c - 5) @ "] = '" @ %field @ "';");
									}
								}
								
								%member.zombieHUDOpen = true;
								commandToClient(%member, 'ZAPT_HUD_SET', %b, 1, %name, %hp, %canvas, %status, %icon0, %icon1, %icon2, %icon3, %icon4, %icon5);
							}
							else
							{
								%member.zombieHUDOpen = true;
								commandToClient(%member, 'ZAPT_HUD_SET', %b, 0);
							}
							
							break;
						}
					}
				}
			}
		}
	}
}

// Attempts to have others track %client.
function ZombieFriends::setFriend(%so, %client)
{
	if(isObject(%client))
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			for(%a = 0; %a < %mini.numMembers; %a++)
			{
				%member = %mini.member[%a];
				
				if(%client.getID() != %member.getID() && %member.zombieHudCells > -1 && !%so.isFriend(%member, %client))
				{
					%buddies = false;
					
					for(%b = 1; %b <= %member.zombieHudCells; %b++)
					{
						%buddy = %member.zombieBuddy[%b];
						
						if(isObject(%buddy) && %buddy.getID() == %client.getID())
						{
							%member.zombieFriend[%b] = %client.getID();
							%buddies = true;
							break;
						}
					}
					
					if(!%buddies && %member.zombieFriends < %member.zombieHudCells)
					{
						for(%b = 1; %b <= %member.zombieHudCells; %b++)
						{
							%friend = %member.zombieFriend[%b];
							
							if(!isObject(%friend))
							{
								%member.zombieFriend[%b] = %client.getID();
								%member.zombieFriends++;
								%so.sendData(%member);
								break;
							}
						}
					}
				}
			}
		}
	}
}

// Removes %client from all HUDs without consideration.
function ZombieFriends::removeFriend(%so, %client)
{
	if(isObject(%client))
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			%client.zombieDoNotFriend = true;
			
			for(%a = 0; %a < %mini.numMembers; %a++)
			{
				%member = %mini.member[%a];
				
				if(%client.getID() != %member.getID() && %member.zombieHudCells > 0 && %so.isFriend(%member, %client))
				{
					%chainMovement = false;
					
					for(%b = 1; %b <= %member.zombieHudCells; %b++)
					{
						%friend = %member.zombieFriend[%b];
						
						if(%chainMovement)
						{
							%member.zombieFriend[%b - 1] = %member.zombieFriend[%b];
							%member.zombieFriend[%b] = "";
							
							%member.zombieBuddy[%b - 1] = %member.zombieBuddy[%b];
							%member.zombieBuddy[%b] = "";
						}
						else
						{
							%buddy = %member.zombieBuddy[%b];
							
							if(isObject(%buddy) && %buddy.getID() == %client.getID())
							{
								%member.zombieBuddy[%b] = "";
							}
							
							if(!isObject(%friend) || %friend.getID() == %client.getID())
							{
								%member.zombieFriend[%b] = %so.getFriend(%member);
								
								if(!isObject(%member.zombieFriend[%b]))
								{
									%member.zombieFriends--;
									%chainMovement = true;
								}
							}
						}
					}
					
					%so.sendData(%member);
				}
			}
			
			%client.zombieDoNotFriend = false;
		}
	}
}

// Removes %client from all HUDs with consideration of placement and new friends.
function ZombieFriends::replaceFriend(%so, %client)
{
	if(isObject(%client))
	{
		%mini = getMiniGameFromObject(%client);
		
		if(zombieGameSetUpRight(%mini))
		{
			for(%a = 0; %a < %mini.numMembers; %a++)
			{
				%member = %mini.member[%a];
				
				if(%client.getID() != %member.getID() && %member.zombieHudCells > 0 && %so.isFriend(%member, %client) && !(%client.isZombie == %member.isZombie))
				{
					%chainMovement = false;
					
					for(%b = 1; %b <= %member.zombieHudCells; %b++)
					{
						%friend = %member.zombieFriend[%b];
						
						if(%chainMovement)
						{
							%member.zombieFriend[%b - 1] = %member.zombieFriend[%b];
							%member.zombieFriend[%b] = "";
							
							%member.zombieBuddy[%b - 1] = %member.zombieBuddy[%b];
							%member.zombieBuddy[%b] = "";
						}
						else
						{
							%buddy = %member.zombieBuddy[%b];
							
							if(isObject(%buddy) && %buddy.getID() != %client.getID())
							{
								%member.zombieFriend[%b] = %member.zombieBuddy[%b];
							}
							else
							{
								if(!isObject(%friend) || %friend.getID() == %client.getID())
								{
									%member.zombieFriend[%b] = %so.getFriend(%member);
									
									if(!isObject(%member.zombieFriend[%b]))
									{
										%member.zombieFriends--;
										%chainMovement = true;
									}
								}
							}
						}
					}
					
					%so.sendData(%member);
				}
			}
		}
	}
}

// ============================================================
// Section 4 : Mutations
// ============================================================
function ZombieMutations::addMutation(%so, %file)
{
	if(isFile(%file))
	{
		%clean = compile(%file);
		
		if(%clean)
		{
			exec(%file);
			
			if($ZAPT_Temp::Mutation::Name !$= "")
			{
				if(getWordCount($ZAPT_Temp::Mutation::Name) == 1)
				{
					if(%so.file[fileBase(%file)])
					{
						%inc = false;
						%slot = %so.file[fileBase(%file)] - 1;
						
						if(isObject(%so.mutation[%slot]))
						{
							%so.mutation[%slot].delete();
						}
					}
					else
					{
						%inc = true;
						%slot = %so.count;
					}
					
					%so.mutation[%slot] = new ScriptObject("ZombieMutation_" @ $ZAPT_Temp::Mutation::Name)
					{
						name	= $ZAPT_Temp::Mutation::Name;
						bots	= $ZAPT_Temp::Mutation::Bots ? true : false;
						respawn	= $ZAPT_Temp::Mutation::HumanRespawn ? true : false;
						safe	= $ZAPT_Temp::Mutation::Saferooms ? true : false;
						uiName	= $ZAPT_Temp::Mutation::uiName $= "" ? $ZAPT_Temp::Mutation::Name : $ZAPT_Temp::Mutation::uiName;
					};
					
					if(isObject(%so.mutation[%slot]))
					{
						%so.file[fileBase(%file)] = %slot + 1;
						
						if(%inc)
						{
							%so.count++;
						}
						
						return true;
					}
					else
					{
						error("::addMutation - Failed to create ScriptObject!");
					}
				}
				else
				{
					error("::addMutation - File provided name that is more than one word (" @ %file @ ").");
				}
			}
			else
			{
				error("::addMutation - File provided no name (" @ %file @ "). Try harder.");
			}
			
			deleteVariables("$ZAPT_Temp::Mutation::*");
		}
		else
		{
			error("::addMutation - File provided has errors (" @ %file @ "). Try harder.");
		}
	}
	else
	{
		error("::addMutation - Failed to provide real file.");
	}
	
	return false;
}

function ZombieMutations::mutationList(%so)
{
	for(%a = 0; isObject(%so.mutation[%a]); %a++)
	{
		if(!%a)
		{
			%list = %so.mutation[%a].uiName;
		}
		else
		{
			%list = %list TAB %so.mutation[%a].uiName;
		}
	}
	
	return %list;
}